package designer.admin;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
import com.ibm.mqe.mqbridge.*;
import java.util.*;
import designer.model.*;
public class RuntimeAdministrator extends UsefulQueueManager implements AdministrationInterface {
	private static final String LOCAL_QUEUE_TYPE = "MQeQueue";
	private static final String ADMIN_QUEUE_TYPE = "MQeAdminQueue";
	private static final String REMOTE_QUEUE_TYPE = "MQeRemoteQueue";
	private static final String HOMESERVER_QUEUE_TYPE = "MQeHomeServerQueue";
	private static final String STOREANDFORWARD_QUEUE_TYPE = "MQeStoreAndForwardQueue";
	private static final String BRIDGE_QUEUE_TYPE = "MQeMQBridgeQueue";
	
	protected static final int REAL_NAME = 0;
	protected static final int MAPPED_NAME = 1;
	// we only need a mapping heere from gui name to mqe name.....
	// but we really ought setup the GUI attribute names as constants in
	// the appropriate classes
	protected static final String[][] mappingData = new String[][] { 
		{MQeQueueManagerAdminMsg.QMgr_Description 		, AttributedModel.ATTRIBUTE_DESCRIPTION},
		{MQeQueueManagerAdminMsg.QMgr_Rules 			, QueueManager.ATTRIBUTE_QM_RULES},
		{MQeQueueManagerAdminMsg.QMgr_ChnlTimeout 		, QueueManager.ATTRIBUTE_QM_CHANNEL_TIMEOUT},
		{MQeQueueManagerAdminMsg.QMgr_ChnlAttrRules 	, QueueManager.ATTRIBUTE_QM_CHANNEL_ATTRIBUTE},
		{MQeQueueManagerAdminMsg.QMgr_QueueStore 		, QueueManager.ATTRIBUTE_QM_DEFAULT_QSTORE},
		
		{MQeQueueAdminMsg.Queue_Active					, Queue.ATTRIBUTE_QUEUE_ACTIVE},
		{MQeQueueAdminMsg.Queue_Description				, AttributedModel.ATTRIBUTE_DESCRIPTION},
		{MQeQueueAdminMsg.Queue_FileDesc 				, Queue.ATTRIBUTE_QUEUE_FILE_DESCRIPTOR},
		{MQeQueueAdminMsg.Queue_Cryptor					, Queue.ATTRIBUTE_QUEUE_CRYPTOR},
		{MQeQueueAdminMsg.Queue_Authenticator			, Queue.ATTRIBUTE_QUEUE_AUTHENTICATOR},
		{MQeQueueAdminMsg.Queue_Compressor				, Queue.ATTRIBUTE_QUEUE_COMPRESSOR}, 
		{MQeQueueAdminMsg.Queue_TargetRegistry			, Queue.ATTRIBUTE_QUEUE_TARGET_REGISTRY},
		{MQeQueueAdminMsg.Queue_AttrRule				, Queue.ATTRIBUTE_QUEUE_ATTRIBUTE_RULE},
		{MQeQueueAdminMsg.Queue_Rule					, Queue.ATTRIBUTE_QUEUE_RULE},
		{MQeQueueAdminMsg.Queue_MaxMsgSize				, Queue.ATTRIBUTE_QUEUE_MAX_MESSAGE_SIZE},
		{MQeQueueAdminMsg.Queue_Priority				, Queue.ATTRIBUTE_QUEUE_DEFAULT_PRIORITY},
		{MQeQueueAdminMsg.Queue_Mode 					, Queue.ATTRIBUTE_QUEUE_MODE},
		{MQeQueueAdminMsg.Queue_Expiry 					, Queue.ATTRIBUTE_QUEUE_EXPIRY},
		{MQeQueueAdminMsg.Queue_MaxQSize				, Queue.ATTRIBUTE_QUEUE_MAX_SIZE},
		{MQeQueueAdminMsg.Queue_CurrentSize				, Queue.ATTRIBUTE_QUEUE_SIZE},
		
		{MQeQueueAdminMsg.Queue_CreationDate 			, Queue.ATTRIBUTE_QUEUE_CREATION},
		
		{MQeAdminQueueAdminMsg.Queue_QTimerInterval		, AdminQueue.ATTRIBUTE_ADMINQ_TIMER_INTERVAL},
		
		{MQeRemoteQueueAdminMsg.Queue_Transporter		, AbstractRemoteQueue.ATTRIBUTE_REMQ_TRANSPORTER},
		{MQeRemoteQueueAdminMsg.Queue_TransporterXOR 	, AbstractRemoteQueue.ATTRIBUTE_REMQ_TRANSPORTER_XOR},
		{MQeRemoteQueueAdminMsg.Queue_CloseIdle			, AbstractRemoteQueue.ATTRIBUTE_REMQ_CLOSE_IDLE},
		
		{MQeConnectionAdminMsg.Con_Description 			, AttributedModel.ATTRIBUTE_DESCRIPTION},
		{MQeConnectionAdminMsg.Con_Channel 				, Connection.ATTRIBUTE_CON_CHANNEL},
		
		{MQeConnectionAdminMsg.Con_Adapter 			, "Name"},
		{MQeConnectionAdminMsg.Con_AdapterRulesData 	, "Rules"},
		{MQeConnectionAdminMsg.Con_AdapterAsciiParm 	, "Ascii Parameters"},
		{MQeConnectionAdminMsg.Con_AdapterEncodedParm 	, "Encoded Parameters"},
		{MQeConnectionAdminMsg.Con_AdapterOptions 		, "Options"},
		{MQeConnectionAdminMsg.Con_Aliases 				, "Aliases"},

		{MQeMQBridgeQueueAdminMsg.Queue_RemoteQName 		, "Remote Name"},
		{MQeMQBridgeQueueAdminMsg.Queue_Transformer 		, "Transformer"},
		{MQeMQBridgeQueueAdminMsg.Queue_MaxIdleTime 		, "Maximum Idle Time"},
		
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_RUN_STATE 						, AbstractBridgeObject.ATTRIBUTE_RUN_STATE},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_STARTUP_RULE_CLASS 				, AbstractBridgeObject.ATTRIBUTE_STARTUP_RULE_CLASS},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER 				, AbstractBridgeObject.ATTRIBUTE_DEFAULT_TRANSFORMER},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_HEARTBEAT_INTERVAL 				, AbstractBridgeObject.ATTRIBUTE_HEARTBEAT_INTERVAL},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_HOST_NAME 						, AbstractBridgeObject.ATTRIBUTE_HOST_NAME},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_PORT 								, AbstractBridgeObject.ATTRIBUTE_PORT},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_USER_ID 							, AbstractBridgeObject.ATTRIBUTE_USER_ID},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_PASSWORD 							, AbstractBridgeObject.ATTRIBUTE_PASSWORD},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_SECURITY_EXIT 					, AbstractBridgeObject.ATTRIBUTE_SECURITY_EXIT},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_RECEIVE_EXIT 						, AbstractBridgeObject.ATTRIBUTE_RECEIVE_EXIT},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_SEND_EXIT 						, AbstractBridgeObject.ATTRIBUTE_SEND_EXIT},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_CCSID 							, AbstractBridgeObject.ATTRIBUTE_CCSID},
//		{MQeCharacteristicLabels.MQE_FIELD_LABEL_MQ_BRIDGE_ADAPTER_CLASS 			, AbstractBridgeObject.ATTRIBUTE_ADAPTOR_CLASS},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_SYNC_Q_PURGER_RULES_CLASS 		, AbstractBridgeObject.ATTRIBUTE_SYNC_Q_PURGER_RULES_CLASS},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_MAX_CONNECTION_IDLE_TIME 			, AbstractBridgeObject.ATTRIBUTE_MAX_CONNECTION_IDLE_TIME},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_SYNC_Q_NAME 						, AbstractBridgeObject.ATTRIBUTE_SYNC_Q_NAME},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_SYNC_Q_PURGE_INTERVAL 			, AbstractBridgeObject.ATTRIBUTE_SYNC_Q_PURGE_INTERVAL},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_DEAD_LETTER_Q_NAME 				, AbstractBridgeObject.ATTRIBUTE_DEAD_LETTER_Q_NAME},
//		{MQeCharacteristicLabels.MQE_FIELD_LABEL_SECONDS_WAIT_FOR_MSG 				, AbstractBridgeObject.ATTRIBUTE_W},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_UNDELIVERED_MESSAGE_RULE_CLASS 	, AbstractBridgeObject.ATTRIBUTE_UNDELIVERED_MESSAGE_RULE_CLASS},
//		{MQeCharacteristicLabels.MQE_FIELD_LABEL_LISTENER_STATE_STORE_ADAPTER 		, AbstractBridgeObject.ATTRIBUTE_STA},
		{MQeCharacteristicLabels.MQE_FIELD_LABEL_FLOWS_PER_COMMIT 					, AbstractBridgeObject.ATTRIBUTE_FLOWS_PER_COMMIT},
//		{MQeCharacteristicLabels.MQE_FIELD_LABEL_TRANSFORMER 						, AbstractBridgeObject.ATTRIBUTE_FIE},
		
	};
	private Hashtable knownQueueManagers = new Hashtable();
	private boolean isStarted = false;
/**
 * RuntimeAdministrator constructor comment.
 */
public RuntimeAdministrator() {
	super();
}
private final void _digestAttributeData(AttributedModel model, MQeFields data) {
	try {
		Enumeration enum = data.fields();
		while (enum.hasMoreElements()) {
			String fieldName = (String) enum.nextElement();
			System.out.println(fieldName);
			String fieldValue = data.dumpToString("#2", fieldName);
			if ("null".equals(fieldValue)) {
				fieldValue = "";
			}
			String[] map = _getAttributeMappingDataFromRealName(fieldName);
			if (null != map) {
				String mappedName = map[MAPPED_NAME];
				//System.out.println(model.getShortDescription()
				//	+" setting attribute: "
				//	+mappedName
				//	+" ==> " + fieldValue);
				Attribute attr = model.getAttribute(mappedName);
				if (null != attr) {
					attr.setValue(fieldValue);
					attr.declareUpdated();
				}
			}

		}
	} catch (Exception e) {
		System.err.println("error digesting attributes info: " + e);
	}
}
private final boolean _digestConnectionData(QueueManager queueManager, MQeFields fieldsData) {
	boolean success = false;

	try {
		if (fieldsData.contains(MQeQueueManagerAdminMsg.QMgr_Connections)) {
			String connectionData[] = fieldsData.getAsciiArray(MQeQueueManagerAdminMsg.QMgr_Connections);
			int connectionCount = connectionData.length;
			Vector originalConnections = (Vector)queueManager.getConnections().clone();
			if (0 < connectionCount) {
				for (int i = 0; i < connectionCount; i++) {
					String connName = connectionData[i];
					Connection connection = queueManager.getConnection(connName);
					if (null == connection) {
						connection = new Connection(queueManager);
						connection.setName(connName);
						queueManager.addConnection(connection);
						queueManager.info(connection.getShortDescription());
					}
					
					originalConnections.remove(connection);		// we have validated tha this one truly exists
					// delete the queues array elements to save the attribute code from
					//trying to mapp the elements
					connection.declareCreated();
					fieldsData.delete(MQeQueueManagerAdminMsg.QMgr_Connections + ":" + i);
					enquire(connection);
				}
			}
			// remember to save any connections that need to be created.
			for (int i = 0; i < originalConnections.size(); i++){
				Connection con = (Connection)originalConnections.elementAt(i);
				queueManager.removeConnection(con);
			}

			Vector v = queueManager.getConnections();
			for (int i = 0; i < originalConnections.size(); i++){
				Connection con = (Connection)v.elementAt(i);
				con.refresh();
			}
		}
		fieldsData.delete(MQeQueueManagerAdminMsg.QMgr_Connections);
		success = true;
	} catch (Exception e) {
		System.err.println("error digesting QM connection info: " + e);
	}
	return success;
}
private final Queue _digestQueueData(QueueManager queueManager, MQeFields qData) throws Exception {
	Queue queue = null;
	String queueManagerName = qData.getAscii(MQeQueueManagerAdminMsg.QMgr_QueueQMgrName);
	String queueName = qData.getAscii(MQeQueueManagerAdminMsg.QMgr_QueueName);
	String queueType = qData.getAscii(MQeQueueManagerAdminMsg.QMgr_QueueType);
	queueType = MQe.abbreviate(queueType, 0);
	int index = queueType.lastIndexOf('.');
	queueType = queueType.substring(index + 1);
	queueManager.info(queueName+"@"+queueManagerName+" ("+queueType+")");

	queue =queueManager. getQueue(queueManagerName, queueName);
	if (null == queue) {
		if (STOREANDFORWARD_QUEUE_TYPE.equals(queueType)) {
			queue = new StoreAndForwardQueue(queueManager);
		} else if (HOMESERVER_QUEUE_TYPE.equals(queueType)) {
			queue = new HomeServerQueue(queueManager);
		} else if (ADMIN_QUEUE_TYPE.equals(queueType)) {
			queue = new AdminQueue(queueManager);
		} else if (LOCAL_QUEUE_TYPE.equals(queueType)) {
			queue = new LocalQueue(queueManager);
		} else if (REMOTE_QUEUE_TYPE.equals(queueType)) {
			queue = new RemoteQueue(queueManager);
		} else if (BRIDGE_QUEUE_TYPE.equals(queueType)) {
			queue = new BridgeQueue(queueManager);
		} else {
			queue = new Queue(queueManager);
		}
		queue.setName(queueManagerName, queueName);
		queueManager.addQueue(queue);
		enquire(queue);
	}
	return queue;
}
private final boolean _digestQueueManagerData(QueueManager queueManager, MQeFields fieldsData) {
	boolean success = false;
	
	try {
	//	setName(fieldsData.getAscii(MQeQueueManagerAdminMsg.Admin_Name));
		// remove fields after we have used them
		fieldsData.delete(MQeQueueManagerAdminMsg.Admin_Name);
		success = true;
	} catch (Exception e) {
		System.err.println("error digesting QM info: " + e);
	}
	return success;
}
private final boolean _digestQueuesData(QueueManager queueManager, MQeFields fieldsData) {
	boolean success = false;
	try {
		Vector originalQueues = (Vector)queueManager.getQueues().clone();
		if (fieldsData.contains(MQeQueueManagerAdminMsg.QMgr_Queues)) {
			MQeFields[] queueData = fieldsData.getFieldsArray(MQeQueueManagerAdminMsg.QMgr_Queues);
			int queueCount = queueData.length;
			for (int i = 0; i < queueCount; i++) {
				MQeFields qData = queueData[i];
				Queue queue = _digestQueueData(queueManager, qData);
				originalQueues.remove(queue);
				// delete the queues array elements to save the attribute code from
				//trying to mapp the elements
				fieldsData.delete(MQeQueueManagerAdminMsg.QMgr_Queues + ":" + i);
			}
			// delete fields we have finished with
			fieldsData.delete(MQeQueueManagerAdminMsg.QMgr_Queues);
		}
		for (int i = 0; i < originalQueues.size(); i++){
			Queue queue = (Queue)originalQueues.elementAt(i);
			queueManager.removeQueue(queue);
		}

		Vector queues = queueManager.getQueues();
		for (int i = 0; i < queues.size(); i++){
			Queue queue = (Queue)queues.elementAt(i);
			queue.refresh();
		}
		success = true;
	} catch (Exception e) {
		System.err.println("error digesting QM queue info: " + e);
	}
	return success;
}
private String[] _getAttributeMappingDataFromGuiName(String guiName) {
	String[] found = null;
	for (int i = 0; null == found && i < mappingData.length; i++) {
		if (mappingData[i][MAPPED_NAME].equals(guiName)) {
			found = mappingData[i];
		}
	}
	return found;
}
private String[] _getAttributeMappingDataFromRealName(String realName) {
	String[] found = null;
	for (int i = 0; null == found && i < mappingData.length; i++) {
		// using a real name coming in
		if (mappingData[i][REAL_NAME].equals(realName)) {
			found = mappingData[i];
		}
	}
	return found;
}
private final MQeAdminMsg _queryBridgeConnection(String queueManagerName, String bridgeName, String bridgeQMName, String connectionName) {
	MQeAdminMsg reply = null;
	try {
		MQeClientConnectionAdminMsg msg =  new MQeClientConnectionAdminMsg(bridgeName, bridgeQMName, connectionName,false);
		msg.setAction(MQeAdminMsg.Action_InquireAll);
		reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			_error("Failed to inquire bridgeConnection data on " + queueManagerName + " no reply received");
		} else if (!_isSuccess(reply)) {
			_error("Failed to inquire bridgeConnection data on" + queueManagerName);
			_error("Reason: " + _getReason(reply));
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while querying bridgeConnection data", e);
	}
	return reply;
}
private final MQeMQBridgesAdminMsg _queryBridgeList(String queueManagerName) {
	MQeMQBridgesAdminMsg reply = null;
	try {
		MQeMQBridgesAdminMsg msg = new MQeMQBridgesAdminMsg();
		msg.setAction(MQeAdminMsg.Action_InquireAll);
		reply = (MQeMQBridgesAdminMsg) processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			_error("Failed to inquire bridge data on " + queueManagerName + " no reply received");
		} else if (!_isSuccess(reply)) {
			_error("Failed to inquire bridge data on" + queueManagerName);
			_error("Reason: " + _getReason(reply));
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while querying bridge data", e);
	}
	return reply;
}
private final MQeAdminMsg _queryBridgeQueueManager(String queueManagerName, String bridgeName, String bridgeQMName) {
	MQeAdminMsg reply = null;
	try {
		MQeMQQMgrProxyAdminMsg msg = new MQeMQQMgrProxyAdminMsg();
		msg.putMQQMgrProxyName(bridgeQMName);
		msg.putBridgeName(bridgeName);
		msg.setAction(MQeAdminMsg.Action_InquireAll);
		reply = processRemoteAdminMsg(queueManagerName, msg, true);
		if (null == reply) {
			_error("Failed to inquire bridgeQM data on " + queueManagerName + " no reply received");
		} else if (!_isSuccess(reply)) {
			_error("Failed to inquire bridgeQM data on" + queueManagerName);
			_error("Reason: " + _getReason(reply));
		}
	} catch (NoClassDefFoundError e) {
	} catch (Exception e) {
		_comment("exception while querying bridgeQM data", e);
	}
	return reply;
}
/**
 * addQueueManager method comment.
 */
public void addQueueManager(StoreAndForwardQueue storeAndForwardQueue, String queueManagerName) {}
public final boolean canConnect() {
	return false;
}
public final boolean canDisconnect() {
	return false;
}
public final void close() {
	stop();
}
/**
 * commit any changes required on the given model element.
 */
public void commit(StateModel model) {
	boolean online = true;
	if (online) {
		QueueManager qm = model.getQueueManager();
		if (null != qm) {
			if (isRegistered(qm)) {
				// could start a thread here to perform async admin
				// reallyCommit could become the run method.
				reallyCommit(model);
			}
		}
	}
}
/**
 *
 */
public void create(AdminQueue queue) {
    String targetQueueManagerName = queue.getQueueManager().getName();
    String queueName = queue.getQueueName();
    if (createAdminQueueRemotely(targetQueueManagerName, queueName, null, true)) {
        queue.performCreate();
    }
}
/**
 * delete method comment.
 */
public void create(Bridge bridge) {
	String queueManagerName = bridge.getQueueManager().getName();
	String bridgeName = bridge.getName();
	if (createBridgeRemotely(queueManagerName, bridgeName, true)) {
		bridge.performCreate();
	}
}
/**
 * delete method comment.
 */
public void create(BridgeConnection bridgeConnection) {
	String queueManagerName = bridgeConnection.getQueueManager().getName();
	String bridgeName = bridgeConnection.getBridgeName();
	String bridgeQueueManagerName = bridgeConnection.getBridgeQueueManagerName();
	String bridgeConnectionName = bridgeConnection.getName();
	if (createBridgeConnectionRemotely(queueManagerName, bridgeName, bridgeQueueManagerName, bridgeConnectionName, true)) {
		bridgeConnection.performCreate();
	}
}
/**
 * delete method comment.
 */
public void create(BridgeListener bridgeListener) {
	String queueManagerName = bridgeListener.getQueueManager().getName();
	String bridgeName = bridgeListener.getBridgeName();
	String bridgeQueueManagerName = bridgeListener.getBridgeQueueManagerName();
	String bridgeConnectionName = bridgeListener.getBridgeConnectionName();
	String bridgeListenerName = bridgeListener.getName();
	if (createBridgeConnectionRemotely(queueManagerName, bridgeName, bridgeQueueManagerName, bridgeConnectionName, true)) {
		bridgeListener.performCreate();
	}
}
/**
 *
 */
public void create(BridgeQueue queue) {
/*
	String queueQueueManagerName = queue.getQueueQueueManagerName();
	String queueManagerName = queue.getQueueManagerName();
	String queueName = queue.getQueueName();
	if (createBridgeQueueRemotely(queueQueueManagerName, queueManagerName, queueName, null, true)) {
		queue.performCreate();
	}

*/
}
/**
 * delete method comment.
 */
public void create(BridgeQueueManager bridgeQueueManager) {
	String queueManagerName = bridgeQueueManager.getQueueManager().getName();
	String bridgeName = bridgeQueueManager.getBridgeName();
	String bridgeQueueManagerName = bridgeQueueManager.getName();
	if (createBridgeQueueManagerRemotely(queueManagerName, bridgeName, bridgeQueueManagerName, true)) {
		bridgeQueueManager.performCreate();
	}
}
/**
 * delete method comment.
 */
public void create(Connection connection) {
	String srcQmgr = connection.getQueueManager().getName();
	String dstQmgr = connection.getName();
	boolean created = createConnectionDefinitionRemotely(
						srcQmgr, 
						dstQmgr, 
						connection.getRoute(), 
						connection.getChannel(), 
						true); 
	if (created) {
		connection.performCreate();
	}
}
/**
 *
 */
public void create(HomeServerQueue queue) {
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueManagerName = queue.getQueueManagerName();
	String queueName = queue.getQueueName();
	if (createHomeServerQueueRemotely(targetQueueManagerName, queueManagerName, queueName, true)) {
		queue.performCreate();
	}
}
/**
 *
 */
public void create(LocalQueue queue) {
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueName = queue.getQueueName();
	if (createLocalQueueRemotely(targetQueueManagerName, queueName, null, true)) {
		queue.performCreate();
	}
}
/**
 * 
 */
public void create(Queue queue) {}
/**
 * removeAlias method comment.
 */
public void create(QueueAlias alias) {
	Queue queue = alias.getQueue();
	String queueManagerName = queue.getQueueManagerName();
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueName = queue.getQueueName();
	if (addQueueAliasRemotely(targetQueueManagerName, queueManagerName, queueName, alias.getName(), true)) {
		alias.performCreate();
	}
}
/**
 * removeAlias method comment.
 */
public void create(QueueManagerAlias alias) {
	String srcQmgr = alias.getName();
	String dstQmgr = alias.getValue();
	if (addAliasToConnectionDefinitionRemotely(
			alias.getQueueManager().getName(), 
			alias.getValue(), 
			alias.getName(), true)) {
		alias.performCreate();
	}
}
/**
 * removeQueueManager method comment.
 */
public void create(QueueManagerEntry queueManagerEntry) {
	StoreAndForwardQueue queue = (StoreAndForwardQueue)queueManagerEntry.getParent();
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueManagerName = queue.getQueueManagerName();
	String queueName = queue.getQueueName();
	if (addQueueManagerToStoreAndForwardQueueRemotely(targetQueueManagerName, queueManagerName, queueName, queueManagerEntry.getName(), true)) {
		queueManagerEntry.performCreate();
	}
}
/**
 *
 */
public void create(RemoteQueue queue) {
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueManagerName = queue.getQueueManagerName();
	String queueName = queue.getQueueName();
	if (createRemoteQueueRemotely(targetQueueManagerName, queueManagerName, queueName, null, true)) {
		queue.performCreate();
	}
}
/**
 * createSynchronousRemoteQueue method comment.
 */
public void create(StateModel model) {
    model.info("Create "+model.getShortDescription());
	QueueManager qm = model.getQueueManager();
    if (null != qm) {
        if (isRegistered(qm)) {
            if (model.isLocalQueue()) {
                create((LocalQueue) model);
            } else if (model.isBridgeQueue()) {
                create((BridgeQueue) model);
            } else if (model.isRemoteQueue()) {
                create((RemoteQueue) model);
            } else if (model.isStoreAndForwardQueue()) {
                create((StoreAndForwardQueue) model);
            } else if (model.isHomeServerQueue()) {
                create((HomeServerQueue) model);
            } else if (model.isQueueAlias()) {
                create((QueueAlias) model);
            } else if (model.isQueueManagerEntry()) {
                create((QueueManagerEntry) model);
            } else if (model.isConnection()) {
                create((Connection) model);
            } else if (model.isQueueManagerAlias()) {
                create((QueueManagerAlias) model);
            } else if (model.isBridge()) {
                create((Bridge) model);
            } else if (model.isBridgeQueueManager()) {
                create((BridgeQueueManager) model);
            } else if (model.isBridgeConnection()) {
                create((BridgeConnection) model);
            } else if (model.isBridgeListener()) {
                create((BridgeListener) model);
            }
        }
    }
}
/**
 *
 */
public void create(StoreAndForwardQueue queue) {
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueManagerName = queue.getQueueManagerName();
	String queueName = queue.getQueueName();
	if (createStoreAndForwardQueueRemotely(targetQueueManagerName, queueManagerName, queueName, true)) {
		queue.performCreate();
	}
}
/**
 * delete method comment.
 */
public void delete(Bridge bridge) {
	String queueManagerName = bridge.getQueueManager().getName();
	String bridgeName = bridge.getName();
	if (deleteBridgeRemotely(queueManagerName, bridgeName, true)) {
		bridge.performDelete();
	}
}
/**
 * delete method comment.
 */
public void delete(BridgeConnection bridgeConnection) {
	String queueManagerName = bridgeConnection.getQueueManager().getName();
	String bridgeName = bridgeConnection.getBridgeName();
	String bridgeQueueManagerName = bridgeConnection.getBridgeQueueManagerName();
	String bridgeConnectionName = bridgeConnection.getName();
	if (deleteBridgeConnectionRemotely(queueManagerName, bridgeName, bridgeQueueManagerName, bridgeConnectionName, true)) {
		bridgeConnection.performDelete();
	}
}
/**
 * delete method comment.
 */
public void delete(BridgeListener bridgeListener) {
	String queueManagerName = bridgeListener.getQueueManager().getName();
	String bridgeName = bridgeListener.getBridgeName();
	String bridgeQueueManagerName = bridgeListener.getBridgeQueueManagerName();
	String bridgeConnectionName = bridgeListener.getBridgeConnectionName();
	String bridgeListenerName = bridgeListener.getName();
	if (deleteBridgeConnectionRemotely(queueManagerName, bridgeName, bridgeQueueManagerName, bridgeConnectionName, true)) {
		bridgeListener.performDelete();
	}
}
/**
 * delete method comment.
 */
public void delete(BridgeQueueManager bridgeQueueManager) {
	String queueManagerName = bridgeQueueManager.getQueueManager().getName();
	String bridgeName = bridgeQueueManager.getBridgeName();
	String bridgeQueueManagerName = bridgeQueueManager.getName();
	if (deleteBridgeQueueManagerRemotely(queueManagerName, bridgeName, bridgeQueueManagerName, true)) {
		bridgeQueueManager.performDelete();
	}
}
/**
 * delete method comment.
 */
public void delete(Connection connection) {
	String srcQmgr = connection.getQueueManager().getName();
	String dstQmgr = connection.getName();
	if (deleteConnectionDefinitionRemotely(srcQmgr, dstQmgr, true)) {
		connection.performDelete();
	}
}
/**
 * removeAlias method comment.
 */
public void delete(QueueAlias alias) {
	Queue queue = alias.getQueue();
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueManagerName = queue.getQueueManagerName();
	String queueName = queue.getQueueName();
	if (removeQueueAliasRemotely(targetQueueManagerName, queueManagerName, queueName, alias.getName(), true)) {
		alias.performDelete();
	}
}
/**
 * removeAlias method comment.
 */
public void delete(QueueManagerAlias alias) {
	if (removeAliasFromConnectionDefinitionRemotely(
				alias.getQueueManager().getName(), 
				alias.getValue(), 
				alias.getName(), true)) {
		alias.performDelete();
	}
}
/**
 * removeQueueManager method comment.
 */
public void delete(QueueManagerEntry queueManagerEntry) {
	StoreAndForwardQueue queue = (StoreAndForwardQueue)queueManagerEntry.getParent();
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueManagerName = queue.getQueueManagerName();
	String queueName = queue.getQueueName();
	if (removeQueueManagerFromStoreAndForwardQueueRemotely(targetQueueManagerName, queueManagerName, queueName, queueManagerEntry.getName(), true)) {
		queueManagerEntry.performDelete();
	}
}
/**
 * delete method comment.
 */
public void delete(StateModel model) {
    model.info("Delete "+model.getShortDescription());
	QueueManager qm = model.getQueueManager();
    if (null != qm) {
        if (isRegistered(qm)) {
            if (model.isQueue()) {
                deleteQueue((Queue) model);
            } else if (model.isQueueAlias()) {
                delete((QueueAlias) model);
            } else if (model.isQueueManagerEntry()) {
                delete((QueueManagerEntry) model);
            } else if (model.isConnection()) {
                delete((Connection) model);
            } else if (model.isQueueManagerAlias()) {
                delete((QueueManagerAlias) model);
            } else if (model.isBridge()) {
                delete((Bridge) model);
            } else if (model.isBridgeQueueManager()) {
                delete((BridgeQueueManager) model);
            } else if (model.isBridgeConnection()) {
                delete((BridgeConnection) model);
            } else if (model.isBridgeListener()) {
                delete((BridgeListener) model);
            }
        }
    }
}
/**
 * deleteQueue method comment.
 */
public void deleteQueue(Queue queue) {
	String targetQueueManagerName = queue.getQueueManager().getName();
	String queueManagerName = queue.getQueueManagerName();
	String queueName = queue.getQueueName();
	if (deleteQueueRemotely(targetQueueManagerName, queueManagerName, queueName, true)) {
		queue.performDelete();
	}
}
/**
 * enquire method comment.
 */
public void enquire(Bridge bridge) {
	MQeFields data = inquireAllBridgeRemotely(
				bridge.getQueueManager().getName()
				, bridge.getName(), true);
	if (null != data) {
		
		_digestAttributeData(bridge, data);
		bridge.declareUpdated();
	}
}
/**
 * enquire method comment.
 */
public void enquire(BridgeConnection bridgeConnection) {
	MQeFields data = inquireAllBridgeConnectionRemotely(
				bridgeConnection.getQueueManager().getName()
				, bridgeConnection.getBridgeName()
				, bridgeConnection.getBridgeQueueManagerName()
				, bridgeConnection.getName(), true);
	if (null != data) {
		
		_digestAttributeData(bridgeConnection, data);
		bridgeConnection.declareUpdated();
	}
}
/**
 * enquire method comment.
 */
public void enquire(BridgeQueueManager bridgeQueueManager) {
	MQeFields data = inquireAllBridgeQueueManagerRemotely(
				bridgeQueueManager.getQueueManager().getName()
				, bridgeQueueManager.getBridgeName()
				, bridgeQueueManager.getName(), true);
	if (null != data) {
		
		_digestAttributeData(bridgeQueueManager, data);
		bridgeQueueManager.declareUpdated();
	}
}
/**
 * enquire method comment.
 */
public void enquire(Connection connection) {
	MQeConnectionAdminMsg msg = inquireAllConnectionDefinitionRemotely(connection.getQueueManager().getName(), connection.getName(), true);
	if (null != msg) {
		try {
	        Vector newAdapters = new Vector();
			MQeFields fieldsData = msg.getOutputFields();
			String channel = fieldsData.getAscii(msg.Con_Channel);
			MQeFields[] adaptors = fieldsData.getFieldsArray(msg.Con_Adapters);
			String adaptor = adaptors[0].getAscii(msg.Con_Adapter);
			String adaptorParam = adaptors[0].getAscii(msg.Con_AdapterAsciiParm);
			//connection.setName(connection.getName());
			connection.setChannel(channel);
			connection.setRoute(adaptor);
			String[] aliases = fieldsData.getAsciiArray(msg.Con_Aliases);
			for (int i = 0; i < aliases.length; i++){
				String alias = aliases[i];
				connection.getQueueManager().createAlias(alias, connection.getName());
			}
			
			// for some reason the 0 element always exists, even if there were no elements...
			//...so delete it specially
			if (fieldsData.contains(msg.Con_Adapters + ":0")) {
				fieldsData.delete(msg.Con_Adapters + ":0");
			}
			
			// remove used fields
			fieldsData.delete(msg.Con_Channel);
			fieldsData.delete(msg.Con_Adapters);
			fieldsData.delete(msg.Con_Aliases);

			_digestAttributeData(connection, fieldsData);
		} catch (Exception e) {
			System.out.println("error digesting connection msg: " + e);
		}
	}
}
/**
 * enquire method comment.
 */
public void enquire(Queue queue) {
    MQeQueueAdminMsg msg = inquireAllQueueRemotely(queue.getQueueManager().getName(), queue.getQueueManagerName(), queue.getName(), true);
    if (null != msg) {
        try {
            MQeFields qData = msg.getOutputFields();
            String[] alii = qData.getAsciiArray(msg.Queue_QAliasNameList);
            // take a safe copy of the aliases so we know which ones to delete later
            Vector oldAlii = (Vector)queue.getAliases().clone();
            for (int i = 0; i < alii.length; i++) {
                queue.info("QueueAlias: " + alii[i]);
                QueueAlias alias = queue.addAlias(alii[i]);
                alias.declareCreated();
                oldAlii.remove(alias);  // we have justified the existence of this one
                // remove array elements - saves the attribute code from
                // trying to map them
                qData.delete(msg.Queue_QAliasNameList + ":" + i);
            }
            // now remove any alii that were not validated by the incoming data
            for (int i = 0; i < oldAlii.size(); i++){
            	queue.removeAlias((QueueAlias)oldAlii.elementAt(i));
            }
            // for some reason the 0 element always exists, even if there were no elements...
            //...so delete it specially
            if (qData.contains(msg.Queue_QAliasNameList + ":0")) {
                qData.delete(msg.Queue_QAliasNameList + ":0");
            }
            qData.delete(msg.Queue_QAliasNameList);
			if (queue.isStoreAndForwardQueue()) {
                StoreAndForwardQueue safq = (StoreAndForwardQueue) queue;
                // take a clean copy of the original contents so we can remove any not
                // found in the enquire. (need a copy as the vector is the actual one)
                Vector originalQMEs = (Vector)safq.getQueueManagerEntries().clone();
                String[] qms = qData.getAsciiArray(MQeStoreAndForwardQueueAdminMsg.Queue_QMgrNameList);
                for (int i = 0; i < qms.length; i++) {
                    QueueManagerEntry qme = safq.addQueueManagerEntry(qms[i]);
                    qme.declareCreated();
                    originalQMEs.remove(qme);  // it should exist so remove it from list
                }
                qData.delete(MQeStoreAndForwardQueueAdminMsg.Queue_QMgrNameList);
                // now remove any elements left in the original list that have not been
                // validated by the incoming data
				for (int i = 0; i < originalQMEs.size(); i++){
					safq.removeQueueManagerEntry((QueueManagerEntry)originalQMEs.elementAt(i));
				}
            }
            _digestAttributeData(queue, qData);
        } catch (Exception e) {
            System.out.println("error refreshing queue " + e);
        }
        queue.declareChanged();
    }
}
/**
 * enquire method comment.
 */
public void enquire(QueueManager queueManager) {
	boolean success = false;
	queueManager.info("Refresh QueueManager: " + queueManager.getName());
	MQeQueueManagerAdminMsg msg = inquireAllQueueManagerRemotely(queueManager.getName(), true);
	if (null != msg) {
		MQeFields fieldsData = msg.getOutputFields();
		success = _digestQueueManagerData(queueManager, fieldsData);
		if (success) {
			success = _digestQueuesData(queueManager, fieldsData);
		}
		if (success) {
			success = _digestConnectionData(queueManager, fieldsData);
		}
		if (success) {
			_digestAttributeData(queueManager, fieldsData);
		}
		queueManager.info("query bridge");
		_queryBridgeList(queueManager.getName());
		queueManager.declareChanged();
	}
}
/**
 * enquire method comment.
 */
public void enquire(StateModel model) {
    model.info("Enquire "+model.getShortDescription());
	QueueManager qm = model.getQueueManager();
    if (null != qm) {
        if (isRegistered(qm)) {
            if (model.isQueue()) {
                enquire((Queue) model);
            } else if (model.isQueueManager()) {
                enquire((QueueManager) model);
            } else if (model.isQueueManagerEntry()) {
                enquire((QueueManagerEntry) model);
            } else if (model.isConnection()) {
                enquire((Connection) model);
            } else if (model.isBridge()) {
                enquire((Bridge) model);
            } else if (model.isBridgeQueueManager()) {
                enquire((BridgeQueueManager) model);
            } else if (model.isBridgeConnection()) {
                enquire((BridgeConnection) model);
            } else if (model.isBridgeListener()) {
                enquire((BridgeListener) model);
            }
        }
    }
}
public final boolean isNullAdministrator() {
	return false;
}
private boolean isRegistered(QueueManager queueManager) {
    start();
    boolean registered = false;
    String queueManagerName = queueManager.getName();
    KnownQueueManager knownQM = (KnownQueueManager) knownQueueManagers.get(queueManagerName);
    if (null == knownQM) {
        Enumeration listeners = queueManager.getListeners();
        if (listeners.hasMoreElements()) {
	        Listener listener = (Listener)listeners.nextElement();
            String route = listener.getAdapter()
                           +":"
                           + queueManager.getHost().getAddress()
                           +":"
                           +listener.getPort();
            String channel = "DefaultChannel" /*listener.getChannel()*/;
            knownQM = new KnownQueueManager(queueManagerName, route, channel);
            System.out.println("QueueManager " + queueManagerName + " now reconised");
            knownQueueManagers.put(queueManagerName, knownQM);
        }
    }
    if (null != knownQM) {
        if (knownQM.connected) {
            registered = true;
        } else {
            if (createConnectionDefinition(queueManagerName, knownQM.route, knownQM.channel, true)) {
                knownQM.connected = true;
                System.out.println("QueueManager " + queueManagerName + " now connected");
                registered = true;
            }
        }
    }
    return registered;
}
public final boolean isRepositoryAdministrator() {
	return false;
}
public final boolean isRuntimeAdministrator() {
	return true;
}
/**
 * commit any changes required on the given model element.
 */
public void reallyCommit(StateModel model) {
	System.out.println("Admin commit");
	switch (model.getState()) {
		case STATE_TOBECREATED :
			model.setState(STATE_CREATING);
			create(model);
			break;
		case STATE_CREATING :
			break;
		case STATE_TOBEUPDATED :
			model.setState(STATE_UPDATING);
			update(model);
			model.setState(STATE_CREATED);
			break;
		case STATE_UPDATING :
			model.setState(STATE_CREATED);
			break;
		case STATE_TOBEREFRESHED :
			model.setState(STATE_REFRESHING);
			enquire(model);
			model.setState(STATE_CREATED);
			break;
		case STATE_REFRESHING :
			model.setState(STATE_CREATED);
			break;
		case STATE_CREATED :
			break;
		case STATE_TOBEDELETED :
			model.setState(STATE_DELETING);
			delete(model);
			break;
		case STATE_DELETING :
			break;
		case STATE_DELETED :
			break;
		default :
			break;
	}
}
public final boolean	start() {
	if (!isStarted) {
		Hashtable args = new Hashtable();
		args.put(ARG_QUEUEMANAGER_NAME, "SnoopQM");
		args.put(ARG_QUEUEMANAGER_PORT, "NONE");
		args.put(ARG_CLEANUP, "true");
		isStarted = start(args);
		try {
			// let the queue manager catch up
			Thread.sleep(2000);
		} catch (InterruptedException e) {
		}
	}
	return isStarted;

}
public final boolean stop() {
	shutdown(true);
	isStarted = false;
	return true;
}
/**
 * update method comment.
 */
public void update(Bridge model) {
    //System.out.println("AdminUpdateQM:");
    Vector attrs = model.getAttributes();
    MQeFields data = new MQeFields();
    try {
        boolean changed = false;
        for (int i = 0; i < attrs.size(); i++) {
            Attribute attr = (Attribute) attrs.elementAt(i);
            String value = attr.getValue();
            if (null != value && value.length() > 0 && attr.isChanged()) {
                changed = true;
                //System.out.println("AdminUpdateQM: " + attr.getName() + " ==> " + value);
                String attrName = attr.getName();
                if (attrName.equals(AttributedModel.ATTRIBUTE_DESCRIPTION)) {
                    data.putUnicode(MQeCharacteristicLabels.MQE_FIELD_LABEL_DESCRIPTION, value);
                } else if (attrName.equals(Bridge.ATTRIBUTE_HEARTBEAT_INTERVAL)) {
                    data.putInt(MQeCharacteristicLabels.MQE_FIELD_LABEL_HEARTBEAT_INTERVAL, Integer.parseInt(value));
                } else if (attrName.equals(Bridge.ATTRIBUTE_STARTUP_RULE_CLASS)) {
                    data.putAscii(MQeCharacteristicLabels.MQE_FIELD_LABEL_STARTUP_RULE_CLASS, value);
                } else if (attrName.equals(Bridge.ATTRIBUTE_DEFAULT_TRANSFORMER)) {
                    data.putAscii(MQeCharacteristicLabels.MQE_FIELD_LABEL_DEFAULT_TRANSFORMER, value);
                }
                
            }
        }
        if (changed) {
	        boolean updated = updateBridgeRemotely(
	            				model.getQueueManager().getName(),
	            				model.getName(), 
	            				data, true);
            if (updated) {
                model.declareUpdated();
                for (int i = 0; i < attrs.size(); i++) {
                    Attribute attr = (Attribute) attrs.elementAt(i);
                    if (attr.isChanged()) {
                        attr.declareUpdated();
                    }
                }
            }
        } else {
	        model.declareUpdated(); // model needed no change - must be updated
        }
    } catch (Exception e) {
        System.out.println("Error building bridge update fields: " + e);
    }
}
/**
 * update method comment.
 */
public void update(BridgeConnection bridgeConnection) {}
/**
 * update method comment.
 */
public void update(BridgeQueueManager bridgeQueueManager) {}
/**
 * update method comment.
 */
public void update(Connection model) {
    //System.out.println("AdminUpdateQM:");
    Vector attrs = model.getAttributes();
    MQeFields data = new MQeFields();
    try {
        boolean changed = false;
        for (int i = 0; i < attrs.size(); i++) {
            Attribute attr = (Attribute) attrs.elementAt(i);
            String value = attr.getValue();
            if (null != value && value.length() > 0 && attr.isChanged()) {
                changed = true;
                //System.out.println("AdminUpdateQM: " + attr.getName() + " ==> " + value);
                String attrName = attr.getName();
                if (attrName.equals(AttributedModel.ATTRIBUTE_DESCRIPTION)) {
                    data.putUnicode(MQeConnectionAdminMsg.Con_Description, value);
                } else if (attrName.equals(Connection.ATTRIBUTE_CON_CHANNEL)) {
                    data.putAscii(MQeConnectionAdminMsg.Con_Channel, value);
                }
            }
        }
        if (changed) {
	        boolean updated = updateConnectionRemotely(
	            				model.getQueueManager().getName(),
	            				model.getName(), 
	            				data, true);
            if (updated) {
                model.declareUpdated();
                for (int i = 0; i < attrs.size(); i++) {
                    Attribute attr = (Attribute) attrs.elementAt(i);
                    if (attr.isChanged()) {
                        attr.declareUpdated();
                    }
                }
            }
        } else {
	        model.declareUpdated(); // model needed no change - must be updated
        }
    } catch (Exception e) {
        System.out.println("Error building connection update fields: " + e);
    }
}
/**
 * update method comment.
 */
public void update(Queue model) {
    //System.out.println("AdminUpdateQM:");
    Vector attrs = model.getAttributes();
    MQeFields data = new MQeFields();
    try {
        boolean changed = false;
        for (int i = 0; i < attrs.size(); i++) {
            Attribute attr = (Attribute) attrs.elementAt(i);
            String value = attr.getValue();
            if (null != value && value.length() > 0 && attr.isChanged()) {
                changed = true;
                //System.out.println("AdminUpdateQM: " + attr.getName() + " ==> " + value);
                String attrName = attr.getName();
                if (attrName.equals(AttributedModel.ATTRIBUTE_DESCRIPTION)) {
                    data.putUnicode(MQeQueueAdminMsg.Queue_Description, value);
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_FILE_DESCRIPTOR)) {
                    data.putAscii(MQeQueueAdminMsg.Queue_FileDesc, value);
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_CRYPTOR)) {
                    data.putAscii(MQeQueueAdminMsg.Queue_Cryptor, value);
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_AUTHENTICATOR)) {
                    data.putAscii(MQeQueueAdminMsg.Queue_Authenticator, value);
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_COMPRESSOR)) {
                    data.putAscii(MQeQueueAdminMsg.Queue_Compressor, value);
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_TARGET_REGISTRY)) {
                    data.putByte(MQeQueueAdminMsg.Queue_TargetRegistry, Byte.parseByte(value));
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_ATTRIBUTE_RULE)) {
                    data.putAscii(MQeQueueAdminMsg.Queue_AttrRule, value);
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_RULE)) {
                    data.putAscii(MQeQueueAdminMsg.Queue_Rule, value);
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_MAX_MESSAGE_SIZE)) {
                    data.putInt(MQeQueueAdminMsg.Queue_MaxMsgSize , Integer.parseInt(value));
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_DEFAULT_PRIORITY)) {
                    data.putByte(MQeQueueAdminMsg.Queue_Priority , Byte.parseByte(value));
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_MODE)) {
                    data.putByte(MQeQueueAdminMsg.Queue_Mode, Byte.parseByte(value));
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_EXPIRY)) {
                    data.putLong(MQeQueueAdminMsg.Queue_Expiry , Long.parseLong(value));
                } else if (attrName.equals(Queue.ATTRIBUTE_QUEUE_MAX_SIZE)) {
                    data.putInt(MQeQueueAdminMsg.Queue_MaxQSize , Integer.parseInt(value));
                } else if (attrName.equals(AdminQueue.ATTRIBUTE_ADMINQ_TIMER_INTERVAL)) {
                    data.putLong(MQeAdminQueueAdminMsg.Queue_QTimerInterval , Long.parseLong(value));
                } else if (attrName.equals(AbstractRemoteQueue.ATTRIBUTE_REMQ_TRANSPORTER)) {
                    data.putAscii(MQeRemoteQueueAdminMsg.Queue_Transporter , value);
                } else if (attrName.equals(AbstractRemoteQueue.ATTRIBUTE_REMQ_TRANSPORTER_XOR)) {
                    data.putAscii(MQeRemoteQueueAdminMsg.Queue_TransporterXOR , value);
                } else if (attrName.equals(AbstractRemoteQueue.ATTRIBUTE_REMQ_CLOSE_IDLE)) {
                    data.putAscii(MQeRemoteQueueAdminMsg.Queue_CloseIdle , value);
                }
            }
        }
        if (changed) {
	        boolean updated = updateQueueRemotely(
	            				model.getQueueManager().getName(),
	            				model.getQueueQueueManagerName(),
	            				model.getName(), 
	            				data, true);
            if (updated) {
                model.declareUpdated();
                for (int i = 0; i < attrs.size(); i++) {
                    Attribute attr = (Attribute) attrs.elementAt(i);
                    if (attr.isChanged()) {
                        attr.declareUpdated();
                    }
                }
            }
        } else {
	        model.declareUpdated(); // model needed no change - must be updated
        }
    } catch (Exception e) {
        System.out.println("Error building Queue update fields: " + e);
    }
}
/**
 * update method comment.
 */
public void update(QueueManager model) {
    //System.out.println("AdminUpdateQM:");
    Vector attrs = model.getAttributes();
    MQeFields data = new MQeFields();
    try {
        boolean changed = false;
        for (int i = 0; i < attrs.size(); i++) {
            Attribute attr = (Attribute) attrs.elementAt(i);
            String value = attr.getValue();
            if (null != value && value.length() > 0 && attr.isChanged()) {
                changed = true;
                //System.out.println("AdminUpdateQM: " + attr.getName() + " ==> " + value);
                String attrName = attr.getName();
                if (attrName.equals(AttributedModel.ATTRIBUTE_DESCRIPTION)) {
                    data.putUnicode(MQeQueueManagerAdminMsg.QMgr_Description, value);
                } else if (attrName.equals(QueueManager.ATTRIBUTE_QM_CHANNEL_TIMEOUT)) {
                    data.putLong(MQeQueueManagerAdminMsg.QMgr_ChnlTimeout, Long.parseLong(value));
                } else if (attrName.equals(QueueManager.ATTRIBUTE_QM_CHANNEL_ATTRIBUTE)) {
                    data.putAscii(MQeQueueManagerAdminMsg.QMgr_ChnlAttrRules, value);
                } else if (attrName.equals(QueueManager.ATTRIBUTE_QM_DEFAULT_QSTORE)) {
                    data.putAscii(MQeQueueManagerAdminMsg.QMgr_QueueStore, value);
                }
            }
        }
        if (changed) {
            if (updateQueueManagerRemotely(model.getName(), data, true)) {
                model.declareUpdated();
                for (int i = 0; i < attrs.size(); i++) {
                    Attribute attr = (Attribute) attrs.elementAt(i);
                    if (attr.isChanged()) {
                        attr.declareUpdated();
                    }
                }
            }
        } else {
	        model.declareUpdated(); // model needed no change - must be updated
        }
    } catch (Exception e) {
        System.out.println("Error building QM update fields: " + e);
    }
}
/**
 * update method comment.
 */
public void update(StateModel model) {
    model.info("Update "+model.getShortDescription());
	QueueManager qm = model.getQueueManager();
    if (null != qm) {
        if (isRegistered(qm)) {
            if (model.isQueue()) {
                update((Queue) model);
            } else if (model.isConnection()) {
                update((Connection) model);
            } else if (model.isBridge()) {
                update((Bridge) model);
            } else if (model.isQueueManager()) {
                update((QueueManager) model);
            } else if (model.isBridgeQueueManager()) {
                update((BridgeQueueManager) model);
            } else if (model.isBridgeConnection()) {
                update((BridgeConnection) model);
            } else if (model.isBridgeListener()) {
                //update((BridgeListener) model);
            }
        }
    }
}
}
