package designer.admin;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import com.ibm.mqe.*;
import com.ibm.mqe.administration.*;
public class WrappedQueueManager extends LocalQueueManager {
/**
 * UsefulQueueManager constructor comment.
 */
public WrappedQueueManager() {
	super();
}
/**
 * access to the standard API on the local QueueManager
 */
public final MQeEnumeration browseMessages(String queueManagerName, String queueName, MQeFields Filter, MQeAttribute attr, boolean justUID, boolean showErrors) {
	MQeEnumeration enum = null;
	try {
		enum = browseMessages(queueManagerName, queueName, Filter, attr, justUID);
	} catch (Exception e) {
		if (showErrors) {
			_error("browsing messages", e);
		}
	}
	return enum;
}
/**
 * access to the standard API on the local QueueManager
 */
public final MQeEnumeration browseMessagesAndLock(
	String queueManagerName,
	String queueName,
	MQeFields Filter,
	MQeAttribute attr,
	long confirmid,
	boolean justUID,
	boolean showErrors) {
	MQeEnumeration enum = null;
	try {
		enum = browseMessagesAndLock(queueManagerName, queueName, Filter, attr, confirmid, justUID);
	} catch (Exception e) {
		if (showErrors) {
			_error("browsing messages with lock", e);
		}
	}
	return enum;
}
/**
 * access to the standard API on the local QueueManager
 */
public final boolean confirm(String queueManagerName, String queueName, long confirmID, boolean showErrors) {
	boolean success = false;
	try {
		confirm(queueManagerName, queueName, confirmID);
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("confirm", e);
		}
	}
	return success;
}
/**
 * access to the standard API on the local QueueManager
 */
protected final boolean confirmGetMessage(String queueManagerName, String queueName, MQeFields filter, boolean showErrors) {
	boolean success = false;
	try {
		confirmGetMessage(queueManagerName, queueName, filter);
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("confirming put message to "+queueName+"@"+queueManagerName, e);
		}
	}
	return success;
}
/**
 * access to the standard API on the local QueueManager
 */
protected final boolean confirmPutMessage(String queueManagerName, String queueName, MQeFields filter, boolean showErrors) {
	boolean success = false;
	try {
		confirmPutMessage(queueManagerName, queueName, filter);
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("confirming put message to "+queueName+"@"+queueManagerName, e);
		}
	}
	return success;
}
/**
 * access to the standard API on the local QueueManager
 */
public final boolean deleteMessage(String queueManagerName, String queueName, MQeFields filter, boolean showErrors) {
	boolean success = false;
	try {
		deleteMessage(queueManagerName, queueName, filter);
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("deleting message", e);
		}
	}
	return success;
}
/**
 * Utility method for getting message
 * wraps published API in an (optional) error reporter
 */
protected final MQeMsgObject getMessage(String queueManagerName, String queueName, MQeFields filter, MQeAttribute attr, long confirmID, boolean showErrors) {
	MQeMsgObject msg = null;
	try {
		msg = getMessage(queueManagerName, queueName, filter, attr, confirmID);
	} catch (Exception e) {
		if (showErrors) {
			_error("getting message", e);
		}
	}
	return msg;
}
/**
 * access to the standard API on the local QueueManager
 */
protected final boolean putMessage(String queueManagerName, String queueName, MQeMsgObject msg, MQeAttribute attribute, long confirmID, boolean showErrors) {
	boolean success = false;
	try {
		putMessage(queueManagerName, queueName, msg, attribute, confirmID);
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("putting message to "+queueName+"@"+queueManagerName, e);
		}
	}
	return success;
}
/**
 * removes a subscription to MQe Message Events
 *
 *
 * @exception MQeException Except_QMgr_NotActive
 * @exception MQeException Except_QMgr_QdoesNotExist
 *
 * access to the standard API on the local QueueManager
 */
public boolean removeMessageListener(MQeMessageListenerInterface objref, String queueName, MQeFields filter, boolean showErrors) {
	boolean success = false;
	try {
		removeMessageListener(objref, queueName, filter);
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("removing message listener", e);
		}
	}
	return success;
}
public final boolean triggerTransmission(boolean showErrors) {
	boolean success = false;
	try {
		triggerTransmission();
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("triggering transmission", e);
		}
	}
	return success;
}
/**
 * access to the standard API on the local QueueManager
 */
public final boolean undo(String queueManagerName, String queueName, long confirmID, boolean showErrors) {
	boolean success = false;
	try {
		undo(queueManagerName, queueName, confirmID);
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("undo", e);
		}
	}
	return success;
}
/**
 * access to the standard API on the local QueueManager
 */
public final boolean unlockMessage(String queueManagerName, String queueName, MQeFields filter, boolean showErrors) {
	boolean success = false;
	try {
		unlockMessage(queueManagerName, queueName, filter);
		success = true;
	} catch (Exception e) {
		if (showErrors) {
			_error("unlocking message", e);
		}
	}
	return success;
}
/**
   * Wait for message - waits for a mesaage to
   * arrive on the reply queue
   * return message, or null if timed out
   */
/**
 * access to the standard API on the local QueueManager
 */
protected final MQeMsgObject waitForMessage(
	String queueManagerName,
	String queueName,
	MQeFields match,
	MQeAttribute attribute,
	long confirmID,
	int milliseconds,
	boolean showErrors) {
	MQeMsgObject msg = null;
	try {
		msg = waitForMessage(queueManagerName, queueName, match, attribute, confirmID, milliseconds);
	} catch (Exception e) {
		if (showErrors) {
			_error("waiting for message on " +queueName+"@"+queueManagerName, e);
		}
	}
	return msg;
}
}
