package designer.gui;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import designer.model.*;
//import java.awt.*;
import java.awt.Frame;
import java.awt.Choice;
import java.awt.Dialog;
import java.awt.Panel;
import java.awt.GridLayout;
import java.awt.BorderLayout;
import java.awt.Label;
import java.util.Vector;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;

/**
 * Insert the type's description here.
 * Creation date: (11/04/2002 07:48:03)
 * @author: 
 */
public class QueueSelectDialogue extends glyph.dialogues.RendererDialogue implements ItemListener {
	private final Choice 	hostChoice 		= new Choice();
	private final Choice 	qmChoice	 	= new Choice();
	private final Choice 	qChoice 		= new Choice();
	private Vector queues;
	//private Queue selectedQueue = null;
/**
 * DetailsDialogue constructor comment.
 * @param parent java.awt.Frame
 * @param title java.lang.String
 */
public QueueSelectDialogue(Dialog parent, String title, Vector queues) {
	super(parent, title);
	_setup(queues);
	pack();
}
/**
 * DetailsDialogue constructor comment.
 * @param parent java.awt.Frame
 * @param title java.lang.String
 */
public QueueSelectDialogue(Frame parent, String title, Vector queues) {
	super(parent, title);
	_setup(queues);
	pack();
}
private void _selectQueue(String hostName, String qmName, String qName) {
	LogicalQueue selectedQueue = null;
    for (int i = 0;null == selectedQueue && i < queues.size(); i++) {
        LogicalQueue q = (LogicalQueue) queues.elementAt(i);
        QueueManager queueManager = q.getQueueManager();
        Host host = queueManager.getHost();
        if (host.getName().equals(hostName) && queueManager.getName().equals(qmName) && q.getDisplayName().equals(qName)) {
            selectedQueue = q;
        }
    }
}
private final void _setup(Vector queues) {
	this.queues = queues;
	Panel panel = new Panel(new GridLayout(3,2));

		// 	set up the visual bits
	hostChoice.addItemListener(this);
	hostChoice.setEnabled(true);
	
	qmChoice.addItemListener(this);
	qmChoice.removeAll();

	qChoice.addItemListener(this);
	qChoice.removeAll();

	_setupHostList();

	// arrange the visual bits
	panel.add(new Label("Host:"));
	panel.add(hostChoice);

	panel.add(new Label("QueueManager:"));
	panel.add(qmChoice);
	
	panel.add(new Label("Queue/Alias:"));
	panel.add(qChoice);

	add(BorderLayout.CENTER, panel);
	add(BorderLayout.SOUTH, createOkCancelPanel());
}
private void _setupHostList() {
	Vector added = new Vector();
	hostChoice.removeAll();
	for (int i = 0; i < queues.size(); i++){
		LogicalQueue q = (LogicalQueue) queues.elementAt(i);
        QueueManager queueManager = q.getQueueManager();
        Host host = queueManager.getHost();
		String hostName = host.getName();
		if (!added.contains(hostName)) {
			hostChoice.add(hostName);
			added.addElement(hostName);
		}
	}
	if (added.size() > 0) {
		String str = (String)added.elementAt(0);
		hostChoice.select(str);
		_setupQueueManagerList(str);
	}
}
private void _setupQueueList(String hostName, String qmName) {
    Vector added = new Vector();
    qChoice.removeAll();
    qChoice.setEnabled(true);

    for (int i = 0; i < queues.size(); i++) {
        LogicalQueue q = (LogicalQueue) queues.elementAt(i);
        QueueManager queueManager = q.getQueueManager();
        Host host = queueManager.getHost();
        if (host.getName().equals(hostName) && queueManager.getName().equals(qmName)) {
            String qName = q.getDisplayName();
            if (!added.contains(qName)) {
                qChoice.add(qName);
                added.addElement(qName);
            }
            /*
            java.util.Enumeration aliases = q.getAliases().elements();
            while (aliases.hasMoreElements()) {
                QueueAlias qAlias = (QueueAlias) aliases.nextElement();
                String qaName = qAlias.getName();
                if (!added.contains(qaName)) {
                    qChoice.add(qaName);
                    added.addElement(qaName);
                }
            }
            */
        }
    }
    if (qChoice.getItemCount() > 0) {
        qChoice.select(0);
    }
}
private void _setupQueueManagerList(String hostName) {
    Vector added = new Vector();
    qmChoice.removeAll();
    qmChoice.setEnabled(true);
    qChoice.setEnabled(false);
    qChoice.removeAll();
    
	for (int i = 0; i < queues.size(); i++){
		LogicalQueue q = (LogicalQueue) queues.elementAt(i);
        QueueManager queueManager = q.getQueueManager();
        Host host = queueManager.getHost();
        if (host.getName().equals(hostName)) {
            String qmName = queueManager.getName();
            if (!added.contains(qmName)) {
                qmChoice.add(qmName);
                added.addElement(qmName);
            }
        }
    }
	if (added.size() > 0) {
		String str = (String)added.elementAt(0);
		qmChoice.select(str);
		_setupQueueList(hostName, str);
    }
}
	/**
	 * Invoked when an item has been selected or deselected.
	 * The code written for this method performs the operations
	 * that need to occur when an item is selected (or deselected).
	 */
public void itemStateChanged(ItemEvent e) {
	if (e.getSource() == hostChoice) {
		String hostName = hostChoice.getSelectedItem();
    	if (null != hostName) {
			_setupQueueManagerList(hostName);
    	}
	} else if (e.getSource() == qmChoice) {
		String hostName = hostChoice.getSelectedItem();
		String qmName = qmChoice.getSelectedItem();
    	if (null != hostName && null != qmName) {
			_setupQueueList(hostName, qmName);
    	}
	} else if (e.getSource() == qChoice) {
		String hostName = hostChoice.getSelectedItem();
		String qmName = qmChoice.getSelectedItem();
    	String qName = qChoice.getSelectedItem();
    	if (null != hostName && null != qmName && null != qName) {
			_selectQueue(hostName, qmName, qName);
    	}
	}
}
public final String selectedQueueManagerName() {
	return qmChoice.getSelectedItem();
}
public final String selectedQueueName() {
	return qChoice.getSelectedItem();
}
}
