package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
/**
 * One step in the linked list representing a message route 
 * resolution.  Superclass for all step types
 *
 */
class AbstractStep {
    Model model;
    /** the name of the queue manager to which the message is put */
    protected String putQMName;
    /** the name of the queue to which the message is put */
    protected String putQName;
    private ResolutionLine line;
    AbstractStep nextStep;
protected AbstractStep(Model model) {
    this.model = model;
}
void calculateLines() {
    if (null != line) {
        line.detach();
        line = null;
    }
    if (model.isHomeServerQueue()) {
	    System.out.println();
    }
    // find next step that is detached, or is last step
    if (null != nextStep) {
        AbstractStep lineTo = nextStep;
        boolean keepLooking = true;
        while (keepLooking) {
            if (lineTo.model.isAttached()) {
                AbstractStep oneAfter = lineTo.nextStep;
                if (null == oneAfter) {
                    // line to is the last one, so we can stop looking
                    keepLooking = false;
                } else {
                    // lineTo is attached, and has a following step, so
                    // examne the following step
                    lineTo = oneAfter;
                }
            } else {
                // lineTo is detached, we can stop looking
                keepLooking = false;
            }
        }

        if (null != lineTo) {
            String label = lineTo.putQName + "@" + lineTo.putQMName;
            line = new ResolutionLine(this.model, lineTo.model, label, false, lineTo.isPullStep());
            lineTo.calculateLines();
        }
    }
}
void detach() {
	if (null != line) {
		line.detach();
	}
	if (null != nextStep) {
		nextStep.detach();
	}
}
public final String getDescription() {
	StringBuffer buf = new StringBuffer();
	getDescription(buf);
	return buf.toString();
}
protected void getDescription(StringBuffer buf) {
	if (null != nextStep) {
		buf.append(nextStep.model.getDescription());
		buf.append("[");
		buf.append(nextStep.putQName);
		buf.append("@");
		buf.append(nextStep.putQMName);
		buf.append("]\n");

		nextStep.getDescription(buf);
	}
}
boolean hasVisibleLines() {
	return (null != nextStep)
			&& (null != line)
				&& (line.isVisible());
}
boolean isPullStep() {
	return false;
}
/**
 * create a trailing step, or return the current one if appropriate
 */
protected void setNextStep(AbstractStep step) {
    if (null != line) {
		line.detach();
		line = null;
	}
	this.nextStep = step;
	//if (null != nextStep) {
		//String label = nextStep.putQName+"@"+ nextStep.putQMName;
		//System.out.println(nextStep.model.getDescription() + "[" + label+ "]");
	//}
}
protected void setTarget(String putQMName, String putQName) {
    this.putQMName = putQMName;
	this.putQName = putQName;
}
void setVisible(boolean set) {
	if (null != line) {
		line.setVisible(set);
	}
	if (null != nextStep) {
		nextStep.setVisible(set);
	}
}
}
