package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

/**
 * This type is a central repository/factory for administrators.  Why create
 * many when we can create only one of each?
 * Creation date: (09/10/2001 07:07:00)
 * @author: 
 */
public class AdministratorFactory {
	// name of class to null admin
	private static final String NULL_ADMIN_CLASS_NAME = "designer.admin.NullAdministrator";
	// name of class to runtime admin
	private static final String RUNTIME_ADMIN_CLASS_NAME = "designer.admin.RuntimeAdministrator";
	// name of class to registry admin
	private static final String REGISTRY_ADMIN_CLASS_NAME = "designer.admin.RegistryAdministrator";
	
	private static AdministrationInterface nullAdministrator;
	private static AdministrationInterface runtimeAdministrator;
	private static AdministrationInterface registryAdministrator;
/**
 * AdministratorFactory constructor comment.
 */
public AdministratorFactory() {
	super();
}
public static final void close() {
	if (null != nullAdministrator) {
		nullAdministrator.close();
	}
	if (null != runtimeAdministrator) {
		runtimeAdministrator.close();
	}
	if (null != registryAdministrator) {
		registryAdministrator.close();
	}
}
public static final AdministrationInterface getRegistryAdministrator() {
	if (null == registryAdministrator) {
		//nullAdministrator = new NullAdministrator();
	}
	return registryAdministrator;
}
public static final AdministrationInterface getRuntimeAdministrator() {
	//System.out.println("getting runtime admin");
	if (null == runtimeAdministrator) {
		try {
			//System.out.println("class name is "+RUNTIME_ADMIN_CLASS_NAME);
			Class cl = Class.forName(RUNTIME_ADMIN_CLASS_NAME);
			runtimeAdministrator = (AdministrationInterface) cl.newInstance();
		} catch (Exception e) {
			//e.printStackTrace();
		} catch (NoClassDefFoundError e) {
			//e.printStackTrace();
		}
	}
	//System.out.println("getting runtime admin = "+runtimeAdministrator);
	return runtimeAdministrator;
}
}
