package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;
import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;

/**
 * Insert the type's description here.
 * Creation date: (10/09/2001 07:19:51)
 * @author: 
 */
public abstract class AttributedModel extends StateModel {
	public static final String ATTRIBUTE_DESCRIPTION = "Description";
	private static final String MI_EDIT_ATTRIBUTES = "Edit Attributes";
	protected static final String MI_REFRESH = "Refresh";
	
	private final Vector attributes = new Vector();
/**
 * Insert the method's description here.
 * Creation date: (14/11/2001 08:01:12)
 */
public AttributedModel() {
	super();
	createAttribute(ATTRIBUTE_DESCRIPTION, "", true);
}
public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	if (MI_EDIT_ATTRIBUTES.equals(cmd)) {
		int nAttr = attributes.size();
		String data[][] = new String[nAttr][2];
		for (int i = 0; i < nAttr; i++){
			Attribute attr = (Attribute)attributes.elementAt(i);
			data[i][0] = attr.getName();
			data[i][1] = attr.getValue();
		}
		boolean changed = false;		// detect if any attribute has changed
        if (getView().getStringData(getShortDescription()+ " Attributes", data)) {
			for (int i = 0; i < nAttr; i++){
				Attribute attr = (Attribute)attributes.elementAt(i);
				// if the value has changed and the attribute is editable, then
				//set the new value, and note that we need an update.
				if (attr.isEditable()) {
					String newValue = data[i][1];
					if (!attr.getValue().equals(newValue)) {
						System.out.println("Changing attribute: "+attr.getValue()+" ==> " + newValue);
						attr.setValue(newValue);
						changed = true;
					}
				}
			}
        }
    	if (changed) {
	    	setRequiresUpdate();
	    	System.out.println("committing");
	    	commit();
    	} 
	} else {
		super.actionPerformed(e);
	}
}
/**
 * called by instances of subclasses to create the attributes required
 * by the instance.
 */
protected final void createAttribute(String name, String value, boolean editable) {
	Attribute attr = getAttribute(name);
	if (null == attr) {
		attr = new Attribute(this, name, value);
		attributes.addElement(attr);
	} else {
		attr.setValue(value);
	}
   attr.setEditable(editable);
}
public final Attribute getAttribute(String attributeName) {
	Attribute attr = null;
	for (int i = 0; null == attr && i < attributes.size(); i++){
		Attribute a = (Attribute)attributes.elementAt(i);
		if (a.getName().equals(attributeName)) {
			attr = a;
		} 
	}
	
	return attr;
}
public final Vector getAttributes() {
	return attributes;
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem = new MenuItem(MI_EDIT_ATTRIBUTES);
    menuItem.addActionListener(this);
    popup.add(menuItem);

	return popup;
}
/**
 * called by external entities to set an attribute value,  If the attribute
 * does not exit then the call is ignored.
 */
public void setAttribute(String name, String value) {
	Attribute attr = getAttribute(name);
	if (null != attr) {
		attr.setValue(value);
	}
}
}
