package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;
import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Color;

/**
 * Insert the type's description here.
 * Creation date: (12/09/2001 07:11:45)
 * @author: 
 */
public class Bridge extends AbstractBridgeObject {
	private static final String MI_CREATE_QM = "Add Queue Manager";
	private static final Color BRIDGE_COLOR = new Color(255,120,255);
	
	private QueueManager queueManager;
	private Vector bridgeQueueManagers = new Vector();
/**
 * Insert the method's description here.
 * Creation date: (12/09/2001 07:30:33)
 * @param bridgeList snoop.model.BridgeList
 */
public Bridge(QueueManager queueManager) {
	super();
	this.queueManager = queueManager;
	createAttribute(ATTRIBUTE_HEARTBEAT_INTERVAL,"0",true);
	createAttribute(ATTRIBUTE_STARTUP_RULE_CLASS,"",true);
	createAttribute(ATTRIBUTE_DEFAULT_TRANSFORMER,"",true);
	createAttribute(ATTRIBUTE_RUN_STATE,"",false);
}
public void actionPerformed(java.awt.event.ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_CREATE_QM.equals(cmd)) {
        String data[][] = new String[][] {{"QM Name", "MQ_QueueManager"},
	        {"Host Name", "remotehost"}};
        if (getView().getStringData("Add MQ QueueManager", data)) {
            BridgeQueueManager bqm = newBridgeQueueManager(data[0][1]);
            bqm.setHostName(data[1][1]);
        }
    } else {
        super.actionPerformed(e);
    }
}
/**
 * delete this node if not already deleted
 */
public void delete() {
    for (int i = 0; i < bridgeQueueManagers.size(); i++){
    	((AbstractBridgeObject)bridgeQueueManagers.elementAt(i)).delete();
    }
    super.delete();
}
protected final AdministrationInterface getAdministrator() {
	return queueManager.getAdministrator();
}
/**
 * return the BridgeConnection resolving to the given name.
 * The name will resolve to an alias for the connection
 */
public final BridgeConnection getBridgeConnection(String name) {
	BridgeConnection found = null;
	Enumeration enum = bridgeQueueManagers.elements();
	while (null == found && enum.hasMoreElements()) {
		BridgeQueueManager bqm =(BridgeQueueManager)enum.nextElement();
		found = bqm.getBridgeConnection(name);
	}
	return found;
}
public final BridgeQueueManager getBridgeQueueManager(String name) {
	BridgeQueueManager found = null;
	Enumeration enum = bridgeQueueManagers.elements();
	while (null == found && enum.hasMoreElements()) {
		BridgeQueueManager bridgeQM = (BridgeQueueManager)enum.nextElement();
		if (bridgeQM.getName().equals(name)) {
			found = bridgeQM;
		}
	}
	return found;
}
public final Enumeration getBridgeQueueManagers() {
	return bridgeQueueManagers.elements();
}
public String getDescription() {
	return getShortDescription() + " on " + getQueueManager().getDescription();
}
/**
 * Search among the recievers children for a bridge listener pulling
 * messages from the specified transmit queue.
 * return null if not found
 */
BridgeListener getListenerFor(MQXmitQueue txQueue) {
    BridgeListener bridgeListener = null;
    for (int i = 0; null == bridgeListener && i < bridgeQueueManagers.size(); i++) {
        bridgeListener =((BridgeQueueManager) bridgeQueueManagers.elementAt(i)).getListenerFor(txQueue);
    }
    return bridgeListener;
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem;

	menuItem = new MenuItem(MI_CREATE_QM);
	menuItem.addActionListener(this);
	popup.add(menuItem);

	return popup;
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return queueManager.getModel();
}
public Model getParent() {
	return queueManager;
}
public final QueueManager getQueueManager() {
	return queueManager;
}
public String getType() {
	return "Bridge";
}
public boolean isBridge() {
	return true;
}
public final BridgeQueueManager newBridgeQueueManager(String name) {
    BridgeQueueManager bridgeQM = getBridgeQueueManager(name);
    if (null == bridgeQM) {
        bridgeQM = new BridgeQueueManager(this);
        bridgeQueueManagers.addElement(bridgeQM);
        bridgeQM.setName(name);
        addNode(bridgeQM);
    }
    return bridgeQM;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	for (int i = 0; i < bridgeQueueManagers.size(); i++){
		((BridgeQueueManager)bridgeQueueManagers.elementAt(i)).notifyChange();
	}
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {
	queueManager.removeBridge(this);
	declareDeleted();
	delete();
}
public final void removeQueueManager(BridgeQueueManager queueManager) {
	bridgeQueueManagers.removeElement(queueManager);
	declareChanged();
}
}
