package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;
import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Color;
/**
 * Insert the type's description here.
 * Creation date: (12/09/2001 07:12:20)
 * @author: 
 */
public class BridgeQueueManager extends AbstractBridgeObject {
	public static final String ATTRIBUTE_HOST = "HostName";
	private static final String MI_CREATE_Connection = "Add Connection";
	private static final Color BRIDGE_QUEUEMANAGER_COLOR = new Color(255,140,255);
    private static final String MI_SHOWDEFAULT = "Show Default Connection";
    private static final String MI_HIDEDEFAULT = "Hide Default Connection";
    private boolean showDefault = true;
	private Bridge bridge;
	private Vector bridgeConnections = new Vector();
    private DefaultRelationship defaultRelationship = null;
/**
 * Insert the method's description here.
 * Creation date: (12/09/2001 07:31:56)
 */
public BridgeQueueManager(Bridge bridge) {
	super();
	this.bridge = bridge;
	createAttribute("Startup Rules","",false);
	createAttribute("Transformer","",false);
	createAttribute(ATTRIBUTE_HOST, "", false);
}
public void actionPerformed(java.awt.event.ActionEvent e) {
	String cmd = e.getActionCommand();
	if (MI_SHOWDEFAULT.equals(cmd)) {
        showDefault = true;
        declareChanged();
    } else if (MI_HIDEDEFAULT.equals(cmd)) {
        showDefault = false;
        declareChanged();
    } else if (MI_CREATE_Connection.equals(cmd)) {
		String data[][] = new String[][] {{"Connection Name", "Connection"},};
		if (getView().getStringData("Add Connection", data)) {
			newBridgeConnection(data[0][1]);
		}
	} else {
		super.actionPerformed(e);
	}
}
/**
 * delete this node if not already deleted
 */
public void delete() {
    for (int i = 0; i < bridgeConnections.size(); i++){
    	((AbstractBridgeObject)bridgeConnections.elementAt(i)).delete();
    }
    super.delete();
}
protected final AdministrationInterface getAdministrator() {
	return bridge.getAdministrator();
}
public final BridgeConnection getBridgeConnection(String name) {
	BridgeConnection found = null;
	Enumeration enum = bridgeConnections.elements();
	while (null == found && enum.hasMoreElements()) {
		BridgeConnection connection = (BridgeConnection)enum.nextElement();
		if (connection.getName().equals(name)) {
			found = connection;
		}
	}
	return found;
}
public final Enumeration getBridgeConnections() {
	return bridgeConnections.elements();
}
public final String getBridgeName() {
	return bridge.getName();
}
public String getDescription() {
	return getShortDescription() + " on " + bridge.getDescription();
}
String getHostName() {
	return getAttribute(ATTRIBUTE_HOST).getValue();
}
/**
 * Search among the recievers children for a bridge listener pulling
 * messages from the specified transmit queue.
 * return null if not found
 */
BridgeListener getListenerFor(MQXmitQueue txQueue) {
    BridgeListener bridgeListener = null;
    if (txQueue.getOwner() == getMQQueueManager()) {
        // only bother if we resolve to the txQueues owner
        for (int i = 0; null == bridgeListener && i < bridgeConnections.size(); i++) {
            bridgeListener = ((BridgeConnection) bridgeConnections.elementAt(i)).getListenerFor(txQueue);
        }
    }
    return bridgeListener;
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem;
	menuItem = new MenuItem(MI_CREATE_Connection);
	menuItem.addActionListener(this);
	popup.add(menuItem);
	if (showDefault) {
		menuItem = new MenuItem(MI_HIDEDEFAULT);
	} else {
		menuItem = new MenuItem(MI_SHOWDEFAULT);
	}
	menuItem.addActionListener(this);
	popup.add(menuItem);

	return popup;
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return bridge.getModel();
}
MQQueueManager getMQQueueManager() {
	MQQueueManager mqqm = null;
	Host host = getModel().getHostByAddress(getHostName());
	if (null != host) {
		mqqm = host.getMQQueueManager(getName());
	}
	return mqqm;
}
public Model getParent() {
	return bridge;
}
public final QueueManager getQueueManager() {
	return bridge.getQueueManager();
}
public String getType() {
	return "MQ Queue Manager Proxy";
}
public boolean isBridgeQueueManager() {
	return true;
}
public final BridgeConnection newBridgeConnection(String name) {
    BridgeConnection connection = getBridgeConnection(name);
    if (null == connection) {
        connection = new BridgeConnection(this);
        bridgeConnections.addElement(connection);
        connection.setName(name);
        addNode(connection);
    }
    return connection;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	for (int i = 0; i < bridgeConnections.size(); i++){
		((BridgeConnection)bridgeConnections.elementAt(i)).notifyChange();
	}
	if (null != defaultRelationship) {
		// if there is a relationship, check to see if it is still valid
		// and if not then cut it free
		if (!defaultRelationship.isAttached()) {
			defaultRelationship = null;
		}
	}
	if (null == defaultRelationship && !isToBeDeleted()) {
		MQQueueManager mqqm = getMQQueueManager();
		if (null != mqqm) {
			defaultRelationship = new DefaultRelationship(this, mqqm, "refers to");
		}
	}
    if (null != defaultRelationship) {
	    defaultRelationship.setVisible(showDefault && ShowDefaultRelationships);
    }
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {
	bridge.removeQueueManager(this);
	declareDeleted();
	delete();
}
public final void removeConnection(BridgeConnection connection) {
	bridgeConnections.removeElement(connection);
	declareChanged();
}
void setHostName(String hostname) {
	setAttribute(ATTRIBUTE_HOST, hostname);
}
}
