package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import glyph.Node;
import java.awt.PopupMenu;
import java.awt.Color;
import java.util.Vector;
/**
 * Insert the type's description here.
 * Creation date: (08/02/2002 17:20:26)
 * @author: 
 */
class Category extends Node {
	private String name = "unnamed";
	private Node parentNode;
	private final Vector children = new Vector();
/**
 * Insert the method's description here.
 * Creation date: (10/02/2002 09:43:10)
 */
public Category(Node parent, String name) {
	this.parentNode = parent;
	parentNode.addNode(this);
	this.name = name;
}
final Vector _getChildren() {
	return children;
}
public final void addNode(Node child) {
	children.addElement(child);
	super.addNode(child);
}
/**
 * return true if the receivers representation can be
 * detached from its parent by dragging.  Subclasses should
 * override to prevent detach.
 */
protected boolean canDetach() {
	return false;
}
/**
 * Note that this may fail if the children are not state models
 */
public final void commitAll() {
	for (int i = 0; i < children.size(); i++){
		StateModel model = (StateModel)children.elementAt(i);
		model.commitAll(); 
	}
}
/**
 * delete this node if not already deleted
 */
public void delete() {
	for (int i = 0; i < children.size(); i++){
		((Node)children.elementAt(i)).delete();
	}
	super.delete();
}
/**
 * Some elements do not need to be drawn if they have no
 * visible attached children.  Returning true forces
 * the display of 'empty' nodes.
 * Subclasses should override to suppress the draw by
 * returning false; 
 */
protected boolean drawIfNoChildren() {
	return false;
}
/**
 * reply the body - the bit of text above the line in the box
 * reply null (or empy array) to show no body.
 * Each string in the array will apear on a separate line
 */
protected java.lang.String[] getBody() {
	return new String[]{name};
}
public final Model getChild(String name) {
	for (int i = 0; i < children.size(); i++){
		Model ch = (Model)children.elementAt(i);
		if (ch.getName().equals(name)) {
			return ch;
		} 
	}
	return null;
}
public String getDescription() {return name + " on "+parentNode.getDescription();}
/**
 * return  a textual description of the receiver
 */
public String getExplanation() {
	return "This is a category called "+name+" on " + parentNode.getShortDescription() +".";
}
/**
 * reply the name
 */
protected String getHeader() {
	return null;
}
public String getShortDescription() {
	 
	return name + " on "+parentNode.getShortDescription();
}
public final boolean isCategory() {
	return true;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	for (int i = 0; i < children.size(); i++){
		((Model)children.elementAt(i)).notifyChange();
	}
}
final void removeChild(Node child) {
	children.removeElement(child);
	if (null != child) {
		child.delete();
	}
}
public String toString() {return name +" on "+parentNode.getDescription();}
}
