package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Color;

import java.awt.event.ActionEvent;

/**
 * Insert the type's description here.
 * Creation date: (20/08/2001 14:02:38)
 * @author: 
 */
public class Connection extends AttributedModel {
	//public static final String ATTRIBUTE_CON_MANAGER = "Connection Manager";
	//public static final String ATTRIBUTE_CON_MANAGER_RULES = "Connection Manager Rule";
	public static final String ATTRIBUTE_CON_CHANNEL = "Channel";
	public static final String ATTRIBUTE_CON_ROUTE = "Route";

	private static final String MI_SNOOP = "Explore ";

	private static final Color CONNECTION_COLOUR = new Color(200,255,255);
	private static final String MI_SINGLELINE = "Single Line Name";
	private static final String MI_MULTILINE = "Multiline Name";
	private static final String MI_SHOWDEFAULT = "Show Default Connection";
	private static final String MI_HIDEDEFAULT = "Hide Default Connection";
	private boolean showDefault = true;
	private boolean multiline = false;		

	static final String MI_EDITPURPOSE = "Edit Purpose";		// package-private as used by message route
	static final String MI_SHOWPURPOSE = "Show Purpose";		// package-private as used by message route
	static final String MI_SHOWNAME = "Show Name";				// package-private as used by message route
	private boolean showPurpose = false;
	private String purpose;

	private QueueManager owner;
	private boolean isViaConnection = false;
	private DefaultRelationship defaultRelationship = null;
	private Link link = null;
/**
 * Queue constructor comment.
 */
public Connection(QueueManager owner) {
	super();
	this.owner = owner;
	createAttribute(ATTRIBUTE_CON_CHANNEL, "DefaultChannel", true);
	//createAttribute(ATTRIBUTE_CON_MANAGER, "", true);
	//createAttribute(ATTRIBUTE_CON_MANAGER_RULES, "", true);
	createAttribute(ATTRIBUTE_CON_ROUTE, "", true);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or unreachable
 *
 * Most subclasses will be quite content with the creation method used here 
 * - the step representing the receiver will be returned for the benefit of the
 * caller.  If the receiver wishes to propagate the resolution it should
 * provide an alternate implementation of this method.
 */
AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = previousStep;
    if (!isAttached()) {
	    step = super._resolveMessageRoute(previousStep, qmName, qName);
    }
    if (isViaConnection()) {
        Connection connection = owner.getConnection(getRoute());
        if (null != connection) {
            connection._resolveMessageRoute(step, qmName, qName);
        }
    } else {
        Host host = resolveHost();
        if (null != host) {
	        if (IncludeHostResolution) {
		        step = host._resolveMessageRoute(step, qmName, qName);
	        }
        	Listener listener = null;
            listener = host.getListenerByPort(getPort());
            if (null != listener) {
                listener._resolveMessageRoute(step, qmName, qName);
            }
        }
    }
    return step;
}
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (cmd.startsWith(MI_SNOOP)) {
        // removed during rewrite
        exploreQueueManager();
        //if (null != queueManager) {
        //	if (queueManagerGlyph.getHostGlyph().reReadModel()) {
        //		getView().repaint();
        //	}
        //}
    } else if (MI_SINGLELINE.equals(cmd)) {
        multiline = false;
        declareChanged();
    } else if (MI_MULTILINE.equals(cmd)) {
        multiline = true;
        declareChanged();
    } else if (MI_EDITPURPOSE.equals(cmd)) {
	    String str = "";
	    if (null != purpose) {
		    str = purpose;
	    }
	    String data[][] = new String[][] { {"Purpose", str }, };
        if (getView().getStringData("Edit Purpose", data)) {
	        purpose = data[0][1];
            declareChanged();
        }
    } else if (MI_SHOWPURPOSE.equals(cmd)) {
	    showPurpose = true;
	    declareChanged();
    } else if (MI_SHOWNAME.equals(cmd)) {
	    showPurpose = false;
	    declareChanged();
    } else if (MI_SHOWDEFAULT.equals(cmd)) {
        showDefault = true;
        declareChanged();
    } else if (MI_HIDEDEFAULT.equals(cmd)) {
        showDefault = false;
        declareChanged();
    } else if (MI_REFRESH.equals(cmd)) {
        refresh();
    } else {
        super.actionPerformed(e);
    }
}
boolean digest(String name) {
	boolean success = false;
	setName(name);
	refresh();
	return true;
}
public final QueueManager exploreQueueManager() {

	QueueManager queueManager = null;
	if (isViaConnection) {
		// find the real connection that this uses to next hop, and ensure that we have explored the
		// connection.
		Connection logical = this;
		while (null != logical && logical.isViaConnection()) {
			String route = logical.getRoute();
			logical = owner.getConnection(route);
		}
		if (null != logical) {
			// we have found a resolution so ensure we know the actual connection (as opposed to an alias)
		    Connection realConnection = logical.resolve();
		    if (null != realConnection) {
			    // now use the real connection to explore the queue manager we go via
			    queueManager = realConnection.exploreQueueManager();
		    }
		    if (null != queueManager) {
			    // we have explored the intermediate queue manager, so now find the connection that
			    // takes our route onward.  Note that this call could, in theory, be to a via connection
			    // and the whole shebang could continue.....
			    logical =queueManager.getConnection(getName());
			    if (null != logical) {
				    realConnection = logical.resolve();
				    if (null != realConnection) {
					    realConnection.exploreQueueManager();
				    }
			    }
		    }
		}
	} else {
		String adaptor = getRoute();
		String useAdaptor = getRoute();
		String hostName = getHostName();
		if ("localhost".equalsIgnoreCase(hostName) || "127.0.0.1".equals(hostName)) {
			hostName = getQueueManager().getHost().getAddress();
		}

		Host host = getQueueManager().getOrCreateHost(hostName);
		if (null != host) {
			queueManager = host.getOrCreateQueueManager(getName());
			queueManager.createListener("DefaultListener", getPort(), getAdapter());
		}
	}
	return queueManager;
}
public final String getAdapter() {
	String route = getAttribute(ATTRIBUTE_CON_ROUTE).getValue();
	String adapter = null;
	int index = route.indexOf(':');
	if (-1 < index) {
		adapter = route.substring(0, index);
	}
	return adapter;
}
protected final AdministrationInterface getAdministrator() {
	return owner.getAdministrator();
}
public String[] getBody() {
	if (multiline && null != getRoute()) {
		return new String[]{getName(), "("+getRoute()+")"};
	} else {
		return super.getBody();
	}
}
public final String getChannel() {
	return getAttribute(ATTRIBUTE_CON_CHANNEL).getValue();
}
public String getDescription() {
	return getShortDescription()
		+ " defined on " + owner.getDescription();
}
/**
 * Get the string representing the receiver in a gui
 */
public String getDisplayName() {
	return getName() + "("+getRoute()+")";
}
/**
 * return  an explanation of the receiver
 */
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" defines the means by which ");
	buf.append(owner.getShortDescription());
	buf.append(" can connect to a queue manager called ");
	buf.append(getName());
	buf.append(".");
	if (isViaConnection()) {
		buf.append(" Since this is a via connection, all communications with queue manager ");
		buf.append(getName());
		buf.append(" are sent using the connection data in the connection definition names ");
		buf.append(getRoute());
		buf.append(".");
	} else {
		buf.append(" Connection will be attempted using the following:\n\thost:     ");
		buf.append(getHostName());
		buf.append("\n\tport:     ");
		buf.append(getPort());
		buf.append("\n\tprotocol: ");
		buf.append(getProtocol());
	}
	return buf.toString();
}
public String getHostName() {
	String route = getRoute();
	String hostName = null;
	if (null != route) {
		int firstColon = route.indexOf(':');
		if (-1 < firstColon) {
			int secondColon = route.indexOf(':', firstColon + 1);
			if (-1 < secondColon) {
				hostName = route.substring(firstColon + 1, secondColon);
			}
		}
	}
	return hostName;
}
public static String GetHostNameFromRoute(String route) {
	String hostName = null;
	if (null != route) {
		int firstColon = route.indexOf(':');
		if (-1 < firstColon) {
			int secondColon = route.indexOf(':', firstColon + 1);
			if (-1 < secondColon) {
				hostName = route.substring(firstColon + 1, secondColon);
			}
		}
	}
	return hostName;
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem;
	if (null != getRoute()) {
		menuItem = new MenuItem(MI_SNOOP + getName());
		menuItem.addActionListener(this);
		popup.add(menuItem);
	}
    menuItem = new MenuItem(MI_EDITPURPOSE);
    menuItem.addActionListener(this);
    popup.add(menuItem);
    
    if (showPurpose) {
        menuItem = new MenuItem(MI_SHOWNAME);
        menuItem.addActionListener(this);
        popup.add(menuItem);
    } else { // we are showing name
        if (null != purpose) {
            menuItem = new MenuItem(MI_SHOWPURPOSE);
            menuItem.addActionListener(this);
            popup.add(menuItem);
        }
        if (multiline) {
            menuItem = new MenuItem(MI_SINGLELINE);
        } else {
            menuItem = new MenuItem(MI_MULTILINE);
        }
        menuItem.addActionListener(this);
        popup.add(menuItem);
    }
	//if (multiline) {
	//	menuItem = new MenuItem(MI_SINGLELINE);
	//} else {
	//	menuItem = new MenuItem(MI_MULTILINE);
	//}
	menuItem.addActionListener(this);
	popup.add(menuItem);
	if (showDefault) {
		menuItem = new MenuItem(MI_HIDEDEFAULT);
	} else {
		menuItem = new MenuItem(MI_SHOWDEFAULT);
	}
	menuItem.addActionListener(this);
	popup.add(menuItem);
	menuItem = new MenuItem(MI_REFRESH);
	menuItem.addActionListener(this);
	popup.add(menuItem);

	return popup;
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return owner.getModel();
}
public Model getParent() {
	return owner;
}
public final String getPort() {
	String port = "";
	String adaptor = getRoute();
	int index = adaptor.lastIndexOf(':');
	if (-1 < index) {
		port = adaptor.substring(index+1);
	}
	return port;
}
public String getProtocol() {
    String route = getRoute();
    String protocol = "";
    if (null != route) {
	    protocol = route;
        int firstColon = route.indexOf(':');
        if (-1 < firstColon) {
            protocol = route.substring(0, firstColon);
        }
    }
    return protocol;
}
/**
 * reply business purpose if set, null if not
 */	
public final String getPurpose() {
	return purpose;
}
public final QueueManager getQueueManager() {
	return owner;
}
/**
 * reply the queue manager (if any) to which this connections resolves
 */
final QueueManager getResolvedQueueManager() {
	QueueManager queueManager = null;
    if (isViaConnection()) {
        Connection connection = owner.getConnection(getRoute());
        if (null != connection) {
            queueManager = connection.getResolvedQueueManager();
        }
    } else {
        Host host = resolveHost();
        if (null != host) {
	       	Listener listener = host.getListenerByPort(getPort());
            if (null != listener) {
                queueManager = listener.getQueueManager();
            }
        }
    }
	return queueManager;
}
public final String getRoute() {
	return getAttribute(ATTRIBUTE_CON_ROUTE).getValue();
}
final boolean getShowPurpose() {
	return showPurpose;
}
public String getType() {
	if (isViaConnection) {
		return "ViaConnection";
	}
	return "Connection";
}
public boolean isConnection() {
	return true;
}
public final boolean isViaConnection() {
	return isViaConnection;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
    if (null != defaultRelationship) {
        // if there is a relationship, check to see if it is still valid
        // and if not then cut it free
        if (!defaultRelationship.isAttached()) {
            defaultRelationship = null;
        }
    }
    if (null != link && !link.isValid()) {
        getQueueManager().removeLink(link);
        link = null;
    }

    if (!isToBeDeleted()) {
        if (isViaConnection()) {
            Connection conn = owner.getConnection(getRoute());
            if (null != conn && null == defaultRelationship) {
                defaultRelationship = new DefaultRelationship(this, conn, "connects via");
                if (null == link) {
                    Listener destListener = conn.resolveListener();
                    if (null != destListener) {
             	       link = new Link(this, destListener);
              	      owner.addLink(link);
                    }
                }
            }
        } else {
            Listener destListener = resolveListener();
            if (null != destListener) {
                if (null == defaultRelationship) {
                    defaultRelationship = new DefaultRelationship(this, destListener, "connects to");
                }
                if (null == link) {
                    // create link and add to parent queue manager
                    link = new Link(this, destListener);
                    owner.addLink(link);
                }
            }
        }
    }

    if (null != defaultRelationship) {
        defaultRelationship.setVisible(showDefault && ShowDefaultRelationships);
    }
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public void performDelete() {
	owner.removeConnection(this);
	declareDeleted();
}
/**
 * return the underlying Connection represented by the receiver
 *
 * ordinary connections return themselves, via connections
 * return the result of querying their referened connection.
 *
 * If a chain of via connections does not have a real connection
 * then null is returned.
 */
public final Connection resolve() {
	Connection con = this;
	if (isViaConnection()) {
		con = owner.getConnection(getRoute());
		if (null != con) {
			con = con.resolve();
		}
	}
	return con;
}
public final Host resolveHost() {
	Host host = null;
	String hostName = getHostName();
	if (null == hostName) {
		//System.out.println("ERROR: null hostName in connection");
	} else {
		host = owner.getHostByAddress(hostName);
		if (null == host) {
	        // if we did not find a host it could be because we are using the
	        // local host and refering to it by a special term - either as
	        // 'localhost' or as '127.0.0.1'.  If this is so then we allow sme
	        // lassitude by looking up the real local host name.
			if ("127.0.0.1".equals(hostName) || "localhost".equals(hostName)) {
				try {
					java.net.InetAddress ina = java.net.InetAddress.getLocalHost();
					host = owner.getHostByAddress(ina.getHostName());
				} catch (java.net.UnknownHostException e) {
				}
			}
		}
	}
	return host;
}
public final Listener resolveListener() {
	Listener listener = null;
	Host host = resolveHost();
	if (null != host) {
		listener = host.getListenerByPort(getPort());
	}
	return listener;
}
public final boolean resolvesTo(String name) {
	if (getName().equals(name)) {
		return true;
	}
	return false;
}
/**
 * called by external entities to set an attribute value,  If the attribute
 * does not exit then the call is ignored.
 * Intercept here to detect route being set, so we can set the via Connection flag
 */
public void setAttribute(String name, String value) {
    super.setAttribute(name, value);
    if (ATTRIBUTE_CON_ROUTE.equals(name)) {
        if (null != value) {
            int index = value.indexOf(':');
            if (0 > index) {	// no colon means a via route
                isViaConnection = true;
            }
        }
    }
}
public final void setChannel(String channel) {
	setAttribute(ATTRIBUTE_CON_CHANNEL, channel);
}
public final void setIsViaConnection() {
	isViaConnection = true;
}
/**
 * set business purpose (empty string clears)
 */
public final void setPurpose(String str) {
	if (null == str || str.length()  == 0) {
		purpose = null;
	} else {
		purpose = str;
	}
}
public final void setRoute(String adaptor) {
	setAttribute(ATTRIBUTE_CON_ROUTE, adaptor);
}
}
