package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/
public class HomeServerQueue extends AbstractRemoteQueue {
	private PullMessageRoute messageRoute;
    private DefaultRelationship resolvedQueue = null;
/**
 * HomeServerQueue constructor comment.
 * @param owner snoop.model.QueueManager
 */
public HomeServerQueue(QueueManager owner) {
	super(owner);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or unreachable
 *
 * Most subclasses will be quite content with the creation method used here 
 * - the step representing the receiver will be returned for the benefit of the
 * caller.  If the receiver wishes to propagate the resolution it should
 * provide an alternate implementation of this method.
 */
AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = super._resolveMessageRoute(previousStep, qmName, qName);
    getQueueManager()._resolveMessageRoute(step, qmName, qName);
    return step;
}
/**
 * delete this node if not already deleted
 */
public void delete() {
	if (null != resolvedQueue) {
		resolvedQueue.detach();
	}
    if (null != messageRoute) {
        getQueueManager().removeMessageRoute(messageRoute);
        messageRoute = null;
    }
	super.delete();
}
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" pulls messages from a store and forward queue on queue manager ");
	buf.append(getQueueQueueManagerName());
	buf.append(" and place them on the appropriate queue on its owning queue manager.");
	//TODO: comment on current connectivity
	return buf.toString();
}
public String getType() {
	return "Home Server 'Queue'";
}
public boolean isHomeServerQueue() {
	return true;
}
protected void notifyChange() {
    super.notifyChange();
    if (null != resolvedQueue && !resolvedQueue.isAttached()) {
        resolvedQueue = null;
    }

    if (null != messageRoute && !messageRoute.isValid()) {
        getQueueManager().removeMessageRoute(messageRoute);
	    messageRoute.delete();
        messageRoute = null;
    }

    if (null == messageRoute) {
        // if we have no route then calculate it
        Connection conn = getQueueManager().getConnection(getQueueQueueManagerName());
        if (null != conn) {
            Listener listener = conn.resolveListener();
            if (null != listener) {
                QueueManager dstQM = listener.getQueueManager();
                //Queue dstQueue = dstQM.getQueue(getQueueQueueManagerName(), getQueueName());
                if (null != dstQM && dstQM != getQueueManager()) {
                    StoreAndForwardQueue safQ = dstQM.getStoreAndForwardQueueFor(getQueueManager().getName());
                    if (null != safQ) {
                        messageRoute = new PullMessageRoute(this, conn, listener, safQ);
                        getQueueManager().addMessageRoute(messageRoute);
                        if (!isToBeDeleted()) {
                            if (null == resolvedQueue) {
                                resolvedQueue = new DefaultRelationship(this, safQ, "pulls from");
                            }
                        }
                    }
                }
            }
        }
    }

    if (null != resolvedQueue) {
        resolvedQueue.setVisible(showDefault && ShowDefaultRelationships);
    }
}
}
