package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Color;

import java.awt.event.ActionEvent;

/**
 * Insert the type's description here.
 * Creation date: (23/08/2001 08:24:32)
 * @author: 
 */
public class Host extends Model {
	private static final String MI_ADD_QUEUEMANAGER = "Add QueueManager"; 
	private static final String MI_ADD_MQQUEUEMANAGER = "Add MQ QueueManager"; 

	private final Vector queueManagers = new Vector();
	private MQeModel mqeModel;
	private final Vector mqQueueManagers = new Vector();
public Host(MQeModel mqeModel) {
	// eventually wish to remove snooper from host...
	super();
	this.mqeModel = mqeModel;
}
public void actionPerformed(ActionEvent e) {
	String cmd = e.getActionCommand();
	if (MI_DELETE.equals(cmd)) {
		delete();
		mqeModel.deleteHost(this);
	} else if (MI_ADD_QUEUEMANAGER.equals(cmd)) {
		performAddQueueManager();
	} else if (MI_ADD_MQQUEUEMANAGER.equals(cmd)) {
		performAddMQQueueManager();
	} else {
		super.actionPerformed(e);
	}
}
public final void addMQQueueManager(MQQueueManager queueManager) {
	if (!mqQueueManagers.contains(queueManager)) {
		mqQueueManagers.addElement(queueManager);
	}
}
public final void addQueueManager(QueueManager queueManager) {
	if (!queueManagers.contains(queueManager)) {
		queueManagers.addElement(queueManager);
	}
}
/**
 * Commit all changes for receiver and receivers children
 */
public void commitAll() {
	Enumeration enum = queueManagers.elements();
	while (enum.hasMoreElements()) {
		StateModel model = (StateModel)enum.nextElement();
		model.commitAll();
	}
}
/**
 * delete this node if not already deleted
 */
public void delete() {
	for (int i = 0; i < queueManagers.size(); i++){
		((Model)queueManagers.elementAt(i)).delete();
	}
	for (int i = 0; i < mqQueueManagers.size(); i++){
		((Model)mqQueueManagers.elementAt(i)).delete();
	}
	super.delete();
}
public final StoreAndForwardQueue findStoreAndForwardQueueFor(String queueManagerName) {
	StoreAndForwardQueue found = null;
	Enumeration enum = queueManagers.elements();
	while (null == found && enum.hasMoreElements()) {
		QueueManager qm = (QueueManager)enum.nextElement();
		found = qm.findStoreAndForwardQueueFor(queueManagerName);
	}
	return found;
}
public final String getAddress() {
	return getName();
}
public final QueueManager getChildQueueManager(String name) {
	QueueManager found = null;
	Enumeration enum = queueManagers.elements();
	while (enum.hasMoreElements()) {
		QueueManager qm = (QueueManager)enum.nextElement();
		if (qm.getName().equals(name)) {
			found = qm;
		}
	}
	return found;
}
public final QueueManager getChildQueueManagerByPort(String port) {
	QueueManager found = null;
	Enumeration enum = queueManagers.elements();
	while (null == found && enum.hasMoreElements()) {
		QueueManager qm = (QueueManager)enum.nextElement();
		Enumeration listeners = qm.getListeners();
		while (null == found && listeners.hasMoreElements()) {
			String listeningPort = ((Listener)listeners.nextElement()).getPort();
			if (listeningPort.equals(port)) {
				found = qm;
			}
		}
	}
	return found;
}
public String getDescription() {
	return "(Host)" + getName();
}
/**
 * return  a textual description of the receiver
 */
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	Enumeration mqeEnum = getQueueManagers();
	Enumeration mqEnum = getMQQueueManagers();
	if (mqeEnum.hasMoreElements() || mqEnum.hasMoreElements()) {
		buf.append(" provides a platform for the following: ");
		while (mqeEnum.hasMoreElements()) {
			Model model = (Model)mqeEnum.nextElement();
			buf.append("\n\t");
			buf.append(model.getShortDescription());
		}
		while (mqEnum.hasMoreElements()) {
			Model model = (Model)mqEnum.nextElement();
			buf.append("\n\t");
			buf.append(model.getShortDescription());
		}
	} else {
		buf.append(" currently hosts no queue managers ");
	}
	return buf.toString();
}
public final Host getHost(String name) {
	return mqeModel.getHost(name);
}
public final Host getHostByAddress(String name) {
	return mqeModel.getHostByAddress(name);
}
public final Listener getListenerByPort(String port) {
	Listener found = null;
	Enumeration enum = queueManagers.elements();
	while (null == found && enum.hasMoreElements()) {
		QueueManager qm = (QueueManager)enum.nextElement();
		found = qm.getListenerByPort(port);
	}
	return found;
}
/**
 * Search among the recievers children for a bridge listener pulling
 * messages from the specified transmit queue.
 * return null if not found
 */
BridgeListener getListenerFor(MQXmitQueue txQueue) {
    BridgeListener bridgeListener = null;
    for (int i = 0; null == bridgeListener && i < queueManagers.size(); i++) {
        bridgeListener =((QueueManager) queueManagers.elementAt(i)).getListenerFor(txQueue);
    }
    return bridgeListener;
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem;
	menuItem = new MenuItem(MI_ADD_QUEUEMANAGER);
	menuItem.addActionListener(this);
	popup.add(menuItem);

	menuItem = new MenuItem(MI_ADD_MQQUEUEMANAGER);
	menuItem.addActionListener(this);
	popup.add(menuItem);

	return popup;
}
/**
 * reply the model to which the receiver belongs
 */
public final MQeModel getModel() {
	return mqeModel;
}
public final MQQueueManager getMQQueueManager(String queueManagerName) {
	MQQueueManager queueManager = null;
	Enumeration enum = mqQueueManagers.elements();
	while (null == queueManager && enum.hasMoreElements()) {
		MQQueueManager qm = (MQQueueManager)enum.nextElement();
		if (qm.getName().equals(queueManagerName)) {
			queueManager = qm;
		}
	}
	return queueManager;
}
public final Enumeration getMQQueueManagers() {
	return mqQueueManagers.elements();
}
/**
 * Get a host by name.  Create one if it does not exist
 */
public final Host getOrCreateHost(String hostName) {
	return mqeModel.getOrCreateHost(hostName);
}
public final MQQueueManager getOrCreateMQQueueManager(String queueManagerName) {
	MQQueueManager queueManager = getMQQueueManager(queueManagerName);
	if (null == queueManager) {
		queueManager = new MQQueueManager();
		queueManager.setHost(this);
		queueManager.setName(queueManagerName);
		addMQQueueManager(queueManager);
		addNode(queueManager);
	}
	return queueManager;
}
/**
 * This method will become the prefferred (only) method when listeners are fully implemented
 */
public final QueueManager getOrCreateQueueManager(String queueManagerName) {
	QueueManager queueManager = getQueueManager(queueManagerName);
	if (null == queueManager) {
		queueManager = new QueueManager();
		queueManager.setHost(this);
		queueManager.setName(queueManagerName);
		addQueueManager(queueManager);
		addNode(queueManager);
		queueManager.commit();
	}
	return queueManager;
}
public Model getParent() {
	return mqeModel;
}
public final QueueManager getQueueManager(String queueManagerName) {
	QueueManager queueManager = null;
	Enumeration enum = queueManagers.elements();
	while (null == queueManager && enum.hasMoreElements()) {
		QueueManager qm = (QueueManager)enum.nextElement();
		if (qm.getName().equals(queueManagerName)) {
			queueManager = qm;
		}
	}
	return queueManager;
}
public final QueueManager getQueueManagerGlobally(String queueManagerName) {
	return mqeModel.getQueueManager(queueManagerName);
}
public final Enumeration getQueueManagers() {
	return queueManagers.elements();
}
public String getType() {
	return "Host";
}
public boolean isHost() {
	return true;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	for (int i = 0; i < queueManagers.size(); i++){
		((Model)queueManagers.elementAt(i)).notifyChange();
	}
	for (int i = 0; i < mqQueueManagers.size(); i++){
		((Model)mqQueueManagers.elementAt(i)).notifyChange();
	}
}
public void performAddMQQueueManager() {
	// the get data routine prompts user for data and validates the return
	// values.  If the return is non-null we can assume a string array of
	// at least three elements, each of which is non-null;
	String[][] data = new String[][] {
		{"name", "MQ_QueueManager",}};
	
	if (getView().getStringData("Add MQ Queue Manager", data)) {
		String qmName = data[0][1];
		getOrCreateMQQueueManager(qmName);
	}
}
public void performAddQueueManager() {
	// the get data routine prompts user for data and validates the return
	// values.  If the return is non-null we can assume a string array of
	// at least three elements, each of which is non-null;
	String suggestName = getName();
	if (suggestName.toLowerCase().endsWith("host")) {
		suggestName = suggestName.substring(0, suggestName.length() - 4);
	}
	suggestName = suggestName + "QM";
	
	String[][] data = new String[][] {
		{"name", suggestName,}, 
		//{"route", "FastNetwork:"+host.getAddress()+":8082",}, 
		//{"channel", "DefaultChannel",}, 
	};
	
	if (getView().getStringData("Add Queue Manager", data)) {
		String qmName = data[0][1];
		//String qmRoute = data[1][1];
		getOrCreateQueueManager(qmName);
	}
}
public final boolean removeMQQueueManager(MQQueueManager qm) {
	if (mqQueueManagers.contains(qm)) {
		mqQueueManagers.removeElement(qm);
		return true;
	}
	return false;
}
public final boolean removeMQQueueManager(QueueManager qm) {
	if (mqQueueManagers.contains(qm)) {
		mqQueueManagers.removeElement(qm);
		return true;
	}
	return false;
}
public final boolean removeQueueManager(QueueManager qm) {
	if (queueManagers.contains(qm)) {
		queueManagers.removeElement(qm);
		return true;
	}
	return false;
}
public void requestDelete() {
	mqeModel.deleteHost(this);
}
}
