package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.PopupMenu;
import java.awt.MenuItem;
import java.awt.event.ActionEvent;
/**
 * Insert the type's description here.
 * Creation date: (12/04/2002 08:49:43)
 * @author: 
 */
public class Link extends glyph.Node {
	private Connection connection = null;
	private Connection viaConnection = null;
	private Listener listener = null;
	private MessageRouteDependancy connectionLine;
	private MessageRouteDependancy viaConnectionLine;
	private MessageRouteDependancy listenerLine;
	private MessageRouteDependancy destinationLine;
	private static final String MI_EDITPURPOSE = Connection.MI_EDITPURPOSE;
	private static final String MI_SHOWPURPOSE = Connection.MI_SHOWPURPOSE;
	private static final String MI_SHOWNAME = Connection.MI_SHOWNAME;
/**
 * Insert the method's description here.
 * Creation date: (12/04/2002 08:51:15)
 * @param connection designer.model.Connection
 * @param listener designer.model.Listener
 */
public Link(Connection connection, Listener listener) {
	this.connection = connection;
	this.listener = listener;
	connectionLine = new MessageRouteDependancy(this, connection, "uses", false);
	listenerLine = new MessageRouteDependancy(this, listener, "uses", false);
	destinationLine = new MessageRouteDependancy(this, listener.getQueueManager(), null, false);
}
public void actionPerformed(ActionEvent e) {
	
    String cmd = e.getActionCommand();
	if (MI_EDITPURPOSE.equals(cmd)) {
	    connection.actionPerformed(e);
    } else if (MI_SHOWPURPOSE.equals(cmd)) {
	    connection.actionPerformed(e);
    } else if (MI_SHOWNAME.equals(cmd)) {
	    connection.actionPerformed(e);
    } else {
        super.actionPerformed(e);
    }
}
/**
 * reply the body - the bit of text above the line in the box
 * reply null (or empy array) to show no body.
 * Each string in the array will apear on a separate line
 */
public String[] getBody() {
	String[] body = null;
	if (connection.getShowPurpose()) {
        body = new String[]{connection.getPurpose()};
	} else {
        body = new String[]{connection.getProtocol()+":"+connection.getPort()};
	}
	return body;
}
/**
 * return  a textual description of the receiver
 */
public String getExplanation() {
	StringBuffer buf = new StringBuffer("This link describes the logical connection formed from ");
	buf.append(connection.getShortDescription());
	buf.append(" to ");
	buf.append(listener.getShortDescription());
	buf.append(".");
	return buf.toString();
}
/**
 * reply the header - the bit of text above the line in the box
 * reply null to show no header
 */
protected String getHeader() {
	return null;
}
public java.awt.PopupMenu getMenu() {
    PopupMenu popup = super.getMenu();
    MenuItem menuItem = new MenuItem(MI_EDITPURPOSE);
    menuItem.addActionListener(this);
    popup.add(menuItem);
    
    if (connection.getShowPurpose()) {
        menuItem = new MenuItem(MI_SHOWNAME);
        menuItem.addActionListener(this);
        popup.add(menuItem);
    } else { // we are showing name
        if (null != connection.getPurpose()) {
            menuItem = new MenuItem(MI_SHOWPURPOSE);
            menuItem.addActionListener(this);
            popup.add(menuItem);
        }
    }

    return popup;
}
boolean isValid() {
	boolean valid = true;
	if (null != viaConnection) {
		valid = viaConnectionLine.isAttached();
	}
	if (valid) {
		valid = connectionLine.isAttached()
			 && listenerLine.isAttached();
	}
	return valid;
}
void setVia(Connection via) {
	viaConnection = via;
	viaConnectionLine = new MessageRouteDependancy(this, via, "via", false);
}
}
