package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;

import java.awt.event.ActionEvent;
/**
 * Insert the type's description here.
 * Creation date: (26/10/2001 13:39:56)
 * @author: 
 */
public class MQRemoteQueue extends MQQueue {
	public static final String ATTRIBUTE_TX_QUEUE = "TxQueue";
	public static final String ATTRIBUTE_REMOTEQ_NAME = "RemoteQueueName";
	public static final String ATTRIBUTE_REMOTEQM_NAME = "RemoteQueueManagerName";
    private static final String MI_SHOWDEFAULT = "Show Default Connection";
    private static final String MI_HIDEDEFAULT = "Hide Default Connection";
    private boolean showDefault = true;

    private DefaultRelationship defaultRelationship = null;
/**
 * Insert the method's description here.
 * Creation date: (26/10/2001 13:47:06)
 * @param queueManager designer.model.MQQueueManager
 */
public MQRemoteQueue(MQQueueManager queueManager) {
	super(queueManager);
	createAttribute(ATTRIBUTE_TX_QUEUE, "", false);
	createAttribute(ATTRIBUTE_REMOTEQ_NAME, "", false);
	createAttribute(ATTRIBUTE_REMOTEQM_NAME, "", false);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or found to be unreachable)
 *
 * This method takes the put from a queue manager to a queue (local or remote) on
 * the queue manager.  If the queue is a remote queue then the resolution is
 * continued by the queue.
 */
final AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = super._resolveMessageRoute(previousStep, qmName, qName);
    MQQueue queue = getOwner().getQueue(getTxQueueName());
    if (null != queue && queue.isMQTransmissionQueue()) {
	    //qName = getRemoteQueueName();
	    queue._resolveMessageRoute(step, qmName, qName);
    }
    return step;
}
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_SHOWDEFAULT.equals(cmd)) {
        showDefault = true;
        declareChanged();
    } else if (MI_HIDEDEFAULT.equals(cmd)) {
        showDefault = false;
        declareChanged();
    } else {
        super.actionPerformed(e);
    }
}
String getRemoteQueueManagerName() {
	return getAttribute(ATTRIBUTE_REMOTEQM_NAME).getValue();
}
String getRemoteQueueName() {
	return getAttribute(ATTRIBUTE_REMOTEQ_NAME).getValue();
}
String getTxQueueName() {
	return getAttribute(ATTRIBUTE_TX_QUEUE).getValue();
}
public String getType() {
	return "MQ Remote Queue";
}
public boolean isBridgeQueue() {
	return true;
}
public boolean isMQRemoteQueue() {
	return true;
}
public final boolean matches(String qmName, String qName) {
	return qmName.equals(getRemoteQueueManagerName())
			&& qName.equals(getName());
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
    if (null != defaultRelationship) {
        // if there is a relationship, check to see if it is still valid
        // and if not then cut it free
        if (!defaultRelationship.isAttached()) {
            defaultRelationship = null;
        }
    }
    if (null == defaultRelationship) {
        MQQueue mqq = getOwner().getQueue(getTxQueueName());
        if (null != mqq) {
            defaultRelationship = new DefaultRelationship(this, mqq, "uses");
        }
    }
    if (null != defaultRelationship) {
        defaultRelationship.setVisible(showDefault && ShowDefaultRelationships);
    }
}
void setRemoteQueueManagerName(String remoteQueueManagerName) {
	setAttribute(ATTRIBUTE_REMOTEQM_NAME, remoteQueueManagerName);
}
void setRemoteQueueName(String remoteQueueName) {
	setAttribute(ATTRIBUTE_REMOTEQ_NAME, remoteQueueName);
}
void setTxQueueName(String txQueueName) {
	setAttribute(ATTRIBUTE_TX_QUEUE, txQueueName);
}
}
