package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;
import java.util.Hashtable;

import glyph.*;
/**
 * Insert the type's description here.
 * Creation date: (04/10/2001 09:08:31)
 * @author: 
 */
public class MQeModel extends Model {
	private Hashtable hosts = new Hashtable();
	private View view;
public final void addHost(Host host) {
	String name = host.getName();
	if (null != name) {
		hosts.put(name.toLowerCase(), host);
	}
}
/**
 * declare that the receiver has changed in a way that requires
 * a redraw
 * This could be a header or body change.  This call is not required
 * for an addNode() or a delete().
 */
public void declareChanged() {
    Enumeration enum = hosts.elements();
    while (enum.hasMoreElements()) {
        Model model = (Model) enum.nextElement();
        model.declareChanged();
        model.notifyChange();
    }

}
public final void deleteHost(Host host) {
	String name = host.getName().toLowerCase();
	if (null != name) {
		host.delete();
		hosts.remove(name);
	}
}
public final Enumeration getAllHosts() {
	return hosts.elements();
}
public String getDescription() {
	return "MQeModel";
}
public final Host getHost(String name) {
	if (null != name) {
		return (Host)hosts.get(name.toLowerCase());
	}
	return null;
}
public final Host getHostByAddress(String name) {
	Enumeration enum = hosts.elements();
	while (enum.hasMoreElements()) {
		Host host = (Host)enum.nextElement();
		if (host.getAddress().equals(name)) {
			return host;
		}
	}
	return null;
}
/**
 * Search among the recievers children for a bridge listener pulling
 * messages from the specified transmit queue.
 * return null if not found
 */
BridgeListener getListenerFor(MQXmitQueue txQueue) {
    BridgeListener bridgeListener = null;
    Enumeration enum = getAllHosts();
	while (null == bridgeListener && enum.hasMoreElements()) {
		Host host = (Host)enum.nextElement();
		bridgeListener = host.getListenerFor(txQueue);
	}
    return bridgeListener;
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return this;
}
/**
 * Get a host by name.  Create one if it does not exist
 */
public final Host getOrCreateHost(String hostName) {
	Host host = null;
	if (null != hostName) {
		host = getHost(hostName);
		if (null == host) {
			host = new Host(this);
			host.setName(hostName);
			addHost(host);
			view.addNode(host, 100,100);
		}
	}
	return host;
}
public final QueueManager getQueueManager(String queueManagerName) {
	QueueManager found = null;
	Enumeration enum = getAllHosts();
	while (null == found && enum.hasMoreElements()) {
		Host host = (Host)enum.nextElement();
		found = host.getChildQueueManager(queueManagerName);
	}
	return found;
}
public final void info(String txt) {
	if (null != view) {
		view.info(txt);
	}
}
public final boolean isMQeModel() {
	return true;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	Enumeration enum = hosts.elements();
	while (enum.hasMoreElements()) {
		((Model)enum.nextElement()).notifyChange();
	}
}
public final void removeAllHosts() {
	Enumeration enum = hosts.elements();
	while (enum.hasMoreElements()) {
		((Node)enum.nextElement()).delete();
	}
	hosts = new Hashtable();
}
public void setChildrenVisible() {
    Enumeration enum = hosts.elements();
    while (enum.hasMoreElements()) {
        Host host = (Host) enum.nextElement();
        host.setVisible(true);
    }
    super.setChildrenVisible();
}
final void setView(View view) {
	this.view = view;
}
}
