package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/


import glyph.Node;
import glyph.View;

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Color;


/**
 * This class is the root of all model elements.
 * Model elements should NOT know about the GUI.
 *
 * Creation date: (20/08/2001 12:32:04)
 * @author: 
 */
public abstract class Model extends Node {

	public static final String RESOLUTION_USES = "resolves using";
	public static final String RESOLUTION_RESOLVE = "resolves to";
	public static final String RESOLUTION_VIA = "resolves via";
	public static final String MI_DELETE = "Delete"; 


	static boolean ShowDefaultRelationships = true;
	static boolean IncludeHostResolution = false;
	static boolean IncludeQueueManagerResolution = true;
	static boolean UseExpertMode = false;
	 
	private String name;
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or unreachable
 *
 * Most subclasses will be quite content with the creation method used here 
 * - the step representing the receiver will be returned for the benefit of the
 * caller.  If the receiver wishes to propagate the resolution it should
 * provide an alternate implementation of this method.
 */
AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    // the next test checks if we need to create a new node
    // ie: if the given step has a nextStep already, and the nextStep refers
    // to the receiver, then we need not create a new step
    if (null != previousStep.nextStep /*&& this != previousStep.nextStep.model*/) {
	    previousStep.nextStep.detach();
	    previousStep.setNextStep(null);
    }
    
    if (null == previousStep.nextStep) {
        // need to create new step
        AbstractStep step = new PushStep(this);
        step.setTarget(qmName, qName);
        previousStep.setNextStep(step);
    }
    return previousStep.nextStep;
}
/**
 * declare that the receiver has changed in a way that requires
 * a redraw
 * This could be a header or body change.  This call is not required
 * for an addNode() or a delete().
 */
public void declareChanged() {
	getModel().notifyChange();
	super.declareChanged();
}
public Color getBackgroundColour() {
	return Color.lightGray;
}
public String[] getBody() {
	return new String[]{getDisplayName()};
}
public abstract String getDescription();
/**
 * Get the string representing the receiver in a gui
 */
public String getDisplayName() {
	return getName();
}
/**
 * return  a textual description of the receiver
 */
public String getExplanation() {
	return getDescription()+" has no explanation (yet).";
}
public String getHeader() {
	return getType();
}
public PopupMenu getMenu() {
	PopupMenu popup = super.getMenu();
	MenuItem menuItem = new MenuItem(MI_DELETE);
	menuItem.addActionListener(this);
	popup.add(menuItem);
	return popup;
}
/**
 * reply the model to which the receiver belongs
 */
protected abstract MQeModel getModel();
public String getName() {
	return name;
}
public Model getParent() {
	System.out.println("Model Element without parent: "+this);
	return null;
}
public QueueManager getQueueManager() {
	return null;
}
public final String getShortDescription() {
	return "("+getType()+")"+getName();
}
public String getType() {
	return null;
}
public final String getXML() {
	return XML.GenerateFrom(this);
}
public void info(String txt) {
	getModel().info(txt);
}
public boolean isAdminQueue() {
	return false;
}
public boolean isBridge() {
	return false;
}
public boolean isBridgeConnection() {
	return false;
}
public boolean isBridgeListener() {
	return false;
}
public boolean isBridgeQueue() {
	return false;
}
public boolean isBridgeQueueManager() {
	return false;
}
public boolean isConnection() {
	return false;
}
public boolean isConnectionAdapter() {
	return false;
}
public boolean isHomeServerQueue() {
	return false;
}
public boolean isHost() {
	return false;
}
public boolean isLocalQueue() {
	return false;
}
public boolean isLogicalConnection() {
	return false;
}
public boolean isLogicalQueue() {
	return false;
}
public boolean isMQChannel() {
	return false;
}
public boolean isMQeModel() {
	return false;
}
public boolean isMQLocalQueue() {
	return false;
}
public boolean isMQQueueManager() {
	return false;
}
public boolean isMQRemoteQueue() {
	return false;
}
public boolean isMQTransmissionQueue() {
	return false;
}
public boolean isQueue() {
	return false;
}
public boolean isQueueAlias() {
	return false;
}
public boolean isQueueManager() {
	return false;
}
public boolean isQueueManagerAlias() {
	return false;
}
public boolean isQueueManagerEntry() {
	return false;
}
public boolean isRemoteQueue() {
	return false;
}
public boolean isStoreAndForwardQueue() {
	return false;
}
/**
 * Method caled when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
}
/**
 * request the receiver refresh
 */
public void refresh() {
}
public boolean removeAlias(String aliasName) {
	return false;
}
public final void setName(String name) {
	this.name = name;
}
public final String toString() {
	return getXML();
}
public Model unAlias() {
	return this;
}
}
