package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.util.Enumeration;

import java.awt.PopupMenu;
import java.awt.MenuItem;
import java.awt.Menu;
import java.awt.Color;
import java.awt.event.ActionEvent;

/**
 * Insert the type's description here.
 * Creation date: (20/08/2001 14:02:38)
 * @author: 
 */
public class Queue extends LogicalQueue {
	public static final String ATTRIBUTE_QUEUE_ACTIVE = "Active";
	public static final String ATTRIBUTE_QUEUE_FILE_DESCRIPTOR = "File Descriptor";
	public static final String ATTRIBUTE_QUEUE_CRYPTOR = "Cryptor";
	public static final String ATTRIBUTE_QUEUE_COMPRESSOR = "Compressor";
	public static final String ATTRIBUTE_QUEUE_AUTHENTICATOR = "Authenticator";
	public static final String ATTRIBUTE_QUEUE_TARGET_REGISTRY = "Target Registry";
	public static final String ATTRIBUTE_QUEUE_ATTRIBUTE_RULE = "Attribute Rule";
	public static final String ATTRIBUTE_QUEUE_RULE = "Rule";
	public static final String ATTRIBUTE_QUEUE_MAX_MESSAGE_SIZE = "Max Message Size";
	public static final String ATTRIBUTE_QUEUE_DEFAULT_PRIORITY = "Default Priority";
	public static final String ATTRIBUTE_QUEUE_MODE = "Mode";
	public static final String ATTRIBUTE_QUEUE_EXPIRY = "Expiry";
	public static final String ATTRIBUTE_QUEUE_MAX_SIZE = "Max Queue Size";
	public static final String ATTRIBUTE_QUEUE_SIZE = "Current Size";
	public static final String ATTRIBUTE_QUEUE_CREATION = "Creation Date";

	protected static final Color QUEUE_COLOUR = new Color(255, 175, 255);

	private static final String MI_ADDALIAS = "Add alias";
	private static final String MI_SINGLELINE = "Single Line Name";
	private static final String MI_MULTILINE = "Multiline Name";
	
	static final String MI_EDITPURPOSE = "Edit Purpose";		// package-private as used by message route
	static final String MI_SHOWPURPOSE = "Show Purpose";		// package-private as used by message route
	static final String MI_SHOWNAME = "Show Name";				// package-private as used by message route

	
	private boolean multiline = false;		
	private boolean showPurpose = false;
	
	private QueueManager owner;
	private String queueManagerName;
	private String purpose;
	private String queueName;
	private final Category aliasCategory = new Category(this, "Aliases");
/**
 * Queue constructor comment.
 */
public Queue(QueueManager owner) {
    super();
    this.owner = owner;

    createAttribute(ATTRIBUTE_QUEUE_ACTIVE, "false", false);
    createAttribute(ATTRIBUTE_QUEUE_FILE_DESCRIPTOR, "", true);
    createAttribute(ATTRIBUTE_QUEUE_CRYPTOR, "", true);
    createAttribute(ATTRIBUTE_QUEUE_COMPRESSOR, "", true);
    createAttribute(ATTRIBUTE_QUEUE_AUTHENTICATOR, "", true);
    createAttribute(ATTRIBUTE_QUEUE_TARGET_REGISTRY, "", true);
    createAttribute(ATTRIBUTE_QUEUE_ATTRIBUTE_RULE, "", true);
    createAttribute(ATTRIBUTE_QUEUE_RULE, "", true);
    createAttribute(ATTRIBUTE_QUEUE_MAX_MESSAGE_SIZE, "-1", true);
    createAttribute(ATTRIBUTE_QUEUE_MAX_SIZE, "-1", true);
    createAttribute(ATTRIBUTE_QUEUE_DEFAULT_PRIORITY, "4", true);
    createAttribute(ATTRIBUTE_QUEUE_MODE, "0", false);
    createAttribute(ATTRIBUTE_QUEUE_EXPIRY, "", true);
    createAttribute(ATTRIBUTE_QUEUE_SIZE, "", false);
    createAttribute(ATTRIBUTE_QUEUE_CREATION, "", false);
}
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_ADDALIAS.equals(cmd)) {
        String data[][] = new String[][] { { "Alias Name", getName()+"Alias" }};
        if (getView().getStringData("Add Queue Alias", data)) {
            addAlias(data[0][1]);
        }
    } else if (MI_SINGLELINE.equals(cmd)) {
	    multiline = false;
	    declareChanged();
    } else if (MI_MULTILINE.equals(cmd)) {
        multiline = true;
	    declareChanged();
    } else if (MI_EDITPURPOSE.equals(cmd)) {
	    String str = "";
	    if (null != purpose) {
		    str = purpose;
	    }
	    String data[][] = new String[][] { {"Purpose", str }, };
        if (getView().getStringData("Edit Purpose", data)) {
	        purpose = data[0][1];
            declareChanged();
        }
    } else if (MI_SHOWPURPOSE.equals(cmd)) {
	    showPurpose = true;
	    declareChanged();
    } else if (MI_SHOWNAME.equals(cmd)) {
	    showPurpose = false;
	    declareChanged();
    } else if (MI_REFRESH.equals(cmd)) {
        refresh();
    } else {
        super.actionPerformed(e);
    }
}
public final QueueAlias addAlias(String aliasName) {
	QueueAlias alias = (QueueAlias)aliasCategory.getChild(aliasName);
	if (null == alias) {
		alias = new QueueAlias(aliasName, this);
		aliasCategory.addNode(alias);
		alias.commit();
	}
	return alias;
}
/**
 * Commit all changes for receiver and receivers children
 */
public void commitAll() {
	super.commitAll();
	aliasCategory.commitAll();
}
/**
 * delete this node if not already deleted
 */
public void delete() {
    aliasCategory.delete();
    super.delete();
}
protected final AdministrationInterface getAdministrator() {
	return owner.getAdministrator();
}
public final QueueAlias getAlias(String name) {
	return (QueueAlias)aliasCategory.getChild(name);
}
public final Vector getAliases() {
	return aliasCategory._getChildren();
}
public String[] getBody() {
    if (null != purpose && showPurpose) {
	    return new String[] { purpose };
    } else {
        if (multiline) {
            return new String[] { getName(), "@" + getQueueManagerName()};
        } else {
            return super.getBody();
        }
    }
}
/**
 *
 */
public Connection getConnection() {
	return null;
}
public String getDescription() {
	return getShortDescription()
		+ " defined on " + getQueueManager().getDescription();
}
public PopupMenu getMenu() {
    PopupMenu popup = super.getMenu();
    MenuItem menuItem = new MenuItem(MI_ADDALIAS);
    menuItem.addActionListener(this);
    popup.add(menuItem);

    menuItem = new MenuItem(MI_EDITPURPOSE);
    menuItem.addActionListener(this);
    popup.add(menuItem);
    
    if (showPurpose) {
        menuItem = new MenuItem(MI_SHOWNAME);
        menuItem.addActionListener(this);
        popup.add(menuItem);
    } else { // we are showing name
        if (null != purpose) {
            menuItem = new MenuItem(MI_SHOWPURPOSE);
            menuItem.addActionListener(this);
            popup.add(menuItem);
        }
        if (multiline) {
            menuItem = new MenuItem(MI_SINGLELINE);
        } else {
            menuItem = new MenuItem(MI_MULTILINE);
        }
        menuItem.addActionListener(this);
        popup.add(menuItem);
    }

    menuItem = new MenuItem(MI_REFRESH);
    menuItem.addActionListener(this);
    popup.add(menuItem);

    return popup;
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return owner.getModel();
}
public final QueueManager getOwningQueueManager() {
	return owner;
}
public Model getParent() {
	return owner;
}
/**
 * reply business purpose if set, null if not
 */	
public final String getPurpose() {
	return purpose;
}
public final QueueManager getQueueManager() {
	return owner;
}
public String getQueueManagerName() {
	return queueManagerName ;
}
public final String getQueueName() {
	return queueName;
}
public final String getQueueQueueManagerName() {
	return queueManagerName;
}
final boolean getShowPurpose() {
	return showPurpose;
}
public boolean isQueue() {
	return true;
}
public final boolean matches(String qmName, String qName) {
	return qmName.equals(queueManagerName)
			&& qName.equals(queueName);
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {
	owner.removeQueue(this);
	declareDeleted();
}
public final void removeAlias(QueueAlias alias) {
	aliasCategory.removeChild(alias);
}
public final Queue resolve() {
	return this;
}
public LogicalQueue resolve(String queueManagerName, String queueName, boolean allowSAFEntries) {
	LogicalQueue found = null;
	if (this.queueManagerName.equals(queueManagerName)) {
		if (getQueueName().equals(queueName)) {
			found = this;
		} else {
			Enumeration enum = aliasCategory._getChildren().elements();
			while (null == found && enum.hasMoreElements()) {
				QueueAlias alias = (QueueAlias) enum.nextElement();
				if (alias.getName().equals(queueName)) {
					found = alias;
				}
			}
		}
	}
	return found;
}
public void setChildrenVisible() {
	super.setChildrenVisible();
	aliasCategory.setChildrenVisible();
}
public final void setName(String queueManagerName, String queueName) {
	this.queueManagerName = queueManagerName;
	setName(queueName);
	this.queueName = queueName;
}
/**
 * set business purpose (empty string clears)
 */
public final void setPurpose(String str) {
	if (null == str || str.length()  == 0) {
		purpose = null;
	} else {
		purpose = str;
	}
}
}
