package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/


import java.awt.Color;

public class QueueAlias extends LogicalQueue {
	protected static final Color ALIAS_COLOUR = new Color(255,200,255);
	Queue parent;
public QueueAlias(String name, Queue parent) {
	super();
	setName(name);
	this.parent = parent;
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or unreachable
 *
 * Most subclasses will be quite content with the creation method used here 
 * - the step representing the receiver will be returned for the benefit of the
 * caller.  If the receiver wishes to propagate the resolution it should
 * provide an alternate implementation of this method.
 */
AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = previousStep;
    if (!isAttached()) {
	    step = super._resolveMessageRoute(previousStep, qmName, qName);
    }
    parent._resolveMessageRoute(step, qmName, parent.getName());
    return step;
}
protected final AdministrationInterface getAdministrator() {
	return parent.getAdministrator();
}
public String getDescription() {
	return getShortDescription() + " for " + parent.getDescription();
}
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" allows ");
	buf.append(parent.getShortDescription());
	buf.append(" to accepts messages destined for a queue named ");
	buf.append(getName());
	buf.append(".\n This allows the user to separate the physical queue (");
	buf.append(parent.getQueueName());
	buf.append("@");
	buf.append(parent.getQueueQueueManagerName());
	buf.append(") from the logical purpose for the queue(");
	buf.append(getName());
	buf.append(").");
	return buf.toString();
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return parent.getModel();
}
public Model getParent() {
	return parent;
}
public final Queue getQueue() {
	return parent;
}
public final QueueManager getQueueManager() {
	return parent.getQueueManager();
}
public String getType() {
	return "Queue Alias";
}
public boolean isQueueAlias() {
	return true;
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {
	parent.removeAlias(this);
	declareDeleted();
}
public Model unAlias() {
	return parent;
}
}
