package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/


import java.util.Vector;
import java.util.Enumeration;

import java.awt.PopupMenu;
import java.awt.Menu;
import java.awt.MenuItem;
import java.awt.Color;

import java.awt.event.ActionEvent;
//import designer.routedlg.PushRouteCreatorDlg;

public class QueueManager extends AttributedModel {
	public static final String ATTRIBUTE_QM_RULES = "Rules";
	public static final String ATTRIBUTE_QM_CHANNEL_TIMEOUT = "Channel Timeout";
	public static final String ATTRIBUTE_QM_CHANNEL_ATTRIBUTE = "Channel Attribute";
	public static final String ATTRIBUTE_QM_DEFAULT_QSTORE = "Default Queue Store";

	private static final String MI_CREATE_LOCAL_QUEUE = "Local";
	private static final String MI_CREATE_ADMIN_QUEUE = "Admin";
	private static final String MI_CREATE_REMOTE_QUEUE = "Remote";
	private static final String MI_CREATE_SAF_QUEUE = "Store and Forward";
	private static final String MI_CREATE_HS_QUEUE = "'Home Server'";
	private static final String MI_CREATE_CONNECTION = "Ordinary Connection";
	private static final String MI_CREATE_NULL_CONNECTION = "'Null' Connection";
	private static final String MI_CREATE_VIA_CONNECTION = "'Via' Connection";
	private static final String MI_CREATE_BRIDGE = "Bridge";
	private static final String MI_CREATE_LISTENER = "Listener";
	private static final String MI_CREATE_QM_ALIAS = "QM Alias";

	private static final String MI_CREATE_SINGLEPUSHROUTE = "Single";
	private static final String MI_CREATE_MULIPLEPUSHROUTE = "Multiple";
	private static final String MI_CREATE_PULLROUTE = "Pull";

	private static final String MI_SHOW_MSG_ROUTE = "Show Message Route";
	private static final String MI_HIDE_MSG_ROUTE = "Hide Message Route";
	
	private static final String MI_ADMIN_RUNTIME = "Administer Runtime";
	private static final String MI_ADMIN_REPOSITORY = "Administer Repository";
	private static final String MI_ADMIN_REMOVE = "Remove Administrator";
	private static final String MI_ADMIN_CONNECT = "Connect Admin";
	private static final String MI_ADMIN_DISCONNECT = "Disconnect Admin";
	
	private static final Color QUEUEMANAGER_COLOUR = new Color(180,180,255);
		
	private final Category queueCategory = new Category(this, "Queues");
	private final Category aliasCategory = new Category(this, "Aliases");
	private final Category connectionCategory = new Category(this, "Connections");
	private final Category listenerCategory = new Category(this, "Listeners");
	private final Category bridgeCategory = new Category(this, "Bridges");
	
	private final Category routeCategory = new Category(this, "(Message Routes)");
	private final Category linkCategory = new Category(this, "(Links)");
	
	private Host host;
	private AdministrationInterface administrator;
	private String targetQueueName = "TargetQueue";
	private String targetQueueManagerName = "TargetQM";
	private MessageResolution messageResolution = null;
/**
 * SnoopObject constructor comment.
 */
public QueueManager() {
	super();
	createAttribute(ATTRIBUTE_QM_RULES, "com.ibm.mqe.MQeQueueManagerRule", true);
	createAttribute(ATTRIBUTE_QM_CHANNEL_TIMEOUT, "3600000", true);
	createAttribute(ATTRIBUTE_QM_CHANNEL_ATTRIBUTE, "", true);
	createAttribute(ATTRIBUTE_QM_DEFAULT_QSTORE, "", true);
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or found to be unreachable)
 *
 * This method takes the put from a queue manager to a queue (local or remote) on
 * the queue manager.  If the queue is a remote queue then the resolution is
 * continued by the queue.
 * This is the private method used for continuing a chain
 */
private final AbstractStep __continueMessageRoute(AbstractStep step, String qmName, String qName) {
	if (null == step) {
		return null;
	}
    QueueManagerAlias qma = getQueueManagerAlias(qmName);
    if (null != qma) {
        step = qma._resolveMessageRoute(step, qmName, qName);
        qmName = qma.getValue();
    }
    // the queueName-queueManagerName pair will resolve to a logical queue
    // which then indicates the real queue
    LogicalQueue logicalQueue = resolveQueue(qmName, qName);
    if (null != logicalQueue) {
        step = logicalQueue._resolveMessageRoute(step, qmName, qName);
    }
    return step;
}
private final void _createConnectionDefinition() {
	Vector listeners = _potentialConnectionTargets();
	if (listeners.size() > 0) {
		designer.gui.ListenerSelectDialogue dlg = new designer.gui.ListenerSelectDialogue(
			getView().getFrame(),
			"Create a connection to which listener?",
			listeners);
		dlg.show();
		if (dlg.okPressed()) {
			Listener listener = dlg.selectedListener();
			String destination = listener.getQueueManager().getName();
			
			String route = 
				listener.getAdapter() +
				":" +
				listener.getQueueManager().getHost().getName() +
				":" +
				listener.getPort();
			String channel = "DefaultChannel";
			createConnectionDefinition(destination, route, channel);
		}
	}
}
private final void _createConnectionDefinitionExpertStyle() {
    String data[][] = new String[][] { { "Destination", "RemoteQM" }, {
            "Route", "FastNetwork:remotehost:8082" }, {
            "Channel", "DefaultChannel" }, };
    if (getView().getStringData("Create Connection Definition", data)) {
        if (connectionExists(data[0][1])) {
            // throw error here
        } else {
            createConnectionDefinition(data[0][1], data[1][1], data[2][1]);
        }
    }
}
private final void _createHomeServerQueue() {
	Vector queues = _potentialHomeServerQueueTargets();
	if (queues.size() > 0) {
		designer.gui.QueueSelectDialogue dlg = new designer.gui.QueueSelectDialogue(
			getView().getFrame(),
			"Select destination for remote queue",
			queues);
		dlg.show();
		if (dlg.okPressed()) {
			String qName = dlg.selectedQueueName();
			String qmName = dlg.selectedQueueManagerName();
			createHomeServerQueue(qmName, qName);
		}
	}
}
private final void _createHomeServerQueueExpertStyle() {
    String data[][] = new String[][] { { "QueueManager Name", "RemoteQM" }, {
            "Queue Name", "homeServerQueue" }, };
    if (getView().getStringData("Create Home Server 'Queue'", data)) {
        if (queueExists(getName(), data[0][1])) {
            // throw error here
        } else {
            createHomeServerQueue(data[0][1], data[1][1]);
        }
    }
}
private final void _createRemoteQueue() {
	Vector queues = _potentialRemoteQueueTargets();
	if (queues.size() > 0) {
		designer.gui.QueueSelectDialogue dlg = new designer.gui.QueueSelectDialogue(
			getView().getFrame(),
			"Select destination for remote queue",
			queues);
		dlg.show();
		if (dlg.okPressed()) {
			String qName = dlg.selectedQueueName();
			String qmName = dlg.selectedQueueManagerName();
			createRemoteQueue(qmName, qName);
		}
	}
}
private final void _createRemoteQueueExpertStyle() {
        String data[][] = new String[][] { { "QueueManager Name", "RemoteQM" }, {
                "Queue Name", "RemoteQueue" }, };
        if (getView().getStringData("Create Asynchronous Remote Queue", data)) {
            if (queueExists(getName(), data[0][1])) {
                // throw error here
            } else {
                createRemoteQueue(data[0][1], data[1][1]);
            }
        }
    }
private final void _createViaConnectionDefinition() {
    Vector qmNames = _potentialViaConnectionTargetQMs();
    Vector conNames = new Vector();
    Vector cons = connectionCategory._getChildren();
    for (int i = 0; i < cons.size(); i++) {
        Connection c = (Connection) cons.elementAt(i);
        conNames.addElement(c.getName());
    }
    designer.gui.ViaConnectionDialogue dlg = new designer.gui.ViaConnectionDialogue(
	    getView().getFrame(), qmNames, conNames);
    dlg.show();
    if (dlg.okPressed()) {
	    Connection connection = createConnectionDefinition(
		    dlg.selectedQMName(), 
		    dlg.selectedConnectionName(), 
		    "DefaultChannel");
        connection.setIsViaConnection();
    }
}
private final void _createViaConnectionDefinitionExpertStyle() {
    String data[][] = new String[][] { { "Destination", "TargetQM" }, {
            "Via", "RemoteQM" }, };
    if (getView().getStringData("Create 'Via' Connection", data)) {
        if (connectionExists(data[0][1])) {
            // throw error here
        } else {
            Connection connection = createConnectionDefinition(data[0][1], data[1][1], "DefaultChannel");
            connection.setIsViaConnection();
        }
    }
}
private final Vector _potentialConnectionTargets() {
	Vector listeners = new Vector();
	Enumeration hosts = getModel().getAllHosts();
	while (hosts.hasMoreElements()) {
		Host h = (Host)hosts.nextElement();
		Enumeration qms = h.getQueueManagers();
		while(qms.hasMoreElements()) {
			QueueManager qm = (QueueManager)qms.nextElement();
			if (this != qm) {
				Enumeration ls = qm.getListeners();
				while (ls.hasMoreElements()) {
					listeners.addElement(ls.nextElement());
				}
			}
		}
	}
	return listeners;
}
private final Vector _potentialHomeServerQueueTargets() {
	// return vector of S&F queues with
	Vector queues = new Vector();
	Enumeration hosts = getModel().getAllHosts();
	while (hosts.hasMoreElements()) {
		Host h = (Host)hosts.nextElement();
		Enumeration qms = h.getQueueManagers();
		while(qms.hasMoreElements()) {
			QueueManager qm = (QueueManager)qms.nextElement();
			if (this != qm && connectionExists(qm.getName())) {
				Enumeration qs = qm.getQueues().elements();
				while (qs.hasMoreElements()) {
					Queue q = (Queue)qs.nextElement();
					// only add S&F queue for which there is no remote queue already
					// and which has no storeAndForward queue already
					if (q.isStoreAndForwardQueue() 
						&& null == getQueue(q.getQueueManagerName(), q.getQueueName())
							&& null != ((StoreAndForwardQueue)q).getQueueManagerEntry(getName())) {
						queues.addElement(q);
					}
				}
			}
		}
	}
	return queues;
}
private final Vector _potentialRemoteQueueTargets() {
	Vector queues = new Vector();
	Enumeration hosts = getModel().getAllHosts();
	while (hosts.hasMoreElements()) {
		Host h = (Host)hosts.nextElement();
		Enumeration qms = h.getQueueManagers();
		while(qms.hasMoreElements()) {
			QueueManager qm = (QueueManager)qms.nextElement();
			if (this != qm && connectionExists(qm.getName())) {
				Enumeration qs = qm.getQueues().elements();
				while (qs.hasMoreElements()) {
					Queue q = (Queue)qs.nextElement();
					// only add local queue for which there is no remote queue already
					if (q.isLocalQueue()) { 
						if (null == getQueue(q.getQueueManagerName(), q.getQueueName())) {
							queues.addElement(q);
						}
						// can connect to aliases of local queues
						Vector aliases = q.getAliases();
						for (int i = 0; i < aliases.size(); i++){
							QueueAlias alias =(QueueAlias)aliases.elementAt(i);
							// only add aliases for which there is no remote queue already
							if (null == getQueue(q.getQueueManagerName(), alias.getName())) {
								queues.addElement(alias);
							}
						}
					}
				}
			}
		}
	}
	return queues;
}
private final Vector _potentialViaConnectionTargetQMs() {
	// reply vector of names of known queue managers for which we do not
	// currently have a connection definition
	Vector qmNames = new Vector();
	Enumeration hosts = getModel().getAllHosts();
	while (hosts.hasMoreElements()) {
		Host h = (Host)hosts.nextElement();
		Enumeration qms = h.getQueueManagers();
		while(qms.hasMoreElements()) {
			QueueManager qm = (QueueManager)qms.nextElement();
			if (this != qm && !connectionExists(qm.getName())) {
				qmNames.addElement(qm.getName());
			}
		}
	}
	return qmNames;
}
/**
 * Resolve the put message route.  This is done by adding steps to the route
 * until the destination is reached (or found to be unreachable)
 *
 * This method takes the put from a queue manager to a queue (local or remote) on
 * the queue manager.  If the queue is a remote queue then the resolution is
 * continued by the queue.
 */
final AbstractStep _resolveMessageRoute(AbstractStep previousStep, String qmName, String qName) {
    AbstractStep step = previousStep;
    if (IncludeQueueManagerResolution) {
	    step = super._resolveMessageRoute(previousStep, qmName, qName);
    }
    __continueMessageRoute(step, qmName, qName);
    return step;
}
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_SHOW_MSG_ROUTE.equals(cmd)) {
        String data[][] = new String[][] { { "Queue Name", targetQueueName }, {
                "QueueManager Name", targetQueueManagerName }, };
        if (getView().getStringData("Put Message", data)) {
            if (null != messageResolution) {
                messageResolution.setVisible(false);
                messageResolution = null;
                //declareChanged();
            }
            targetQueueName = data[0][1];
            targetQueueManagerName = data[1][1];
            messageResolution = new MessageResolution(this);
            messageResolution.setTarget(targetQueueManagerName, targetQueueName);
            __continueMessageRoute(messageResolution, targetQueueManagerName, targetQueueName);
            messageResolution.calculateLines();
            getView().showHelp(messageResolution.getDescription(), "Message Route", false);
            declareChanged();
        }
    } else if (MI_CREATE_SINGLEPUSHROUTE.equals(cmd)) {
        //PushRouteCreatorDlg dlg = new PushRouteCreatorDlg(getView().getFrame(), this);
        //dlg.show();
    } else if (MI_CREATE_MULIPLEPUSHROUTE.equals(cmd)) {
    } else if (MI_CREATE_PULLROUTE.equals(cmd)) {
    } else if (MI_HIDE_MSG_ROUTE.equals(cmd)) {
        if (null != messageResolution) {
            messageResolution.setVisible(false);
            messageResolution = null;
            declareChanged();
        }
    } else if (MI_CREATE_LOCAL_QUEUE.equals(cmd)) {
        String data[][] = new String[][] { { "Queue Name", "LocalQueue" }, };
        if (getView().getStringData("Create Local Queue", data)) {
            createLocalQueue(data[0][1]);
        }
    } else if (MI_CREATE_ADMIN_QUEUE.equals(cmd)) {
        String data[][] = new String[][] { { "Queue Name", "AdminQueue" }, };
        if (getView().getStringData("Create Admin Queue", data)) {
            createAdminQueue(data[0][1]);
        }
    } else if (MI_CREATE_REMOTE_QUEUE.equals(cmd)) {
        if (UseExpertMode) {
            _createRemoteQueueExpertStyle();
        } else {
            _createRemoteQueue();
        }
    } else if (MI_CREATE_SAF_QUEUE.equals(cmd)) {
        String data[][] = new String[][] { { "QueueManager Name", "RemoteQM" }, {
                "Queue Name", "SafQueue" }, };
        if (getView().getStringData("Create StoreAndForward Queue", data)) {
            if (queueExists(getName(), data[0][1])) {
                // throw error here
            } else {
                createStoreAndForwardQueue(data[0][1], data[1][1]);
            }
        }
    } else if (MI_CREATE_HS_QUEUE.equals(cmd)) {
        if (UseExpertMode) {
            _createHomeServerQueueExpertStyle();
        } else {
            _createHomeServerQueue();
        }
    } else if (MI_CREATE_CONNECTION.equals(cmd)) {
        if (UseExpertMode) {
            _createConnectionDefinitionExpertStyle();
        } else {
            _createConnectionDefinition();
        }
    } else if (MI_CREATE_VIA_CONNECTION.equals(cmd)) {
        if (UseExpertMode) {
            _createViaConnectionDefinitionExpertStyle();
        } else {
            _createViaConnectionDefinition();
        }
    } else if (MI_CREATE_NULL_CONNECTION.equals(cmd)) {
        String data[][] = new String[][] { { "Destination", "TargetQM" }
        };
        if (getView().getStringData("Create 'Null' Connection", data)) {
            if (connectionExists(data[0][1])) {
                // throw error here
            } else {
                createConnectionDefinition(data[0][1], "Network::", "DefaultChannel");
            }
        }
    } else if (MI_CREATE_LISTENER.equals(cmd)) {
        String data[][] = new String[][] { { "Name", "DefaultListener" }, {
                "Port", "8082" }, {
                "Adapter", "FastNetwork" }, };
        if (getView().getStringData("Create Listener", data)) {
            createListener(data[0][1], data[1][1], data[2][1]);
        }
    } else if (MI_CREATE_BRIDGE.equals(cmd)) {
        String data[][] = new String[][] { { "Bridge Name", "Bridge" }, };
        if (getView().getStringData("Create MQ Bridge", data)) {
            Bridge bridge = (Bridge) bridgeCategory.getChild(data[0][1]);
            if (null == bridge) {
                bridge = new Bridge(this);
                bridge.setName(data[0][1]);
                bridgeCategory.addNode(bridge);
                bridge.commit();
            }
        }
    } else if (MI_CREATE_QM_ALIAS.equals(cmd)) {
        String data[][] = new String[][] { { "Alias Name", "AliasQM" }, {
                "Target Name", "TargetQM" }
        };
        if (getView().getStringData("Create QM Alias", data)) {
            createAlias(data[0][1], data[1][1]);
        }
    } else if (MI_DELETE.equals(cmd)) {
        requestDelete();
        host.removeQueueManager(this);
    } else if (MI_REFRESH.equals(cmd)) {
        refresh();
    } else if (MI_ADMIN_RUNTIME.equals(cmd)) {
        // give qm a runtime administrator
        setAdministrator(AdministratorFactory.getRuntimeAdministrator());
    } else if (MI_ADMIN_REPOSITORY.equals(cmd)) {
        // give runtime a repository administrator
    } else if (MI_ADMIN_REMOVE.equals(cmd)) {
        // remove the current administrator (give a null administrator)
        setAdministrator(null);
    } else if (MI_ADMIN_CONNECT.equals(cmd)) {
        // connect administrator
    } else if (MI_ADMIN_DISCONNECT.equals(cmd)) {
        // diconnect administrator
    } else {
        super.actionPerformed(e);
    }
}
public final void addConnection(Connection connection) {
	connectionCategory.addNode(connection);
	MQeModel model = getModel();
	if (null != model) {
		model.notifyChange();
	}
}
public final void addLink(Link link) {
	linkCategory.addNode(link);
}
public final void addMessageRoute(MessageRoute route) {
	routeCategory.addNode(route);
}
public final void addQueue(Queue queue) {
	queueCategory.addNode(queue);
	getModel().notifyChange();
}
public final Enumeration calculateMessageRoutes() {
	/*
	// look at each queue and see if it is a remote or
	// s&f queue.  if the queue has a resolved queue manager
	// then we can create a message route for it
	Vector newRoutes = new Vector();
	Enumeration enum = queues.elements();
	while (enum.hasMoreElements()) {
		Queue queue = (Queue) enum.nextElement();
		if (queue.isRemoteQueue()) {
			MessageRoute msgRoute = _getMessageRoute(queue);
			if (null == msgRoute) {
	            msgRoute = new SingleMessageRoute((RemoteQueue)queue);
			}
			if (msgRoute.isValid()) {
				newRoutes.addElement(msgRoute);
			}
		} else if (queue.isStoreAndForwardQueue()) {
			MessageRoute msgRoute = _getMessageRoute(queue);
			if (null == msgRoute) {
	            msgRoute = new MultipleMessageRoute((StoreAndForwardQueue)queue);
			}
			if (msgRoute.isValid()) {
				newRoutes.addElement(msgRoute);
			}
		} else if (queue.isHomeServerQueue()) {
			MessageRoute msgRoute = _getMessageRoute(queue);
			if (null == msgRoute) {
	            msgRoute = new PullingMessageRoute((HomeServerQueue)queue);
			}
			if (msgRoute.isValid()) {
				newRoutes.addElement(msgRoute);
			}
		}
	}
	msgRoutes = newRoutes;
	*/
	return getMessageRoutes();
}
/**
 * Commit all changes for receiver and receivers children
 */
public void commitAll() {
	super.commitAll();
	Vector v = getConnections();
	for (int i = 0; i < v.size(); i++){
		StateModel model = (StateModel)v.elementAt(i);
		model.commitAll();
	}
	v = getQueues();
	for (int i = 0; i < v.size(); i++){
		StateModel model = (StateModel)v.elementAt(i);
		model.commitAll();
	}
}
public final boolean connectionExists(String destination) {
	boolean exists = false;
	Vector v = connectionCategory._getChildren();
	for (int i = 0; !exists && i < v.size(); i++){
		Connection con = (Connection) v.elementAt(i);
		Connection resolvedCon = con.resolve();
		if (null != resolvedCon) {
			// a via connection without a valid real connection can return null.
			exists = con.getName().equals(destination);
		}
	}
	return exists;
}
public final AdminQueue createAdminQueue(String queueName) {
	AdminQueue queue = (AdminQueue) getQueue(getName(), queueName);
	if (null == queue) {
		queue = new AdminQueue(this);
		queue.setName(getName(), queueName);
		queueCategory.addNode(queue);
		queue.commit();
	}
	return queue;
}
public final QueueManagerAlias createAlias(String name, String target) {
    QueueManagerAlias qma = (QueueManagerAlias) aliasCategory.getChild(name);
    if (null == qma) {
        qma = new QueueManagerAlias(this, name, target);
        aliasCategory.addNode(qma);
        qma.commit();
    }
    return qma;
}
Bridge createBridge(String name) {
    Bridge bridge = (Bridge) bridgeCategory.getChild(name);
    if (null == bridge) {
        bridge = new Bridge(this);
        bridge.setName(name);
        bridgeCategory.addNode(bridge);
        bridge.commit();
    }
    return bridge;
}
public final BridgeQueue createBridgeQueue(String queueManagerName, String queueName, String bridgeName, String connectionName) {
	BridgeQueue bridgeQueue = null;
	Queue queue = (Queue)getQueue(queueManagerName, queueName);
	if (null == queue) {
		bridgeQueue = new BridgeQueue(this);
		bridgeQueue.setName(queueManagerName, queueName);
		bridgeQueue.setBridgeName(bridgeName);
		bridgeQueue.setConnectionName(connectionName);
		addQueue(bridgeQueue);
		bridgeQueue.commit();
	} else if (queue.isBridgeQueue()) {
		bridgeQueue = (BridgeQueue)queue;
	}
	return bridgeQueue;
}
public final Connection createConnectionDefinition(String destination, String route, String channel) {
	Connection connection = null;
	if (null != getConnection(destination)) {
		// error - would result in ambiguous resolution
	} else {
		connection = new Connection(this);
		connection.setName(destination);
		connection.setChannel(channel);
		if (null != route) {
			// coming in from xml parser route is null,
			// but the adapter will be created by the parser
			connection.setRoute(route);
		}
		addConnection(connection);
		connection.commit();
	}
	return connection;
}
public final HomeServerQueue createHomeServerQueue(String queueManagerName, String queueName) {
	HomeServerQueue queue = (HomeServerQueue) getQueue(queueManagerName, queueName);
	if (null == queue) {
		queue = new HomeServerQueue(this);
		queue.setName(queueManagerName, queueName);
		addQueue(queue);
		queue.commit();
	}
	return queue;
}
public Listener createListener(String name, String port, String adapter) {
	Listener listener = getListener(name);
	if (null == listener) {
		listener = new Listener(this, name, port, adapter);
		listenerCategory.addNode(listener);
		listener.commit();
	}
	return listener;
}
public final LocalQueue createLocalQueue(String queueName) {
	LocalQueue queue = (LocalQueue) getQueue(getName(), queueName);
	if (null == queue) {
		queue = new LocalQueue(this);
		queue.setName(getName(), queueName);
		addQueue(queue);
		queue.commit();
	}
	return queue;
}
public final RemoteQueue createRemoteQueue(String queueManagerName, String queueName) {
	RemoteQueue queue = (RemoteQueue)getQueue(queueManagerName, queueName);
	if (null == queue) {
		queue = new RemoteQueue(this);
		queue.setName(queueManagerName, queueName);
		addQueue(queue);
		queue.commit();
	}
	return queue;
}
public final StoreAndForwardQueue createStoreAndForwardQueue(String queueManagerName, String queueName) {
	StoreAndForwardQueue queue = (StoreAndForwardQueue) getQueue(queueManagerName, queueName);
	if (null == queue) {
		queue = new StoreAndForwardQueue(this);
		queue.setName(queueManagerName, queueName);
		addQueue(queue);
		queue.commit();
	}
	return queue;
}
/**
 * delete this node if not already deleted
 */
public void delete() {
    queueCategory.delete();
    aliasCategory.delete();
    connectionCategory.delete();
    listenerCategory.delete();
    bridgeCategory.delete();

    routeCategory.delete();
    super.delete();
}
/**
 * reply the home server queue (if any) on the receiver that pulls messages
 * from the named queue manager
 */
public final HomeServerQueue findHomeServerQueueFor(String queueManagerName) {
	HomeServerQueue found = null;
	Enumeration enum = getQueues().elements();
	while (null == found && enum.hasMoreElements()) {
		Queue queue = (Queue)enum.nextElement();
		if (queue.isHomeServerQueue() && queue.getQueueManagerName().equals(queueManagerName)) {
			found = (HomeServerQueue)queue;
		}
	}
	return found;
}
public final StoreAndForwardQueue findStoreAndForwardQueueFor(String queueManagerName) {
	StoreAndForwardQueue found = null;
	Enumeration enum = getQueues().elements();
	while (null == found && enum.hasMoreElements()) {
		Queue queue = (Queue)enum.nextElement();
		if (queue.isStoreAndForwardQueue()) {
			StoreAndForwardQueue safq = (StoreAndForwardQueue)queue;
			if (safq.acceptsMessagesFor(queueManagerName)) {
				found = safq;
			}
		}
	}
	return found;
}
public final AdministrationInterface getAdministrator() {
	return administrator;
}
public final Enumeration getAliases() {
	return aliasCategory._getChildren().elements();
}
public final Bridge getBridge(String name) {
	return (Bridge)bridgeCategory.getChild(name);
}
public final Enumeration getBridges() {
	return bridgeCategory._getChildren().elements();
}
/**
 * return the connection resolving to the given name.
 * The name will resolve to an alias for the connection
 */
public final Connection getConnection(String name) {
	return (Connection)connectionCategory.getChild(name);
}
public final Vector getConnections() {
	return connectionCategory._getChildren();
}
public String getDescription() {
	return getShortDescription() +
		" running on " + host.getDescription();
}
/**
 * return  a textual description of the receiver
 */
public String getExplanation() {
	StringBuffer buf = new StringBuffer();
	buf.append(getShortDescription());
	buf.append(" provides an entry point to the MQe system for applications running on ");
	buf.append(host.getShortDescription());
	buf.append(".");
	return buf.toString();
}
public final Host getHost() {
	return host;
}
public final Host getHost(String name) {
	return getHost().getHost(name);
}
public final Host getHostByAddress(String name) {
	return getHost().getHostByAddress(name);
}
/**
 * return the listener resolving to the given name.
 */
public final Listener getListener(final String name) {
	Listener found = null;
	Enumeration enum = getListeners();
	while (null == found && enum.hasMoreElements()) {
		Listener listener =(Listener)enum.nextElement();
		if (listener.getName().equals(name)) {
			found = listener;
		}
	}
	return found;
}
/**
 * return the listener resolving to the given name.
 */
public final Listener getListenerByPort(final String port) {
	Listener found = null;
	Enumeration enum = getListeners();
	while (null == found && enum.hasMoreElements()) {
		Listener listener =(Listener)enum.nextElement();
		if (listener.getPort().equals(port)) {
			found = listener;
		}
	}
	return found;
}
/**
 * Search among the recievers children for a bridge listener pulling
 * messages from the specified transmit queue.
 * return null if not found
 */
BridgeListener getListenerFor(MQXmitQueue txQueue) {
    BridgeListener bridgeListener = null;
    Vector bridges = bridgeCategory._getChildren();
    for (int i = 0; null == bridgeListener && i < bridges.size(); i++) {
        bridgeListener =((Bridge) bridges.elementAt(i)).getListenerFor(txQueue);
    }
    return bridgeListener;
}
public final Enumeration getListeners() {
	return listenerCategory._getChildren().elements();
}
public PopupMenu getMenu() {
    PopupMenu popup = super.getMenu();
    Menu createMenu = new Menu("Create");

    Menu addMenu;
    MenuItem menuItem;
    /*	
    addMenu = new Menu("Message Route");
    menuItem = new MenuItem(MI_CREATE_SINGLEPUSHROUTE);
    menuItem.addActionListener(this);
    addMenu.add(menuItem);
    menuItem = new MenuItem(MI_CREATE_MULIPLEPUSHROUTE);
    menuItem.addActionListener(this);
    addMenu.add(menuItem);
    menuItem = new MenuItem(MI_CREATE_PULLROUTE);
    menuItem.addActionListener(this);
    addMenu.add(menuItem);
    createMenu.add(addMenu);
    	*/

    addMenu = new Menu("Queue");

    menuItem = new MenuItem(MI_CREATE_LOCAL_QUEUE);
    menuItem.addActionListener(this);
    addMenu.add(menuItem);
    menuItem = new MenuItem(MI_CREATE_REMOTE_QUEUE);
    menuItem.addActionListener(this);
    if (!UseExpertMode) {
        menuItem.setEnabled(!_potentialRemoteQueueTargets().isEmpty());
    }
    addMenu.add(menuItem);
    menuItem = new MenuItem(MI_CREATE_SAF_QUEUE);
    menuItem.addActionListener(this);
    addMenu.add(menuItem);
    menuItem = new MenuItem(MI_CREATE_HS_QUEUE);
    if (!UseExpertMode) {
        menuItem.setEnabled(!_potentialHomeServerQueueTargets().isEmpty());
    }
    menuItem.addActionListener(this);
    addMenu.add(menuItem);
    menuItem = new MenuItem(MI_CREATE_ADMIN_QUEUE);
    menuItem.addActionListener(this);
    addMenu.add(menuItem);
    createMenu.add(addMenu);

    addMenu = new Menu("Connection");
    menuItem = new MenuItem(MI_CREATE_CONNECTION);
    menuItem.addActionListener(this);
    if (!UseExpertMode) {
        menuItem.setEnabled(!_potentialConnectionTargets().isEmpty());
    }
    addMenu.add(menuItem);
    menuItem = new MenuItem(MI_CREATE_VIA_CONNECTION);
    menuItem.addActionListener(this);
    if (!UseExpertMode) {
        menuItem.setEnabled(!connectionCategory._getChildren().isEmpty() && !_potentialViaConnectionTargetQMs().isEmpty());
    }
    addMenu.add(menuItem);
    menuItem = new MenuItem(MI_CREATE_NULL_CONNECTION);
    menuItem.addActionListener(this);
    addMenu.add(menuItem);
    createMenu.add(addMenu);

    menuItem = new MenuItem(MI_CREATE_BRIDGE);
    menuItem.addActionListener(this);
    createMenu.add(menuItem);

    menuItem = new MenuItem(MI_CREATE_LISTENER);
    menuItem.addActionListener(this);
    createMenu.add(menuItem);

    menuItem = new MenuItem(MI_CREATE_QM_ALIAS);
    menuItem.addActionListener(this);
    createMenu.add(menuItem);

    popup.add(createMenu);

    if (getListeners().hasMoreElements()) {
        if (null != AdministratorFactory.getRuntimeAdministrator()) {
            AdministrationInterface admin = getAdministrator();
            Menu adminMenu = new Menu("Administration");
            if (null == admin) {
                // disconnect
                menuItem = new MenuItem(MI_ADMIN_RUNTIME);
                menuItem.addActionListener(this);
                adminMenu.add(menuItem);
                menuItem = new MenuItem(MI_ADMIN_REPOSITORY);
                menuItem.setEnabled(false);
                menuItem.addActionListener(this);
                adminMenu.add(menuItem);
            } else {
                // remove administrator (replace with null admin)
                menuItem = new MenuItem(MI_REFRESH);
                menuItem.addActionListener(this);
                adminMenu.add(menuItem);
                menuItem = new MenuItem(MI_ADMIN_REMOVE);
                menuItem.addActionListener(this);
                adminMenu.add(menuItem);
                if (admin.canConnect()) {
                    menuItem = new MenuItem(MI_ADMIN_CONNECT);
                    menuItem.addActionListener(this);
                    adminMenu.add(menuItem);
                }
                if (admin.canDisconnect()) {
                    menuItem = new MenuItem(MI_ADMIN_DISCONNECT);
                    menuItem.addActionListener(this);
                    adminMenu.add(menuItem);
                }
            }
            popup.add(adminMenu);
        }
    }

    Menu msgRouteMenu = new Menu("Message Route");
    menuItem = new MenuItem(MI_SHOW_MSG_ROUTE);
    menuItem.addActionListener(this);
    msgRouteMenu.add(menuItem);
    menuItem = new MenuItem(MI_HIDE_MSG_ROUTE);
    menuItem.addActionListener(this);
    menuItem.setEnabled(null != messageResolution);
    msgRouteMenu.add(menuItem);

    popup.add(msgRouteMenu);

    menuItem = new MenuItem(MI_DELETE);
    menuItem.addActionListener(this);
    popup.add(menuItem);

    return popup;
}
public final Enumeration getMessageRoutes() {
	// 
	return routeCategory._getChildren().elements();
}
/**
 * reply the model to which the receiver belongs
 */
protected final MQeModel getModel() {
	return host.getModel();
}
/**
 * Get a host by name.  Create one if it does not exist
 */
public final Host getOrCreateHost(String hostName) {
	return host.getOrCreateHost(hostName);
}
public Model getParent() {
	return host;
}
/**
 * return the connection resolving to the given name.
 * The name will resolve to an alias for the connection
 */
public final Connection getPhysicalConnection(String name) {
	Connection found = null;
	Enumeration enum = getConnections().elements();
	while (null == found && enum.hasMoreElements()) {
		Connection connection =(Connection)enum.nextElement();
		if (connection.getName().equals(name)) {
			found = connection;
		}
	}
	return found;
}
public final Queue getQueue(String queueManagerName, String queueName) {
	Queue found = null;
	Enumeration enum = getQueues().elements();
	while (null == found && enum.hasMoreElements()) {
		Queue queue =(Queue)enum.nextElement();
		if (queue.matches(queueManagerName, queueName)) {
			found = queue;
		}
	}
	return found;
}
public QueueManager getQueueManager() {
	return this;
}
public final QueueManagerAlias getQueueManagerAlias(String name) {
	return (QueueManagerAlias)aliasCategory.getChild(name);
}
public final Vector getQueues() {
	return queueCategory._getChildren();
}
final Enumeration getQueuesFor(String queueManagerName) {
	Vector foundQueues = new Vector();
	Enumeration enum = getQueues().elements();
	while (enum.hasMoreElements()) {
		Queue queue = (Queue)enum.nextElement();
		if (queue.getQueueQueueManagerName().equals(queueManagerName)) {
			foundQueues.addElement(queue);
		}
	}
	return foundQueues.elements();
}
public final String getRoute() {
    Enumeration enum = getListeners();
    if (enum.hasMoreElements()) {
        Listener listener = (Listener) enum.nextElement();
        String port = listener.getPort();
        return listener.getAdapter() + ":" + host.getAddress() + ":" + port;
    }
    return null;
}
public final StoreAndForwardQueue getStoreAndForwardQueueFor(String queueManagerName) {
	StoreAndForwardQueue found = null;
	Enumeration enum = getQueues().elements();
	while (null == found && enum.hasMoreElements()) {
		Queue queue =(Queue)enum.nextElement();
		if (queue.isStoreAndForwardQueue()) {
			StoreAndForwardQueue safQ = (StoreAndForwardQueue)queue;
			if (null != safQ.getQueueManagerEntry(queueManagerName)) {
				found = safQ;
			}
		}
	}
	return found;
}
public String getType() {
	return "Queue Manager";
}
public boolean isQueueManager() {
	return true;
}
/**
 * Method called when a change occurs that may need a 
 * recalculation - when a model object is deleted or
 * similar.  Sublasses should overide as appropriate
 */
protected void notifyChange() {
	queueCategory.notifyChange();
	connectionCategory.notifyChange();
	bridgeCategory.notifyChange();
	if (null != messageResolution) {
		__continueMessageRoute(messageResolution, targetQueueManagerName, targetQueueName);
		messageResolution.calculateLines();
	}
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public final void performDelete() {}
public final boolean queueExists(String queueManagerName, String queueName) {
	return (null != getQueue(queueManagerName, queueName));
}
final void removeAlias(QueueManagerAlias alias) {
	aliasCategory.removeChild(alias);
}
void removeBridge(Bridge bridge) {
	bridgeCategory.removeChild(bridge);
}
public final void removeConnection(Connection connection) {
	connectionCategory.removeChild(connection);
	getModel().notifyChange();
}
public final void removeLink(Link link) {
	linkCategory.removeChild(link);
}
final void removeListener(Listener listener) {
	listenerCategory.removeChild(listener);
}
public final void removeMessageRoute(MessageRoute route) {
	routeCategory.removeChild(route);
}
public final void removeQueue(Queue queue) {
	queueCategory.removeChild(queue);
	getModel().notifyChange();
}
public final void requestDelete() {
	host.removeQueueManager(this);
	delete();
}
/**
 * Determine all the resolutions of a given queueName/queueManagerName
 * pair.
 *
 * A remote queue will resolve to a connection (hopefully) and then use that
 * connection to determine the end point
 */
public final LogicalQueue resolveQueue(String queueManagerName, String queueName) {
	//queueManagerName = resolveQueueManagerName(queueManagerName);
	
	LogicalQueue found = null;
	// look for a queue which will handle messages for the destination
	Enumeration enum = getQueues().elements();
	while (null == found && enum.hasMoreElements()) {
		Queue queue = (Queue) enum.nextElement();
		found = queue.resolve(queueManagerName, queueName, false);
	}
	if (null == found) {
	    // look for a store and forward queue
		enum = getQueues().elements();
		while (null == found && enum.hasMoreElements()) {
			Queue queue = (Queue) enum.nextElement();
			found = queue.resolve(queueManagerName, queueName, true);
		}
	}
	return found;

}
/**
 * Find the real queue manager name (it may be aliased by a connection)
 */
public final String resolveQueueManagerName(String queueManagerName) {
	Connection connection = getConnection(queueManagerName);
	if (null != connection) {
		// if there is a connection it could be an alias so we need to substitute
		// the real queue manager name.
		queueManagerName = connection.resolve().getName();
	}
	return queueManagerName;
}
public final void setAdministrator(AdministrationInterface admin) {
	administrator = admin;
}
public void setChildrenVisible() {
	super.setChildrenVisible();
	queueCategory.setChildrenVisible();
	aliasCategory.setChildrenVisible();
	connectionCategory.setChildrenVisible();
	listenerCategory.setChildrenVisible();
	bridgeCategory.setChildrenVisible();
	routeCategory.setChildrenVisible();
}
public final void setHost(Host host) {
	this.host = host;
}
}
