package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import glyph.Line;
import glyph.Node;
import java.awt.Color;

/**
 * Insert the type's description here.
 * Creation date: (12/02/2002 19:27:56)
 * @author: 
 */
public class ResolutionLine extends Line {
	private static final Color PULL_COLOR = new Color(255,190,255);
	private String[] body;
	private boolean isSource = false;
	private boolean visible = true;
	private boolean isPull = false;
/**
 * MessageRouteDependancy constructor comment.
 * @param src glyph.Node
 * @param dst glyph.Node
 */
public ResolutionLine(Node src, Node dst, String label, boolean isSource) {
	super(src, dst);
	if (null != label) {
		body = new String[]{label};
	}
	this.isSource = isSource;
}
/**
 * MessageRouteDependancy constructor comment.
 * @param src glyph.Node
 * @param dst glyph.Node
 */
public ResolutionLine(Node src, Node dst, String label, boolean isSource, boolean isPull) {
	super(src, dst);
	if (null != label) {
		body = new String[]{label};
	}
	this.isPull = isPull;
	this.isSource = isSource;
}
/**
 * reply the body - the bit of text above the line in the box
 * reply null (or empy array) to show no body.
 * Each string in the array will apear on a separate line
 */
protected java.lang.String[] getBody() {
	return body;
}
/**
 * reply he symbol for the destination end of he line 
 * override to change symbol
 */
protected int getDestinationSymbol() {
	if (isSource) {
		return SYMBOL_NONE;
	} else {
		if (isPull) {
			return SYMBOL_ARROW;
		} else {
			return SYMBOL_BLUNT_ARROW;
		}
	}
}
/**
 * reply the header - the bit of text above the line in the box
 * reply null to show no header
 */
protected String getHeader() {
	return null;
}
	/**
	 * reply importance of line.  Higher importance lines
	 * mask lower importance lines.  Default is 10.
	 */
	public int getImportance() {
		return 20;
	}
/**
 * reply the menu 
 * reply null for no menu
 */
protected java.awt.PopupMenu getMenu() {
	return null;
}
/**
 * reply he symbol for the destination end of he line 
 * override to change symbol
 */
protected int getSourceSymbol() {
	if (isSource) {
		return SYMBOL_BLUNT_ARROW;
	} else {
		return SYMBOL_NONE;
	}
}
/**
 * reply true if the receiver is visible
 * use to control drawing
 */
protected boolean isVisible() {
	return visible;
}
/**
 * reply true if the receiver is visible
 * use to control drawing
 */
protected void setVisible(boolean set) {
	visible = set;
}
}
