package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Vector;
import java.awt.event.ActionEvent;

/**
 * Insert the type's description here.
 * Creation date: (11/09/2001 07:06:57)
 * @author: 
 */
public abstract class StateModel extends Model implements StateModelConstants {

	private int state = STATE_TOBECREATED;
public void actionPerformed(ActionEvent e) {
    String cmd = e.getActionCommand();
    if (MI_DELETE.equals(cmd)) {
        requestDelete();
    } else {
        super.actionPerformed(e);
    }
}
/**
 * Called to request that the receiver performs any
 * outstanding actions on the the resource it represents.
 * The receiver should return true if any visible
 * change has occurred
 */
public final void commit() {
    if (null != getAdministrator()) {
        getAdministrator().commit(this);
    } else {
	    // if there is no administrator then drive the changes through the state model
        switch (getState()) {
            case STATE_TOBECREATED :
                declareCreated();
                break;
            case STATE_TOBEUPDATED :
                declareUpdated();
                break;
            case STATE_CREATING :
                break;
            case STATE_CREATED :
                break;
            case STATE_TOBEDELETED :
                performDelete();
                break;
            case STATE_DELETING :
                break;
            case STATE_DELETED :
                break;
            default :
                break;
        }
    }
}
/**
 * Commit all changes for receiver and receivers children
 */
public void commitAll() {
	commit();
}
public final void declareCreated() {
	setState(STATE_CREATED);
}
public final void declareDeleted() {
	setState(STATE_DELETED);
}
public final void declareUpdated() {
	if (STATE_TOBEUPDATED == state) {
		// only if not pending delete or create
		state = STATE_CREATED;
	}
}
protected abstract AdministrationInterface getAdministrator() ;
public final int getState() {
	return state;
}
public final boolean isToBeCreated() {
	return STATE_TOBECREATED == getState();
}
public final boolean isToBeDeleted() {
	return STATE_TOBEDELETED == getState();
}
/**
 * Called to request that the receiver performs a create upon
 * the resource it represents.
 * The receiver should return true if succesful
 */
public boolean performCreate() {
	return false;
}
/**
 * Called to request that the receiver removes itself
 * from model
 */
public abstract void performDelete() ;
public boolean performRefresh() {
	return false;
}
/**
 * request the receiver refresh
 */
public final void refresh() {
	state = STATE_TOBEREFRESHED;
	commit();
}
public void requestDelete() {
	if (STATE_TOBECREATED == state) {
		state = STATE_DELETENORESOURCE;
		performDelete();
	} else {
		state = STATE_TOBEDELETED;
		commit();
	}
}
public final void setRequiresUpdate() {
	if (STATE_CREATED == state) {
		// only if not pending delete or create
		state = STATE_TOBEUPDATED;
	}
}
public final void setState(int newState) {
	state = newState;
}
}
