package designer.model;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.util.Enumeration;
import glyph.xml.*;

public final class XML {
	private static final String XCOORD = "x";
	private static final String YCOORD = "y";
	private static final String EXPANDED = "expanded";
	private static final String MODEL	 				= "MQeModel";
	private static final String HOST 					= "Host";
	private static final String QUEUEMANAGER	 		= "QueueManager";
	private static final String MQQUEUEMANAGER	 		= "MQQueueManager";
	private static final String CONNECTION 				= "Connection";
	private static final String ATTRIBUTE 				= "Attribute";
	private static final String CONNECTION_ALIAS 		= "ConnectionAlias";
	private static final String CONNECTION_ADAPTER		= "ConnectionAdapter";
	private static final String QUEUE_ALIAS 			= "QueueAlias";
	private static final String QUEUEMANAGER_ALIAS 		= "QueueManagerAlias";
	private static final String LOCAL_QUEUE 			= "LocalQueue";
	private static final String ADMIN_QUEUE 			= "AdminQueue";
	private static final String REMOTE_QUEUE 			= "RemoteQueue";
	private static final String HOMESERVER_QUEUE		= "HomeServerQueue";
	private static final String STOREANDFORWARD_QUEUE 	= "StoreAndForwardQueue";
	private static final String BRIDGE_QUEUE 			= "BridgeQueue";
	private static final String QUEUE	 				= "Queue";
	private static final String LISTENER 				= "Listener";
	private static final String QUEUEMANAGERENTRY		= "QueueManagerEntry";
	private static final String BRIDGE					= "Bridge";
	private static final String BRIDGEQUEUEMANAGER		= "BridgeQueueManager";
	private static final String BRIDGECONNECTION		= "BridgeConnection";
	private static final String BRIDGELISTENER			= "BridgeListener";
	private static final String MQLOCAL_QUEUE			= "MQLocalQueue";
	private static final String MQREMOTE_QUEUE			= "MQRemoteQueue";
	private static final String MQXMIT_QUEUE	 		= "MQXmitQueue";
	private static final String MQCHANNEL	 			= "MQChannel";

	
	private static final String QUEUEMANAGERS_CATEGORY 	= "QueueManagers";
	private static final String MQQUEUEMANAGERS_CATEGORY = "MQQueueManagers";
	private static final String QUEUES_CATEGORY		 	= "Queues";
	private static final String LISTENERS_CATEGORY		= "Listeners";
	private static final String CONNECTIONS_CATEGORY 	= "Connections";
	private static final String ATTRIBUTES_CATEGORY		= "Attributes";
	private static final String ALIASES_CATEGORY 		= "Aliases";
	private static final String ADAPTERS_CATEGORY 		= "Adapters";
	private static final String BRIDGES_CATEGORY 		= "Bridges";
	private static final String OLDBRIDGES_CATEGORY		= "BridgeList";
	private static final String QM_ENTRIES_CATEGORY		= "QueueManagers";
	private static final String CHANNELS_CATEGORY 		= "Channels";

	private static final String PURPOSE_ATTRIBUTE 		= "purpose";	// describes business purpose (currently only queues)
	
	private static final String NAME_ATTRIBUTE 			= "name";		// general purpose name of element
	private static final String QUEUEMANAGERNAME_ATTRIBUTE	= "queueManagerName";		// queue manager name of queue element
	//private static final String ADAPTER_ATTRIBUTE 		= "adapter";	// queue manager element adapter attribute
	//private static final String PORT_ATTRIBUTE 			= "port";		// queue manager element port attribute
	private static final String CHANNEL_ATTRIBUTE 		= "channel";	// connection element channel attribute
	private static final String VALUE_ATTRIBUTE 		= "value";		// attribute element value attribute

	private static final String LISTENER_PORT_ATTRIBUTE	= "port";		// listener element port attribute
	private static final String LISTENER_ADAPTER_ATTRIBUTE	= "adapter";		// listener element adapter attribute
	private static final String COLOUR_ATTRIBUTE = "colour";		// element colour
		
private static final void generateAttributeElements(Element parentElement, AttributedModel model, boolean includeCategories) {
	
	Enumeration enum = model.getAttributes().elements();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = parentElement;
		if (includeCategories) {
			parentOfChildren = new Element(parentElement, ATTRIBUTES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			Attribute attribute = (Attribute)enum.nextElement();
			Element attributeElement = new Element(parentOfChildren, ATTRIBUTE);
			String attName = attribute.getName();
			String attValue = attribute.getValue();
			if (null != attValue) {
				attributeElement.setAttribute(NAME_ATTRIBUTE, attName);
				attributeElement.setAttribute(VALUE_ATTRIBUTE, attValue);
			}
		}
	}
}
private static final void generateCoords(Element element, glyph.Node node) {
    if (!node.isAttached()) {
        element.setAttribute(XCOORD, node.getXcoord());
        element.setAttribute(YCOORD, node.getYcoord());
    }
    //element.setAttribute(COLOUR_ATTRIBUTE, Integer.toHexString(node.getBackgroundColor().getRGB()));
    if (node.isExpanded()) {
        element.setAttribute(EXPANDED, "true");
    } else {
        element.setAttribute(EXPANDED, "false");
    }
}
private static final Element generateElement(AbstractElement parentElement, Bridge model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, BRIDGE);
	generateCoords(modelElement, model);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	generateAttributeElements(modelElement, model, includeCategories);
	Enumeration enum = model.getBridgeQueueManagers();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		AbstractElement parentOfChildren = modelElement;
		if (includeCategories) {
			// not appicable to bridge as all bridge element are really categories
			//parentOfChildren = new Element(parentElement, ATTRIBUTES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			BridgeQueueManager childModel = (BridgeQueueManager)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, BridgeConnection model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, BRIDGECONNECTION);
	generateCoords(modelElement, model);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	generateAttributeElements(modelElement, model, includeCategories);
	Enumeration enum = model.getBridgeListeners();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		AbstractElement parentOfChildren = modelElement;
		if (includeCategories) {
			// not appicable to bridge as all bridge element are really categories
			//parentOfChildren = new Element(parentElement, ATTRIBUTES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			BridgeListener childModel = (BridgeListener)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, BridgeListener model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, BRIDGELISTENER);
	generateCoords(modelElement, model);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	generateAttributeElements(modelElement, model, includeCategories);
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, BridgeQueueManager model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, BRIDGEQUEUEMANAGER);
	generateCoords(modelElement, model);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	generateAttributeElements(modelElement, model, includeCategories);
	Enumeration enum = model.getBridgeConnections();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		AbstractElement parentOfChildren = modelElement;
		if (includeCategories) {
			// not appicable to bridge as all bridge element are really categories
			//parentOfChildren = new Element(parentElement, ATTRIBUTES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			BridgeConnection childModel = (BridgeConnection)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, Connection model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, CONNECTION);
	generateCoords(modelElement, model);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	generateAttributeElements(modelElement, model, includeCategories);
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, Host model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, HOST);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	generateCoords(modelElement, model);
	
	Enumeration enum = model.getQueueManagers();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, QUEUEMANAGERS_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			QueueManager childModel = (QueueManager)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}
	
	enum = model.getMQQueueManagers();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, MQQUEUEMANAGERS_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			MQQueueManager childModel = (MQQueueManager)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, Listener model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, LISTENER);
	generateCoords(modelElement, model);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	modelElement.setAttribute(LISTENER_PORT_ATTRIBUTE, model.getPort());
	modelElement.setAttribute(LISTENER_ADAPTER_ATTRIBUTE, model.getAdapter());
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, MQChannel model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, MQCHANNEL);
	generateCoords(modelElement, model);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	generateAttributeElements(modelElement, model, includeCategories);
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, MQeModel model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, MODEL);
	generateCoords(modelElement, model);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	Enumeration enum = model.getAllHosts();
	while (enum.hasMoreElements()) {
		Host host = (Host)enum.nextElement();
		Element childElement = generateElement(modelElement, host, includeCategories);
		modelElement.addChild(childElement);
	}
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, MQQueue model, boolean includeCategories) {

	String type = QUEUE;
	if (model.isMQLocalQueue()) {
		type = MQLOCAL_QUEUE;
	} else if (model.isMQRemoteQueue()) {
		type = MQREMOTE_QUEUE;
	} else if (model.isMQTransmissionQueue()) {
		type = MQXMIT_QUEUE;
	}
	Element modelElement = new Element(parentElement, type);
	generateCoords(modelElement, model);
	
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	//modelElement.setAttribute(QUEUEMANAGERNAME_ATTRIBUTE, model.getQueueQueueManagerName());
	generateAttributeElements(modelElement, model, includeCategories);
	
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, MQQueueManager model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, MQQUEUEMANAGER);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	generateCoords(modelElement, model);
	Enumeration enum = model.getQueues();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, QUEUES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			MQQueue childModel = (MQQueue)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}

	enum = model.getChannels();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, CHANNELS_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			MQChannel childModel = (MQChannel)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}

	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, Queue model, boolean includeCategories) {
	String type = QUEUE;
	if (model.isAdminQueue()) {
		type = ADMIN_QUEUE;
	} else if (model.isLocalQueue()) {
		type = LOCAL_QUEUE;
	} else if (model.isRemoteQueue()) {
		type = REMOTE_QUEUE;
	} else if (model.isHomeServerQueue()) {
		type = HOMESERVER_QUEUE;
	} else if (model.isStoreAndForwardQueue()) {
		type = STOREANDFORWARD_QUEUE;
	} else if (model.isBridgeQueue()) {
		type = BRIDGE_QUEUE;
	}
	Element modelElement = new Element(parentElement, type);
	generateCoords(modelElement, model);
	
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	modelElement.setAttribute(QUEUEMANAGERNAME_ATTRIBUTE, model.getQueueQueueManagerName());
	String purpose = model.getPurpose();
	if (null != purpose) {
		modelElement.setAttribute(PURPOSE_ATTRIBUTE, purpose);
	}
	generateAttributeElements(modelElement, model, includeCategories);
	generateQueueAliasElements(modelElement, model, includeCategories);

	if (model.isStoreAndForwardQueue()) {
		generateQueueManagerEntryElements(modelElement, 
				(StoreAndForwardQueue)model,
				includeCategories);
	}
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, QueueAlias model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, QUEUE_ALIAS);
	generateCoords(modelElement, model);
	
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, QueueManager model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, QUEUEMANAGER);
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());

	generateAttributeElements(modelElement, model, includeCategories);
	generateCoords(modelElement, model);
	
	Enumeration enum = model.getQueues().elements();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, QUEUES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			Queue childModel = (Queue)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}
	enum = model.getListeners();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, LISTENERS_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			Listener childModel = (Listener)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}

	enum = model.getConnections().elements();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, CONNECTIONS_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			Connection childModel = (Connection)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}
	enum = model.getBridges();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		Element parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, BRIDGES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			Bridge childModel = (Bridge)enum.nextElement();
			Element childElement = generateElement(parentOfChildren, childModel, includeCategories);
		}
	}
	
	enum = model.getAliases();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		AbstractElement parentOfChildren = modelElement;
		if (includeCategories) {
			parentOfChildren = new Element(modelElement, ALIASES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			QueueManagerAlias alias = (QueueManagerAlias)enum.nextElement();
			Element aliasElement = generateElement(parentOfChildren, alias, includeCategories);
		}
	}
	

	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, QueueManagerAlias model, boolean includeCategories) {
	Element modelElement = new Element(parentElement, QUEUEMANAGER_ALIAS);
	generateCoords(modelElement, model);
	
	modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
	modelElement.setAttribute(VALUE_ATTRIBUTE, model.getValue());
	return modelElement;
}
private static final Element generateElement(AbstractElement parentElement, QueueManagerEntry model, boolean includeCategories) {
    Element modelElement = new Element(parentElement, QUEUEMANAGERENTRY);
    generateCoords(modelElement, model);
    modelElement.setAttribute(NAME_ATTRIBUTE, model.getName());
    return modelElement;
}
public static final String GenerateFrom(Model model) {
	return GenerateFrom(model, false);
}
public static final String GenerateFrom(Model model, boolean includeCategories) {
	String xml = null;
	Document document = new Document();
	if (model.isHost()) {
		generateElement(document, (Host)model, includeCategories);
	} else if (model.isMQeModel()) {
		generateElement(document, (MQeModel)model, includeCategories);
	} else if (model.isQueueManager()) {
		generateElement(document, (QueueManager)model, includeCategories);
	} else if (model.isQueueManagerEntry()) {
		generateElement(document, (QueueManagerEntry)model, includeCategories);
	} else if (model.isQueue()) {
		generateElement(document, (Queue)model, includeCategories);
	} else if (model.isQueueAlias()) {
		generateElement(document, (QueueAlias)model, includeCategories);
	} else if (model.isQueueManagerAlias()) {
		generateElement(document, (QueueManagerAlias)model, includeCategories);
	} else if (model.isConnection()) {
		generateElement(document, (Connection)model, includeCategories);
	} else if (model.isBridge()) {
		generateElement(document, (Bridge)model, includeCategories);
	} else if (model.isBridgeQueueManager()) {
		generateElement(document, (BridgeQueueManager)model, includeCategories);
	} else if (model.isBridgeConnection()) {
		generateElement(document, (BridgeConnection)model, includeCategories);
	} else if (model.isBridgeListener()) {
		generateElement(document, (BridgeListener)model, includeCategories);
	} else if (model.isMQQueueManager()) {
		generateElement(document, (MQQueueManager)model, includeCategories);
	} else if (model.isMQLocalQueue()) {
		generateElement(document, (MQLocalQueue)model, includeCategories);
	} else if (model.isMQRemoteQueue()) {
		generateElement(document, (MQRemoteQueue)model, includeCategories);
	} else if (model.isMQTransmissionQueue()) {
		generateElement(document, (MQXmitQueue)model, includeCategories);
	} else if (model.isMQChannel()) {
		generateElement(document, (MQChannel)model, includeCategories);
	} else {
		return "unknown element";
	}
	xml = Unparser.ToString(document);
	return xml;
}
private static final void generateQueueAliasElements(Element parentElement, Queue model, boolean includeCategories) {
	Enumeration enum = model.getAliases().elements();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		AbstractElement parentOfChildren = parentElement;
		if (includeCategories) {
			parentOfChildren = new Element(parentElement, ALIASES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			QueueAlias alias = (QueueAlias)enum.nextElement();
			Element aliasElement = generateElement(parentOfChildren, alias, includeCategories);
			generateCoords(aliasElement, alias);
		}
	}
}
private static final void generateQueueManagerEntryElements(Element parentElement, StoreAndForwardQueue model, boolean includeCategories) {
	Enumeration enum = model.getQueueManagerEntries().elements();
	if (enum.hasMoreElements()) {
		// we can create a category for each group of children
		// and make this switchable
		AbstractElement parentOfChildren = parentElement;
		if (includeCategories) {
			parentOfChildren = new Element(parentElement, QM_ENTRIES_CATEGORY);
		}
		while (enum.hasMoreElements()) {
			QueueManagerEntry qme = (QueueManagerEntry)enum.nextElement();
			Element aliasElement = generateElement(parentOfChildren, qme, includeCategories);
			generateCoords(aliasElement, qme);
		}
	}
}
private static final void parseAdminQueueElement(QueueManager queueManager, Element queueElement) {
	String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
	Enumeration enum = null;

	AdminQueue queue = queueManager.createAdminQueue(queueName);
	if (null != queue) {
		parseModelAttributes(queue, queueElement);
		enum = queueElement.getChildren();
		while (enum.hasMoreElements()) {
			Element element = (Element) enum.nextElement();
			parseQueueChildElement(queue, element);
		}
	}
}
private static final void parseBridgeConnectionElement(BridgeQueueManager bridgeQM, Element element) {
	String name = element.getAttribute(NAME_ATTRIBUTE);
	BridgeConnection model = bridgeQM.newBridgeConnection(name);
	if (null != model) {
		parseModelAttributes(model, element);
		Enumeration enum = element.getChildren();
		while (enum.hasMoreElements()) {
			Element el = (Element) enum.nextElement();
			String elementName = el.getNodeName();
			if (BRIDGELISTENER.equalsIgnoreCase(elementName)) {
				parseBridgeListenerElement(model, el);
			}
		}
	}
}
private static final void parseBridgeElement(QueueManager queueManager, Element element) {
	String name = element.getAttribute(NAME_ATTRIBUTE);
	Bridge model = queueManager.getBridge(name);
	if (null == model) {
		model = queueManager.createBridge(name);
	}
	if (null != model) {
		parseModelAttributes(model, element);
		Enumeration enum = element.getChildren();
		while (enum.hasMoreElements()) {
			Element el = (Element) enum.nextElement();
			String elementName = el.getNodeName();
			if (BRIDGEQUEUEMANAGER.equalsIgnoreCase(elementName)) {
				parseBridgeQMElement(model, el);
			}
		}
	}
}
private static final void parseBridgeListElement(QueueManager queueManager, Element element) {
	Enumeration enum = element.getChildren();
	while (enum.hasMoreElements()) {
		Element el = (Element) enum.nextElement();
		String elementName = el.getNodeName();
		if (BRIDGE.equalsIgnoreCase(elementName)) {
			parseBridgeElement(queueManager, el);
		}
	}
}
private static final void parseBridgeListenerElement(BridgeConnection connection, Element element) {
    String name = element.getAttribute(NAME_ATTRIBUTE);
    BridgeListener model = connection.newBridgeListener(name);
    parseModelAttributes(model, element);
}
/**
 * Parse a child element of a queue element.  This allows us to
 * to cope with optional categories for aliases.
 *
 */
private static final void parseBridgeQMChildElement(BridgeQueueManager bqm, Element element) {
	String elementName = element.getNodeName();
	if (ATTRIBUTE.equalsIgnoreCase(elementName)) {
		String attrValue = element.getAttribute(VALUE_ATTRIBUTE);
		String attrName = element.getAttribute(NAME_ATTRIBUTE);
		bqm.setAttribute(attrName, attrValue);
	} else if ("category".equalsIgnoreCase(elementName)
				|| ATTRIBUTES_CATEGORY.equalsIgnoreCase(elementName)) {
		Enumeration enum = element.getChildren();
		while (enum.hasMoreElements()) {
			Element childElement = (Element) enum.nextElement();
			parseBridgeQMChildElement(bqm, childElement);
		 }
	}
}
private static final void parseBridgeQMElement(Bridge bridge, Element element) {
	String name = element.getAttribute(NAME_ATTRIBUTE);
	BridgeQueueManager model = bridge.newBridgeQueueManager(name);
	if (null != model) {
		parseModelAttributes(model, element);
		Enumeration enum = element.getChildren();
		while (enum.hasMoreElements()) {
			Element el = (Element) enum.nextElement();
			String elementName = el.getNodeName();
			if ("BridgeConnection".equalsIgnoreCase(elementName)) {
				parseBridgeConnectionElement(model, el);
			} else {
				parseBridgeQMChildElement(model, el);
			}
		}
	}
}
private static final void parseBridgeQueueElement(QueueManager queueManager, Element queueElement) {
    String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);

    String queueManagerName = queueElement.getAttribute(QUEUEMANAGERNAME_ATTRIBUTE);
    String bridgeName = queueElement.getAttribute("bridgeName");
    String connectionName = queueElement.getAttribute("connectionName");

    BridgeQueue queue = queueManager.createBridgeQueue(queueManagerName, queueName, bridgeName, connectionName);

    if (null != queue) {
        String purpose = queueElement.getAttribute(PURPOSE_ATTRIBUTE);
        queue.setPurpose(purpose);
        parseModelAttributes(queue, queueElement);
        Enumeration enum = queueElement.getChildren();
        while (enum.hasMoreElements()) {
            Element element = (Element) enum.nextElement();
            parseQueueChildElement(queue, element);
        }
    }
}
/**
 * Parse a child element of a connection element.  This allows us to
 * to cope with optional categories for aliases and adapters.
 *
 * The element passed is the child of the connection element that created
 * the connection.
 */
private static final void parseConnectionChildElement(Connection connection, Element element) {
	String elementName = element.getNodeName();
	if ("category".equalsIgnoreCase(elementName)
			|| ATTRIBUTES_CATEGORY.equalsIgnoreCase(elementName)) {
	    Enumeration enum = element.getChildren();
	    while (enum.hasMoreElements()) {
	    	Element childElement = (Element)enum.nextElement();
	    	parseConnectionChildElement(connection, childElement);
	    }
	}
}
private static final void parseConnectionElement(QueueManager queueManager, Element queueElement) {
	String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
	String channel = queueElement.getAttribute(CHANNEL_ATTRIBUTE);;
	Connection connection = queueManager.createConnectionDefinition(queueName, null, channel);
	if (null != connection) {
		parseModelAttributes(connection, queueElement);
		Enumeration enum = queueElement.getChildren();
		while (enum.hasMoreElements()) {
			Element element = (Element) enum.nextElement();
			parseConnectionChildElement(connection, element);
		}
	}
}
private static final void ParseDocumentInto(Document doc, MQeModel model) {
	Enumeration enum = doc.getDocumentElement().getChildren();
	while (enum.hasMoreElements()) {
		Element element = (Element)enum.nextElement();
		parseHostElement(model, element);
	}
}
public static final void ParseFileInto(String filename, MQeModel model) {
	Document doc = Parser.ParseFile(filename);
	ParseDocumentInto(doc, model);
}
private static final void parseHomeServerQueueElement(QueueManager queueManager, Element queueElement) {
    String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
    String queueManagerName = queueElement.getAttribute(QUEUEMANAGERNAME_ATTRIBUTE);

    HomeServerQueue queue = queueManager.createHomeServerQueue(queueManagerName, queueName);
    if (null != queue) {
        String purpose = queueElement.getAttribute(PURPOSE_ATTRIBUTE);
        queue.setPurpose(purpose);
        parseModelAttributes(queue, queueElement);
        Enumeration enum = queueElement.getChildren();
        while (enum.hasMoreElements()) {
            Element element = (Element) enum.nextElement();
            // hsq should not need child parsing as it should not have aliases
            //parseQueueChildElement(queue, element);
        }
    }
}
private static final void parseHostChildElement(Host host, Element element) {
	String elementName = element.getNodeName();
	if (QUEUEMANAGER.equalsIgnoreCase(elementName)) {
		parseQueueManagerElement(host, element);
	} else if (MQQUEUEMANAGER.equalsIgnoreCase(elementName)) {
		parseMQQueueManagerElement(host, element);
	} else if (MQQUEUEMANAGERS_CATEGORY.equalsIgnoreCase(elementName)) {
		parseHostChildElement(host, element);
	} else if (MQQUEUEMANAGERS_CATEGORY.equalsIgnoreCase(elementName)) {
		parseHostChildElement(host, element);
	}
}
private static final void parseHostElement(MQeModel model, Element hostElement) {
    String hostName = hostElement.getAttribute(NAME_ATTRIBUTE);
    Host host = model.getOrCreateHost(hostName);
    parseModelCoordinates(host, hostElement);
//    String xStr = hostElement.getAttribute(XCOORD);
//   String yStr = hostElement.getAttribute(YCOORD);
//    if (null != xStr && null != yStr) {
//        int x = Integer.parseInt(xStr);
//        int y = Integer.parseInt(yStr);
//        host.setCoordinates(x, y);
//    }
    if (null != host) {
        Enumeration enum = hostElement.getChildren();
        while (enum.hasMoreElements()) {
            Element element = (Element) enum.nextElement();
            parseHostChildElement(host, element);
        }
    }
}
private static final void parseListenerElement(QueueManager queueManager, Element listenerElement) {
	String listenerName = listenerElement.getAttribute(NAME_ATTRIBUTE);
	String adapter = listenerElement.getAttribute(LISTENER_ADAPTER_ATTRIBUTE);;
	String port = listenerElement.getAttribute(LISTENER_PORT_ATTRIBUTE);;
	
	Listener listener = queueManager.createListener(listenerName, port, adapter);
	parseModelAttributes(listener, listenerElement);
		
}
private static final void parseLocalQueueElement(QueueManager queueManager, Element queueElement) {
	String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
	Enumeration enum = null;

	LocalQueue queue = queueManager.createLocalQueue(queueName);
	if (null != queue) {
		String purpose = queueElement.getAttribute(PURPOSE_ATTRIBUTE);
		queue.setPurpose(purpose);
		parseModelAttributes(queue, queueElement);
		enum = queueElement.getChildren();
		while (enum.hasMoreElements()) {
			Element element = (Element) enum.nextElement();
			parseQueueChildElement(queue, element);
		}
	}
}
private static final void parseModelAttributes(AttributedModel model, Element element) {
	parseModelCoordinates(model, element);
	
	Enumeration enum = element.getChildren();
    while (enum.hasMoreElements()) {
        Element childElement = (Element) enum.nextElement();
        String childName = childElement.getNodeName();
        if (ATTRIBUTE.equalsIgnoreCase(childName)) {
            // attribute may be attached direct, or nested in a category
            String attrValue = childElement.getAttribute(VALUE_ATTRIBUTE);
            String attrName = childElement.getAttribute(NAME_ATTRIBUTE);
            model.setAttribute(attrName, attrValue);
        } else if (ATTRIBUTES_CATEGORY.equalsIgnoreCase(childName)) {
            // attribute may be attached direct, or nested in a category
            parseModelAttributes(model, element);
        }
    }
}
private static final void parseModelCoordinates(Model model, Element element) {
	String xStr = element.getAttribute(XCOORD);
	String yStr = element.getAttribute(YCOORD);
	if (null != xStr && null != yStr) {
		int x = Integer.parseInt(xStr);
		int y = Integer.parseInt(yStr);
		model.setCoordinates(x,y);
	}
	String expandedStr = element.getAttribute(EXPANDED);
	boolean expanded = "true".equalsIgnoreCase(expandedStr);
	model.setExpanded(expanded);

	//String colourString = element.getAttribute(COLOUR_ATTRIBUTE);
	//if (null != colourString) {
	//	int c = (int)Long.parseLong(colourString, 16);
	//	model.setBackgroundColor(new java.awt.Color(c));
	//}
}
private static final void parseMQChannelElement(MQQueueManager queueManager, Element queueElement) {
	String name = queueElement.getAttribute(NAME_ATTRIBUTE);
	
	MQChannel channel = queueManager.createChannel(name);
	parseModelAttributes(channel, queueElement);
	
//	if (null != queue) {
//		Enumeration enum = queueElement.getChildren();
//		while (enum.hasMoreElements()) {
//			Element element = (Element) enum.nextElement();
//			parseQueueChildElement(queue, element);
//		}
//	}
}
private static final void parseMQLocalQueueElement(MQQueueManager queueManager, Element queueElement) {
	String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
	Enumeration enum = null;

	MQLocalQueue queue = queueManager.createLocalQueue(queueName);
	parseModelAttributes(queue, queueElement);
	
}
/**
 * Parse a child element of a queue manager element.  This allows us to
 * to cope with optional categories for queue and connections.
 *
 * The element passed is the child of the queue manager element that created
 * the queue manager.
 */
private static final void parseMQQueueManagerChildElement(MQQueueManager queueManager, Element element) {
	String elementName = element.getNodeName();
	String modelName = element.getAttribute("name");
	if (MQLOCAL_QUEUE.equalsIgnoreCase(elementName)) {
		parseMQLocalQueueElement(queueManager, element);
	} else if (MQREMOTE_QUEUE.equalsIgnoreCase(elementName)) {
		parseMQRemoteQueueElement(queueManager, element);
	} else if (MQXMIT_QUEUE.equalsIgnoreCase(elementName)) {
		parseMQXmitQueueElement(queueManager, element);
	} else if (MQCHANNEL.equalsIgnoreCase(elementName)) {
		parseMQChannelElement(queueManager, element);
	} else if ("category".equalsIgnoreCase(elementName)
		// skip over any recognised categories and
		// parse the children as if they were children of the
		// categorys parent
			|| CHANNELS_CATEGORY.equalsIgnoreCase(elementName)
			|| QUEUES_CATEGORY.equalsIgnoreCase(elementName)) {
	    Enumeration enum = element.getChildren();
	    while (enum.hasMoreElements()) {
	    	Element childElement = (Element)enum.nextElement();
	    	parseMQQueueManagerChildElement(queueManager, childElement);
	    }
	}
}
private static final void parseMQQueueManagerElement(Host host, Element queueManagerElement) {
	String queueManagerName = queueManagerElement.getAttribute("name");
	MQQueueManager queueManager = host.getOrCreateMQQueueManager(queueManagerName);
	parseModelAttributes(queueManager, queueManagerElement);
	if (null != queueManager) {
		Enumeration enum = queueManagerElement.getChildren();
		while (enum.hasMoreElements()) {
			Element element = (Element) enum.nextElement();
			parseMQQueueManagerChildElement(queueManager, element);
		}
	}
}
private static final void parseMQRemoteQueueElement(MQQueueManager queueManager, Element queueElement) {
	String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
	String queueManagerName= queueElement.getAttribute(QUEUEMANAGERNAME_ATTRIBUTE);

	MQRemoteQueue queue = queueManager.createRemoteQueue(queueName);
	parseModelAttributes(queue, queueElement);
}
private static final void parseMQXmitQueueElement(MQQueueManager queueManager, Element queueElement) {
	String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
	String queueManagerName= queueElement.getAttribute(QUEUEMANAGERNAME_ATTRIBUTE);

	MQXmitQueue queue = queueManager.createTransmissionQueue(queueName);
	parseModelAttributes(queue, queueElement);
}
/**
 * Parse a child element of a queue element.  This allows us to
 * to cope with optional categories for aliases.
 *
 */
private static final void parseQueueChildElement(Queue queue, Element element) {
	String elementName = element.getNodeName();
	if (QUEUE_ALIAS.equals(elementName)) {
		QueueAlias alias = queue.addAlias(element.getAttribute(NAME_ATTRIBUTE));
		parseModelAttributes(alias, element);
	} else if (ATTRIBUTE.equalsIgnoreCase(elementName)) {
		String attrValue = element.getAttribute(VALUE_ATTRIBUTE);
		String attrName = element.getAttribute(NAME_ATTRIBUTE);
		queue.setAttribute(attrName, attrValue);
	} else if ("category".equalsIgnoreCase(elementName)
				|| ATTRIBUTES_CATEGORY.equalsIgnoreCase(elementName)
				|| ALIASES_CATEGORY.equalsIgnoreCase(elementName)) {
		Enumeration enum = element.getChildren();
		while (enum.hasMoreElements()) {
			Element childElement = (Element) enum.nextElement();
			parseQueueChildElement(queue, childElement);
		 }
	}
}
/**
 * Parse a child element of a queue manager element.  This allows us to
 * to cope with optional categories for queue and connections.
 *
 * The element passed is the child of the queue manager element that created
 * the queue manager.
 */
private static final void parseQueueManagerChildElement(QueueManager queueManager, Element element) {
	String elementName = element.getNodeName();
	String modelName = element.getAttribute("name");
	if (LOCAL_QUEUE.equalsIgnoreCase(elementName)) {
		parseLocalQueueElement(queueManager, element);
	} else if (ADMIN_QUEUE.equalsIgnoreCase(elementName)) {
		parseAdminQueueElement(queueManager, element);
	} else if (REMOTE_QUEUE.equalsIgnoreCase(elementName)) {
		parseRemoteQueueElement(queueManager, element);
	} else if (STOREANDFORWARD_QUEUE.equalsIgnoreCase(elementName)) {
		parseStoreAndForwardQueueElement(queueManager, element);
	} else if (HOMESERVER_QUEUE.equalsIgnoreCase(elementName)) {
		parseHomeServerQueueElement(queueManager, element);
	} else if (BRIDGE_QUEUE.equalsIgnoreCase(elementName)) {
		parseBridgeQueueElement(queueManager, element);
	} else if (CONNECTION.equalsIgnoreCase(elementName)) {
		parseConnectionElement(queueManager, element);
	} else if (LISTENER.equalsIgnoreCase(elementName)) {
		parseListenerElement(queueManager, element);
	} else if (BRIDGE.equalsIgnoreCase(elementName)) {
		parseBridgeElement(queueManager, element);
	} else if (ATTRIBUTE.equalsIgnoreCase(elementName)) {
		String attrValue = element.getAttribute(VALUE_ATTRIBUTE);
		queueManager.setAttribute(modelName, attrValue);
	} else if (QUEUEMANAGER_ALIAS.equalsIgnoreCase(elementName)) {
		String attrValue = element.getAttribute(VALUE_ATTRIBUTE);
		QueueManagerAlias qma = queueManager.createAlias(modelName, attrValue);
		parseModelCoordinates(qma, element);
	} else if ("category".equalsIgnoreCase(elementName)
		// skip over any recognised categories and
		// parse the children as if they were children of the
		// categorys parent
			|| ATTRIBUTES_CATEGORY.equalsIgnoreCase(elementName)
			|| ALIASES_CATEGORY.equalsIgnoreCase(elementName)
			|| BRIDGES_CATEGORY.equalsIgnoreCase(elementName)
			|| OLDBRIDGES_CATEGORY.equalsIgnoreCase(elementName)
			|| QUEUES_CATEGORY.equalsIgnoreCase(elementName)
			|| LISTENERS_CATEGORY.equalsIgnoreCase(elementName)
			|| CONNECTIONS_CATEGORY.equalsIgnoreCase(elementName)) {
	    Enumeration enum = element.getChildren();
	    while (enum.hasMoreElements()) {
	    	Element childElement = (Element)enum.nextElement();
	    	parseQueueManagerChildElement(queueManager, childElement);
	    }
	}
}
private static final void parseQueueManagerElement(Host host, Element queueManagerElement) {
    String queueManagerName = queueManagerElement.getAttribute(NAME_ATTRIBUTE);

    // when the listener handles these, we will not need to read them in......
    //String adapter = queueManagerElement.getAttribute(ADAPTER_ATTRIBUTE);
    //String port = queueManagerElement.getAttribute(PORT_ATTRIBUTE);
    //String channel = queueManagerElement.getAttribute(CHANNEL_ATTRIBUTE);
    QueueManager queueManager = host.getOrCreateQueueManager(queueManagerName);
    parseModelAttributes(queueManager, queueManagerElement);
    if (null != queueManager) {
        //if (null != port && null != adapter) {
        //    // if there is a port we are using old-style xml
        //    queueManager.createListener("DefaultListener", port, adapter);
        //}
        Enumeration enum = queueManagerElement.getChildren();
        while (enum.hasMoreElements()) {
            Element element = (Element) enum.nextElement();
            parseQueueManagerChildElement(queueManager, element);
        }
    }
}
private static final void parseRemoteQueueElement(QueueManager queueManager, Element queueElement) {
    String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
    String queueManagerName = queueElement.getAttribute(QUEUEMANAGERNAME_ATTRIBUTE);

    RemoteQueue queue = queueManager.createRemoteQueue(queueManagerName, queueName);
    if (null != queue) {
        String purpose = queueElement.getAttribute(PURPOSE_ATTRIBUTE);
        queue.setPurpose(purpose);
        parseModelAttributes(queue, queueElement);
        Enumeration enum = queueElement.getChildren();
        while (enum.hasMoreElements()) {
            Element element = (Element) enum.nextElement();
            parseQueueChildElement(queue, element);
        }
    }
}
/**
 * Parse a child element of a queue element.  This allows us to
 * to cope with optional categories for aliases and qmes.
 *
 */
private static final void parseStoreAndForwardQueueChildElement(StoreAndForwardQueue queue, Element element) {
	String elementName = element.getNodeName();
	if (QUEUE_ALIAS.equals(elementName)) {
		queue.addAlias(element.getAttribute(NAME_ATTRIBUTE));
	} else if (QUEUEMANAGERENTRY.equals(elementName)) {
		QueueManagerEntry qme = queue.addQueueManagerEntry(element.getAttribute(NAME_ATTRIBUTE));
		parseModelCoordinates(qme, element);
	} else if (ATTRIBUTE.equalsIgnoreCase(elementName)) {
		String attrValue = element.getAttribute(VALUE_ATTRIBUTE);
		String attrName = element.getAttribute(NAME_ATTRIBUTE);
		queue.setAttribute(attrName, attrValue);
	} else if ("category".equalsIgnoreCase(elementName)
		|| ALIASES_CATEGORY.equalsIgnoreCase(elementName)
		|| ATTRIBUTES_CATEGORY.equalsIgnoreCase(elementName)
		|| QM_ENTRIES_CATEGORY.equalsIgnoreCase(elementName)) {
		Enumeration enum = element.getChildren();
		while (enum.hasMoreElements()) {
			Element childElement = (Element) enum.nextElement();
			parseStoreAndForwardQueueChildElement(queue, childElement);
		}
	}
}
private static final void parseStoreAndForwardQueueElement(QueueManager queueManager, Element queueElement) {
    String queueName = queueElement.getAttribute(NAME_ATTRIBUTE);
    String queueManagerName = queueElement.getAttribute(QUEUEMANAGERNAME_ATTRIBUTE);

    StoreAndForwardQueue queue = queueManager.createStoreAndForwardQueue(queueManagerName, queueName);
    if (null != queue) {
        String purpose = queueElement.getAttribute(PURPOSE_ATTRIBUTE);
        queue.setPurpose(purpose);
        parseModelAttributes(queue, queueElement);
        Enumeration enum = queueElement.getChildren();
        while (enum.hasMoreElements()) {
            Element element = (Element) enum.nextElement();
            parseStoreAndForwardQueueChildElement(queue, element);
        }
    }
}
}
