package glyph;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.*;
import java.awt.event.*;
/**
 * This class represents the bidirectional interface between a view and
 * a line between two model objects being represented in the view.
 *
 * implements action listener to facilitate menu creation
 * Creation date: (08/02/2002 16:30:30)
 * @author: 
 */
public abstract class Line implements ActionListener {
	// constants used for describing lines
	public static final int SYMBOL_NONE 			= 0;
	public static final int SYMBOL_ARROW 			= 1;
	public static final int SYMBOL_HOLLOW_ARROW 	= 2;
	public static final int SYMBOL_DIAMOND 			= 3;
	public static final int SYMBOL_HOLLOW_DIAMOND 	= 4;
	public static final int SYMBOL_BLUNT_ARROW 			= 5;
	public static final int SYMBOL_HOLLOW_BLUNT_ARROW 	= 6;
	
	/**
	 * by hard coding a matching glyph we ensure that there
	 * is always a valid value.
	 */
	private LineGlyph glyph;

public Line(Node src, Node dst) {
	super();
	glyph = new LineGlyph(this, src._getGlyph(), dst._getGlyph());
}
/**
 * package private access to glyph
 */
final LineGlyph _getGlyph() {
	return glyph;
}
public void actionPerformed(ActionEvent event) {
}
/**
 * declare that the receiver has changed in a way that requires
 * a redraw
 * This could be a header or body change.  This call is not required
 * for an addNode() or a delete().
 */
protected final void declareChanged() {
	glyph._changed();
}
/**
 * delete this node if not already deleted
 */
protected final void delete() {
	//glyph.delete();
}
/**
 * delete this node if not already deleted
 */
public final void detach() {
	glyph.detach();
}
/**
 * reply the body - the bit of text above the line in the box
 * reply null (or empy array) to show no body.
 * Each string in the array will apear on a separate line
 */
protected abstract String[] getBody();
/**
 * reply the receivers destination node may be null if detached)
 */
public final Node getDestinationNode() {
	Node n = null;
	AreaGlyph g = glyph.getDestinationGlyph();
	if (null != g) {
		n = g.getNode();
	}
	return n;
}
/**
 * reply he symbol for the destination end of he line 
 * override to change symbol
 */
protected int getDestinationSymbol() {
	return SYMBOL_NONE;
}
/**
 * reply the header - the bit of text above the line in the box
 * reply null to show no header
 */
protected abstract String getHeader();
	/**
	 * reply importance of line.  Higher importance lines
	 * mask lower importance lines.  Default is 10.
	 */
	public int getImportance() {
		return 10;
	}
/**
 * reply background colour for glyph
 * Subclasses can overide to change this
 */
protected Color getLineColor() {
	return Color.black;
}
/**
 * reply the menu 
 * reply null for no menu
 */
protected abstract PopupMenu getMenu();
/**
 * reply the receivers source node may be null if detached)
 */
public final Node getSourceNode() {
	Node n = null;
	AreaGlyph g = glyph.getSourceGlyph();
	if (null != g) {
		n = g.getNode();
	}
	return n;
}
/**
 * reply he symbol for the source end of he line 
 * override to change symbol
 */
protected int getSourceSymbol() {
	return SYMBOL_NONE;
}
protected final View getView() {
	return glyph.getView();
}
/**
 * reply true if the receiver is still attached.  If the receiver
 * replies false then it will not be drawable.
 */
public final boolean isAttached() {
	return glyph.isAttached();
}
/**
 * reply true if the receiver is visible
 * use to control drawing
 */
protected boolean isVisible() {
	return true;
}
}
