package glyph;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.*;
import java.awt.event.*;
import glyph.dialogues.*;
import java.util.Vector;

public class View extends Container 
implements MouseListener, MouseMotionListener, ComponentListener {
//////////////////////////////////
	private AreaGlyph currentGlyph = null;

	/**
	 * Always remember the point at which the mouse was pressed, as
	 * this will be the point at which a drag should start.  But we
	 * wont know until we get a mouseDragged message later.
	 */
	private Point previousPoint = null;
	
	/**
	 * keep track of whether we are dragging, so a mouseReleased event
	 * can trigger a drag end.
	 */
	private boolean dragging = false;
	
	private final Vector glyphs = new Vector();
	
	private final Vector detachedChildren = new Vector();

	private final Vector lines = new Vector();

	// cache this so we dont need to keep constructing one
	private final Rectangle bounds = new Rectangle();

	private Image buffer = null;

	private final Label info = new Label("Information");;
	private Frame frame = null;
	private GlyphHelpDialogue helpDlg = null;

	// setting this to false may help display on slow projectors,
	// but will mess up saving as a bitmap......  Also makes the 
	// display flicker while dragging
	public static boolean doubleBuffered = true;
/**
 * View constructor comment.
 */
public View() {
    super();
    setLayout(new BorderLayout());
    addMouseListener(this);
    addMouseMotionListener(this);
    addComponentListener(this);
    Font f = new Font("Helvetica", Font.PLAIN, 12);
    setFont(f);
    
    info.setBackground(Color.lightGray);
    add(BorderLayout.SOUTH, info);
    doLayout();
}
/**
 * Overide the paint method from component; final so subclasses
 * cannot interfere
 */
private final void __calculate(Graphics graphics) {
    __recalculateDetachedChildren();
    bounds.setBounds(0, 0, 0, 0);
    for (int i = 0; i < detachedChildren.size(); i++) {
        AreaGlyph g = (AreaGlyph) detachedChildren.elementAt(i);
        g._calculate(graphics, bounds);
    }
    
    // find out which lines we can suppress.  This depends upon the current glyphs
    // betwen which it will be drawn. This may not be the end glyphs, as some lines survive
    // the attachment of their src or dst to the parent.  Thus this calculation happens each 
    // time attachment changes/glyph added/glyph removed/routing options changed.  We will do
    // this every time to see what it costs....
    // Algorithm is brute force - compare every pair of lines
    for (int i = 0; i < lines.size(); i++) {
        // unmask all lines
         ((LineGlyph) lines.elementAt(i)).masked = false;
    }
    
    for (int outer = 0; outer < lines.size() - 1; outer++) {
        LineGlyph testLine = (LineGlyph) lines.elementAt(outer);
        AreaGlyph testSrc = testLine.getSourceGlyph().highestAttachedGlyph();
        AreaGlyph testDst = testLine.getDestinationGlyph().highestAttachedGlyph();
        int testImportance = testLine.getImportance();
        if (testLine.isVisible()) {
            // only visible lines count 
            for (int inner = 1; inner < lines.size(); inner++) {
                LineGlyph compareLine = (LineGlyph) lines.elementAt(inner);
                if (inner != outer && compareLine.isVisible()) {
                    AreaGlyph compareSrc = compareLine.getSourceGlyph().highestAttachedGlyph();
                    AreaGlyph compareDst = compareLine.getDestinationGlyph().highestAttachedGlyph();
                    int compareImportance = compareLine.getImportance();
                    if ((compareSrc == testSrc && compareDst == testDst) || (compareSrc == testDst && compareDst == testSrc)) {
                        // same dest and src
                        if (compareImportance > testImportance) {
                            testLine.masked = true;
                        } else if (compareImportance < testImportance) {
                            compareLine.masked = true;
                        }
                    }
                }
            }
        }
    }
    
    for (int i = 0; i < lines.size(); i++) {
        LineGlyph line = (LineGlyph) lines.elementAt(i);
        if (!line.masked && line.isVisible()) {
            line._calculate(graphics, bounds);
        }
    }
}
final void __deselectAll() {
	for (int i = 0; i < glyphs.size(); i++){
		AreaGlyph glyph = (AreaGlyph)glyphs.elementAt(i);
		glyph.deselectAll();
	}
}
/**
 * Invoked when the mouse has been clicked on a component.
 */
private final AreaGlyph __getGlyphAt(Point pt) {
	AreaGlyph found = null;
	for (int i = 0; i < glyphs.size(); i++) {
		AreaGlyph glyph = (AreaGlyph) glyphs.elementAt(i);
		AreaGlyph g = glyph.getGlyph(pt);
		if (null != g) {
	        found = g;
		}
	}
	return found;
}
/**
 * convert mouse event into button .
 */
private final boolean __isLeftButton(MouseEvent e) {
	return ((e.getModifiers() & e.BUTTON1_MASK) == e.BUTTON1_MASK);
}
/**
 * convert mouse event into button .
 */
private final boolean __isPopupTrigger(MouseEvent e) {
	return ((e.getModifiers() & e.BUTTON3_MASK) == e.BUTTON3_MASK) || e.isPopupTrigger();
}
private final void __recalculateDetachedChildren() {
	// recalculate the set of detached glyphs.
	detachedChildren.removeAllElements();
	for (int i = 0; i < glyphs.size(); i++){
		((AreaGlyph)glyphs.elementAt(i))._addDetachedGlyphsTo(detachedChildren);
	}
}
final void _addLine(LineGlyph line) {
	if (!lines.contains(line)) {
		lines.addElement(line);
	}
}
/**
 * Invoked when the mouse has been clicked on a component.
 */
public final AreaGlyph _getGlyphFor(Node node) {
	AreaGlyph found = null;
	for (int i = 0; null == found && i < glyphs.size(); i++) {
		AreaGlyph glyph = (AreaGlyph) glyphs.elementAt(i);
		found = glyph.findGlyphFor(node);
	}
	return found;
}
final void _remove(AreaGlyph glyph) {
	glyphs.removeElement(glyph);
	repaint();
}
final void _removeLine(LineGlyph line) {
	lines.removeElement(line);
}
/**
 * Add a node to the receiver
 * will not add a node a second time
 */
public final void addNode(Node node, int xCoord, int yCoord) {
	if (null == _getGlyphFor(node)) {
		AreaGlyph glyph = node._getGlyph();
		//glyph.setCoords(xCoord, yCoord);
		glyph._setView(this);
		glyphs.addElement(glyph);
		repaint();
	}
}
final void clear() {
	lines.removeAllElements();
	glyphs.removeAllElements();
}
	/**
	 * Invoked when the component has been made invisible.
	 */
public void componentHidden(ComponentEvent e) {}
	/**
	 * Invoked when the component's position changes.
	 */
public void componentMoved(ComponentEvent e) {}
	/**
	 * Invoked when the component's size changes.
	 */
public void componentResized(ComponentEvent e) {
	buffer = null;
	//invalidate();
}
	/**
	 * Invoked when the component has been made visible.
	 */
public void componentShown(ComponentEvent e) {}
public final Frame getFrame() {
	return frame;
}
public final boolean getStringData(String title, String[][] data) {
	boolean done = false;
	if (null != frame) {
		GetStringsDialogue dlg = new GetStringsDialogue(frame, title, data);
		dlg.show();
		done = dlg.okPressed();
	}
	return done;
}
/**
 * Invoked when the mouse has been clicked on a component.
 */
public final boolean hasNode(Node node) {
	return (null != _getGlyphFor(node));
}
public final void info(String txt) {
	info.setText(txt);
}
/**
 * Returns true if this component is painted to an offscreen image
 * ("buffer") that's copied to the screen later.  Component
 * subclasses that support double buffering should override this
 * method to return true if double buffering is enabled.
 * 
 * @return false by default
 */
public boolean isDoubleBuffered() {
	return doubleBuffered;
}
/**
 * Invoked when the mouse has been clicked on a component.
 *
 * public for mouse listener interface, final so subclasses
 * cannot interfere
 */
public final void mouseClicked(MouseEvent e) {
	currentGlyph = null;
	previousPoint = null;
}
/**
 * Invoked when a mouse button is pressed on a component and then 
 * dragged.  Mouse drag events will continue to be delivered to
 * the component where the first originated until the mouse button is
 * released (regardless of whether the mouse position is within the
 * bounds of the component).
 *
 * public for mouse listener interface, final so subclasses
 * cannot interfere
 */
public final void mouseDragged(MouseEvent e) {
	//info("mouse dragged");
	boolean dragged = false;
	if (null != currentGlyph) {
		Point thisPoint = e.getPoint();
		if (!dragging) {
			dragging = true;
		}
		currentGlyph._drag(previousPoint, thisPoint);
		repaint();
		previousPoint = thisPoint;
	}
}
/**
 * Invoked when the mouse enters a component.
 *
 * public for mouse listener interface, final so subclasses
 * cannot interfere
 */
public final void mouseEntered(MouseEvent e) {
}
/**
 * Invoked when the mouse exits a component.
 *
 * public for mouse listener interface, final so subclasses
 * cannot interfere
 */
public final void mouseExited(MouseEvent e) {
}
/**
 * Invoked when the mouse button has been moved on a component
 * (with no buttons no down).
 *
 * public for mouse listener interface, final so subclasses
 * cannot interfere
 */
public final void mouseMoved(MouseEvent e) {
	AreaGlyph found = __getGlyphAt(e.getPoint());
	if (null == found) {
		info("");
	} else {
		info(found.node.getDescription());
	}
}
/**
 * Invoked when a mouse button has been pressed on a component.
 *
 * public for mouse listener interface, final so subclasses
 * cannot interfere
 */
public final void mousePressed(MouseEvent e) {
    //info("mouse pressed");
    currentGlyph = __getGlyphAt(e.getPoint());
    if (__isPopupTrigger(e)) {
        PopupMenu menu = null;
        Point thisPoint = e.getPoint();

        if (null != currentGlyph) {
            menu = currentGlyph.getMenu();
        }
        //Enumeration glyphEnum = getGlyphs();
        //while (null == menu && glyphEnum.hasMoreElements()) {
        //	Glyph glyph = (Glyph) glyphEnum.nextElement();
        //	menu = glyph.getMenu(thisPoint);
        //	//if (null != menu) {
        //	//currentGlyph = glyph;
        //	//}
        //}
        if (null != menu) {
            add(menu);
            menu.show(this, thisPoint.x, thisPoint.y);
        }
        currentGlyph = null;
        previousPoint = null;
    } else {
        if (__isLeftButton(e)) {
            previousPoint = e.getPoint();
            Node curNode = null;

            if (null == currentGlyph) {
                __deselectAll();
            } else {
                curNode = currentGlyph.getNode();
                if (currentGlyph.click(e.getPoint())) {
                } else if (!currentGlyph.isSelected()) {
                    __deselectAll();
                    currentGlyph.select();
                }
            }
            //info("left mouse: " + currentGlyph.getName());
            showHelp(curNode, false);
            repaint();
        }
    }

    dragging = false;
}
/**
 * Invoked when a mouse button has been released on a component.
 *
 * public for mouse listener interface, final so subclasses
 * cannot interfere
 */
public final void mouseReleased(MouseEvent e) {
	if (dragging && null != currentGlyph) {
		Point thisPoint = e.getPoint();
		currentGlyph._dragEnd(thisPoint);
		if (currentGlyph.isSelected()) {
			currentGlyph.deselect();
		}
		dragging = false;
		repaint();
	}
	//currentGlyph = null;
	previousPoint = null;
	dragging = false;
}
/**
 * Overide the paint method from component; final so subclasses
 * cannot interfere
 */
public final void paint(Graphics graphics) {
    Graphics useGraphics = graphics;
    int width = getSize().width;
    int height = getSize().height;
    if (doubleBuffered) {
        if (null == buffer) {
            buffer = createImage(width, height);
        }
        useGraphics = buffer.getGraphics();
    }
    __calculate(useGraphics);
    // since we need different sets of drawing primitives for the
    // AWT and for the Windows Metafile we need to abstract a common
    // functionality into a wrapper object.  The wrapper object is
    // a subclass of DrawContext
    useGraphics.clearRect(0, 0, width, height);
    useGraphics.setColor(Color.black);
    reallyPaint(new GraphicsContext(useGraphics));
    if (doubleBuffered) {
        graphics.drawImage(buffer, 0, 0, null);
    }
}
public final void printDiagram() {
	PrintJob pj = getToolkit().getPrintJob(frame, "Print Diagram", null);
	if (null != pj) {
		print(pj.getGraphics());
		pj.end();
	}
	
}
/**
 * Overide the paint method from component; final so subclasses
 * cannot interfere
 */
public final void reallyPaint(DrawContext graphics) {
    AreaGlyph selectedGlyph = null;
    AreaGlyph dropTargetGlyph = null;
    // draw all the glyphs EXCEPT the selected one (if any) and the 
    // drop target (if there is one), because these appear to 'hover'
    // above the rest.
    for (int i = 0; i < detachedChildren.size(); i++) {
        AreaGlyph glyph = (AreaGlyph) detachedChildren.elementAt(i);
        glyph._drawBoundary(graphics);
        if (glyph.isSelected()) {
            selectedGlyph = glyph;
        } else if (glyph.isDropTarget()) {
	        dropTargetGlyph = glyph;
        } else {
            glyph._draw(graphics);
        }
    }
    // draw all the lines
    for (int i = 0; i < lines.size(); i++) {
        LineGlyph line = (LineGlyph) lines.elementAt(i);
        if (!line.masked && line.isVisible()) {
            line._draw(graphics);
        }
    }

    // draw shadows for drop target and selected glyphs
    if (null != dropTargetGlyph) {
        dropTargetGlyph._drawShadow(graphics);
    }
    if (null != selectedGlyph) {
        selectedGlyph._drawShadow(graphics);
    }
    
    // draw drop target and selected glyphs
    if (null != dropTargetGlyph) {
        dropTargetGlyph._draw(graphics);
    }
    if (null != selectedGlyph) {
        selectedGlyph._draw(graphics);
    }
    
}
/* Write out a bitmap...
  */
public final void saveBitmap(String filename) {
	ImageStorer.saveBitmap(filename, buffer);
}
/* Write out a bitmap...
  */
public final void saveMetafile(String filename) {
	Dimension dim =  getSize();
	MetafileContext context = new MetafileContext(filename, 
									dim.width, dim.height,
									getFont());
	reallyPaint(context);
	context.dispose();
}
public void setFont(Font f) {
	super.setFont(f);
	__deselectAll();
	repaint();
}
public final void setFrame(Frame frame) {
	this.frame = frame;
}
/**
 * force will open window if not showing.  using false allows us
 * to change contents of window already showing.
 */
public final void showHelp(Node node, boolean force) {
    if (null != frame) {
        if (null == helpDlg) {
            helpDlg = new GlyphHelpDialogue(frame);
        }
        helpDlg.setNode(node);
        if (force) {
	        helpDlg.show();
        }
    }
}
/**
 * force will open window if not showing.  using false allows us
 * to change contents of window already showing.
 */
public final void showHelp(String str, String title, boolean force) {
    if (null != frame) {
        if (null == helpDlg) {
            helpDlg = new GlyphHelpDialogue(frame);
        }
        helpDlg.setText(str, title);
        if (force) {
	        helpDlg.show();
        }
    }
}
	/**
	 * Updates this component.
	 * <p>
	 * The AWT calls the <code>update</code> method in response to a
	 * call to <code>repaint</code. The appearance of the
	 * component on the screen has not changed since the last call to
	 * <code>update</code> or <code>paint</code>. You can assume that
	 * the background is not cleared.
	 * <p>
	 * The <code>update</code>method of <code>Component</code>
	 * does the following:
	 * <p>
	 * <blockquote><ul>
	 * <li>Clears this component by filling it
	 *      with the background color.
	 * <li>Sets the color of the graphics context to be
	 *     the foreground color of this component.
	 * <li>Calls this component's <code>paint</code>
	 *     method to completely redraw this component.
	 * </ul></blockquote>
	 * <p>
	 * The origin of the graphics context, its
	 * (<code>0</code>,&nbsp;<code>0</code>) coordinate point, is the
	 * top-left corner of this component. The clipping region of the
	 * graphics context is the bounding rectangle of this component.
	 * @param g the specified context to use for updating.
	 * @see       java.awt.Component#paint
	 * @see       java.awt.Component#repaint()
	 * @since     JDK1.0
	 */
	public void update(Graphics g) {
		paint(g);
	}
}
