package glyph.dialogues;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.awt.*;
import java.awt.event.*;
import java.util.*;
public class FormattingTextPanel extends Canvas implements ComponentListener {
	private String text;
	private boolean changed = true;
	private Vector lines = new Vector();
	
	private class TextLine {
		private int x; 
		private int y;
		private String string;
		TextLine(int x, int y, String string) {
			this.x = x;
			this.y = y;
			this.string = string;
		}
		final void draw(Graphics g) {
			g.drawString(string, x, y);
		}
	}
/**
 * FormattingTextPanel constructor comment.
 */
public FormattingTextPanel() {
	super();
	addComponentListener(this);
}
/**
 * FormattingTextPanel constructor comment.
 */
public FormattingTextPanel(String text) {
	this();
	setText(text);
}
	/**
	 * Invoked when the component has been made invisible.
	 */
public void componentHidden(ComponentEvent e) {}
	/**
	 * Invoked when the component's position changes.
	 */
public void componentMoved(ComponentEvent e) {}
	/**
	 * Invoked when the component's size changes.
	 */
public void componentResized(ComponentEvent e) {
	changed = true;
}
	/**
	 * Invoked when the component has been made visible.
	 */
public void componentShown(ComponentEvent e) {}
public void forceChange() {
	changed = true;
	repaint();
}
private final void newTextLine(int x, int y, String string) {
	lines.addElement(new TextLine(x,y,string));
}
/**
 * This method is called to repaint this canvas. Most applications 
 * that subclass <code>Canvas</code> should override this method in 
 * order to perform some useful operation. 
 * <p>
 * The <code>paint</code> method provided by <code>Canvas</code> 
 * redraws this canvas's rectangle in the background color. 
 * <p>
 * The graphics context's origin (0,&nbsp;0) is the top-left corner 
 * of this canvas. Its clipping region is the area of the context. 
 * @param      g   the graphics context.
 * @see        java.awt.Graphics
 */
public void paint(Graphics g) {
	super.paint(g);
	if (changed) {
		reformat(g);
	}
	// now print the lines
	g.setColor(Color.black);
	Enumeration enum = lines.elements();
	while (enum.hasMoreElements()) {
		((TextLine)enum.nextElement()).draw(g);
	}
}
/**
 */
private final void reformat(Graphics g) {
	FontMetrics fm = g.getFontMetrics();
	// useful info for paint routine
	int textHeight = fm.getHeight();
	int currentY = textHeight;
	// the width we are aiming at for the line is the width of the window
	// minus a margin either side.  Use a constant string to determine
	// the margin
	int margin = fm.stringWidth("W");
	int width = getSize().width - (/*2 **/ margin);

	// clear the buffer full of calculated lines
	lines.removeAllElements();
	// break text into tokens, and build lines that are no longer than
	// the width of the screen.  We will slowly build the line in a buffer
	StringBuffer buf = new StringBuffer();
	String delimiters = " \t\n\r\f";
	StringTokenizer tok = new StringTokenizer(text, delimiters, true);
	int currentLineLength = 0;
	int currentLineX = margin;
	int SPACEWIDTH = fm.stringWidth(" ");
	while (tok.hasMoreTokens()) {
		// pull off a token and see if we have enough space for it
		String word = tok.nextToken();
		if ("\t".equals(word)) {
			// tab
			// if we have a current text line, then finish it and start another
			// with the apprpriate currentLineX.  Do not increment Y value
			if (0 < currentLineLength) {
				newTextLine(currentLineX, currentY, buf.toString());
				buf.setLength(0);
			}
			currentLineX += fm.stringWidth("WW");
		} else if (" ".equals(word)) {
			// ignore whitespace
		} else if (delimiters.indexOf(word) > -1) {
			// new line
			newTextLine(currentLineX, currentY, buf.toString());
			buf.setLength(0);
			currentY += textHeight + (textHeight / 2);
			currentLineX = margin;
		} else {
			int wordLength = fm.stringWidth(word);
			if ((currentLineLength + SPACEWIDTH + wordLength + currentLineX) > width) {
				// line is too long for window if we add the word
				// so add the current line to the list of lines and
				// start a new one.
				newTextLine(currentLineX, currentY, buf.toString());
				buf.setLength(0);
				buf.append(word);
				currentY += textHeight;
				currentLineX = margin;
			} else {
				buf.append(' ');
				buf.append(word);
			}
		}
		currentLineLength = fm.stringWidth(buf.toString());
	}
	// remember to add the last line
	if (0 < currentLineLength) {
		newTextLine(currentLineX, currentY, buf.toString());
	}

	// now clear the change, as we have handled it
	changed = false;
}
/**
 * FormattingTextPanel constructor comment.
 */
public final void setText(String text) {
	this.text = text;
	changed = true;
	repaint();
}
}
