package glyph.xml;

/*------------------------------------------------------------------------------
 * Licensed Materials - Property of IBM                                        
 *  "Restricted Materials of IBM"                                               
 *  WebSphere MQ Everyplace for Multiplatforms: MQeDesigner                                                                    
 *                                                                              
 *  (C) Copyright IBM Corp. 2002  All Rights Reserved.                    
 *                                                                              
 *  US Government Users Restricted Rights - Use, duplication or                 
 *  disclosure restricted by GSA ADP Schedule Contract with                     
 *  IBM Corp.                                                                   
 *------------------------------------------------------------------------------*/

import java.io.*;
import java.util.*;

public final class Element extends AbstractElement implements XMLConstants {
	protected static final String TRUE = "true";
	protected static final String FALSE = "false";
	protected static final String NEWLINE = "\r\n";
	protected static final String TAB = "    ";

	private int startPos;
	private int endPos;		
		
	private boolean isEmptyTag = false;		// a tag with no children eg: <fred />
	private boolean isEndTag = false;		// a tag that starts with a forward slash </fred>
	private String nodeName;
	private Vector children;
	private Hashtable attributes;
	private AbstractElement parent;
	String value = "";
/**
 * ElementImpl constructor comment.
 */
public Element(AbstractElement parent) {
	super();
	attributes = new Hashtable();
	children = new Vector();
	this.parent = parent;
	if (null != parent) {
		parent.addChild(this);
	}
}
/**
 * ElementImpl constructor comment.
 */
public Element(AbstractElement parent, int count) {
	this(parent);
	startPos = count;
}
/**
 * ElementImpl constructor comment.
 */
public Element(AbstractElement parent, String nodeName) {
	this(parent);
	this.nodeName = nodeName;
}
private void _generateXML(Writer writer, int pretty) {
	try {
		// indent....
		for (int i = 0; i < pretty; i++) {
			writer.write(TAB);
		}
		writer.write(START_OF_TAG);
		writer.write(getName());

		// if there is a name attribute, make sure it comes first
		String key = "name";
		String nameValue = getAttribute(key);
		if (null == nameValue) {
			key = "Name";
			nameValue = getAttribute(key);
		}
		if (null != nameValue) {
			writer.write(SPACE);
			writer.write(key);
			writer.write(EQUALS);
			writer.write(QUOTE);
			writer.write(getAttribute(key));
			writer.write(QUOTE);
		}
		
		Enumeration enum = getAttributeNames();
		while (enum.hasMoreElements()) {
			key = enum.nextElement().toString();
			if ("name".equals(key) || "Name".equals(key)) {
			} else {
				writer.write(SPACE);
				writer.write(key);
				writer.write(EQUALS);
				writer.write(QUOTE);
				writer.write(getAttribute(key));
				writer.write(QUOTE);
			}
		}
		enum = getChildren();
		if (!enum.hasMoreElements()) {
			writer.write(FORWARD_SLASH);
			writer.write(END_OF_TAG);
		} else {
			writer.write(END_OF_TAG);
			int newPretty = (pretty < 0) ? pretty : pretty + 1;
			while (enum.hasMoreElements()) {
				writer.write(NEWLINE);
				Element tag = (Element)enum.nextElement();
				tag._generateXML(writer, newPretty);
			}

			// write an end tag
			writer.write(NEWLINE);
			for (int i = 0; i < pretty; i++) {
				writer.write(TAB);
			}
			writer.write(START_OF_TAG);
			writer.write(FORWARD_SLASH);
			writer.write(getName());
			writer.write(END_OF_TAG);
		}
		writer.flush();
	} catch (IOException e) {
		System.out.println(e);
		e.printStackTrace();
	}	
}
final boolean _isEmptyTag() {
	return isEmptyTag;
}
final boolean _isEndTag() {
	return isEndTag;
}
final boolean _isEndTagFor(Element tag) {
	return getName().equals(tag.getName());
}
void _setEndPos(int end) {
	endPos = end;
}
final void _setIsEmptyTag(boolean set) {
	isEmptyTag = set;
}
final void _setIsEndTag(boolean set) {
	isEndTag = set;
}
final void _setName(String str) {
	setNodeName(str);
}
public void addChild(Element tag) {
	if (!children.contains(tag)) {
		// only add child once
		appendChild(tag);
		isEmptyTag = false;
	}
}
/**
 * appendChild method comment.
 */
public Element appendChild(Element newChild) {
	children.addElement(newChild);
	newChild.setParentElement(this);
	return newChild;
}
public void generateXML(Writer writer, boolean pretty) {
	int i = pretty ? 0 : -1;
	_generateXML(writer, i);
}
public void getAbsoluteReference(StringBuffer buf, boolean longForm) {
	getParentElement().getAbsoluteReference(buf, longForm);
	buf.append("/");
	if (longForm) {
		buf.append("child::"); // child is default axis
	}
	buf.append(getName());
	// next print out the seklector by name. This will go TU if we dont
	// have one.
	String nameAttribute = getAttribute("name");
	if (null == nameAttribute) {
		nameAttribute = "";
	}
	buf.append("[");
	if (longForm) {
		buf.append("attribute::");
	} else {
		buf.append("@");	// short for attribute
	}
	buf.append("name=\'");
	buf.append(nameAttribute);
	buf.append("]");
}
/**
 * getAttribute method comment.
 */
public String getAttribute(String name) {
	return (String)attributes.get(name);
}
public final boolean getAttributeBoolean(String attrName) {
	boolean bVal = false;
	String sVal = getAttributeValue(attrName);
	if (null != sVal) {
		bVal = sVal.equals(TRUE);
	}
	return bVal;
}
public final int getAttributeInteger(String attrName) {
	int iVal = 0;
	String sVal = getAttributeValue(attrName);
	if (null != sVal) {
		iVal = Integer.parseInt(sVal);
	}
	return iVal;
}
/**
 * getAttribute method comment.
 */
public final Enumeration getAttributeNames() {
	return attributes.keys();
}
public final String getAttributeValue(String attrName) {
	return getAttribute(attrName);
}
/**
 * getChildNodes method comment.
 */
public Enumeration getChildren() {
	return children.elements();
}
public Document getDocument() {
	Document doc = null;
	if (null != parent) {
		doc = parent.getDocument();
	}
	return doc;
}
public final String getName() {
	return getNodeName();
}
/**
 * getNodeName method comment.
 */
public String getNodeName() {
	return nodeName;
}
public final AbstractElement getParentElement() {
	return parent;
}
public final String getValue() {
	return value;
}
public final boolean isElement() {
	return true;
}
public final void prettyPrint(int indent) {
	for (int i = 0; i < indent; i++) {
		System.out.print("\t");
	}
	System.out.println(toString());
	Enumeration enum = getChildren();
	while (enum.hasMoreElements()) {
		((Element)enum.nextElement()).prettyPrint(indent + 1);
	}
}
/**
 * removeAttribute method comment.
 */
public void removeAttribute(String name) {
	//attributes.remove(name);
}
public final void setAttribute(String attrName, int value) {
	setAttribute(attrName, Integer.toString(value));
}
/**
 * setAttribute method comment.
 */
public void setAttribute(String name, String value) {
	if (null != name && null != value) {
		attributes.put(name, value);
	}
}
public final void setAttribute(String attrName, boolean value) {
	if (value) {
		setAttribute(attrName, TRUE);
	} else {
		setAttribute(attrName, FALSE);
	}
}
/**
 * setNodeValue method comment.
 */
public void setNodeName(String name) {
	nodeName = name;
}
public final void setParentElement(AbstractElement parent) {
	this.parent = parent;
}
public String toString() {
	return Unparser.UnparseToString(this, true);
}
}
